#!/usr/bin/env python
#-*- coding: UTF-8 -*-

import wx
from wx.lib.mixins.listctrl import CheckListCtrlMixin
from   .mumamisc import *


class CheckListCtrl(wx.ListCtrl, CheckListCtrlMixin):
    def __init__(self, parent, config, sbox):
        style = wx.LC_REPORT | wx.BORDER_SUNKEN
        self.sbox = sbox
        if config ['ListCtrl']['grid'] == 'True':
            style |= wx.LC_HRULES | wx.LC_VRULES
        wx.ListCtrl.__init__(self, parent, -1, style=style)
        CheckListCtrlMixin.__init__(self)
        self.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.OnItemActivated)
        self.checkcounter = 0
    #-------------------------------------------------------------------------------------------

    def zeroCandidates(self):
        self.checkcounter = 0
        self.sbox.SetLabel(' Replace candidates [%d|%d] ' % (self.checkcounter,
                                                             self.GetItemCount()))
    #-------------------------------------------------------------------------------------------

    def OnItemActivated(self, evt):
        self.ToggleItem(evt.m_itemIndex)
    #-------------------------------------------------------------------------------------------

    # this is called by the base class when an item is checked/unchecked
    def OnCheckItem(self, index, flag):
        if flag:
            self.checkcounter += 1
        else:
            self.checkcounter -= 1
        self.sbox.SetLabel(' Replace candidates [%d|%d] ' % (self.checkcounter,
                                                             self.GetItemCount()))
      #  data = self.GetItemData(index)
# - class CheckListCtrl(wx.ListCtrl):-----------------------------------------------------------


class BatchEditDialog(wx.Dialog):
    ID_TC_SEARCH = wx.NewId()
    ID_TC_REPLACE = wx.NewId()    
    
    def __init__(self, parent, config, mumadb, writeToLogwin, onQuery):
        windowpos = (wx.Point(int(config['Window']['beposx']),
                              int(config['Window']['beposy'])))
        wx.Dialog.__init__(self, parent, -1, TITLE + ' - Batch edit', pos = windowpos)
                          # style = wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER)

        self.Bind(wx.EVT_CLOSE, self.onCloseWindow)
        self.config = config
        self.mumadb = mumadb
        self.writeToLogwin = writeToLogwin
        self.onQuery = onQuery
        btnadj = int(config['Misc']['norm'] == 'False')
        collist = config['Misc']['hlcolor']
        self.hlcolor = (int(collist[0]), int(collist[1]), int(collist[2]))
        collist = config['Misc']['altcolor']
        self.altcolor = (int(collist[0]), int(collist[1]), int(collist[2]))

        self.matchcount = 0
         
        sizer = wx.BoxSizer(wx.VERTICAL)
        sb = wx.StaticBox(self, -1, ' Find and replace ')
        sbsz1 = wx.StaticBoxSizer(sb, wx.VERTICAL)
        gbsz = wx.GridBagSizer(BORDER, BORDER)
        st = wx.StaticText(self, -1, 'Search for:')
        gbsz.Add(st, wx.GBPosition(0, 0), wx.GBSpan(1, 1), LRT | wx.GROW, GAP) 
        self.tcsearch = wx.TextCtrl(self, self.ID_TC_SEARCH, config['Misc']['besearch'], style = wx.TE_PROCESS_ENTER)
        gbsz.Add(self.tcsearch, wx.GBPosition(0, 1), wx.GBSpan(1, 2), LRT | wx.GROW, GAP) 
        self.Bind(wx.EVT_TEXT_ENTER, self.onFind, id = self.ID_TC_SEARCH)        
        self.hits = wx.StaticText(self, -1, '')
        gbsz.Add(self.hits, wx.GBPosition(0, 3), wx.GBSpan(1, 1), wx.ALL | wx.GROW, GAP) 
        
        st = wx.StaticText(self, -1, 'In column:  ')
        gbsz.Add(st, wx.GBPosition(1, 0), wx.GBSpan(1, 1), LRT | wx.GROW, GAP) 
        collist = ['Title', 'Interpreter', 'Code', 'Composer']
        self.co_column = wx.ComboBox(self, -1, config['Misc']['becolindex'], 
                                     choices = collist, style = wx.CB_DROPDOWN | wx.CB_READONLY)
        gbsz.Add(self.co_column, wx.GBPosition(1, 1), wx.GBSpan(1, 2), LRT | wx.GROW, GAP) 
        self.Bind(wx.EVT_COMBOBOX, self.onColumnCombo)        
        self.btnsearch = wx.Button(self, wx.ID_FIND)
        self.Bind(wx.EVT_BUTTON, self.onFind, id = wx.ID_FIND)
        gbsz.Add(self.btnsearch, wx.GBPosition(1, 3), wx.GBSpan(1, 1), LRT | wx.GROW, GAP) 
      
        st = wx.StaticText(self, -1, 'Replace with:')
        gbsz.Add(st, wx.GBPosition(2, 0), wx.GBSpan(1, 1), LRT | wx.GROW, GAP) 
        self.tcreplace = wx.TextCtrl(self, self.ID_TC_REPLACE, config['Misc']['bereplace'], style = wx.TE_PROCESS_ENTER)
        gbsz.Add(self.tcreplace, wx.GBPosition(2, 1), wx.GBSpan(1, 2), LRT | wx.GROW, GAP)

        st = wx.StaticText(self, -1, 'Folder range:')
        gbsz.Add(st, wx.GBPosition(3, 0), wx.GBSpan(1, 1), LRT | wx.GROW, GAP)        
        self.scmd1 = wx.SpinCtrl(self, -1)
        self.scmd1.SetRange(1000, 8999)           
        self.scmd1.SetValue(int(self.config['Misc']['berlow']))
        self.scmd1.SetToolTip(wx.ToolTip('>= 1000'))
        self.Bind(wx.EVT_SPINCTRL, self.onLow, self.scmd1)
        gbsz.Add(self.scmd1, wx.GBPosition(3, 1), wx.GBSpan(1, 1), LRT | wx.GROW, GAP) 
        st = wx.StaticText(self, -1, ' to (including) ')
        gbsz.Add(st, wx.GBPosition(3, 2), wx.GBSpan(1, 1), LRT | wx.GROW, GAP) 
        self.scmd2 = wx.SpinCtrl(self, -1)
        self.scmd2.SetRange(1000, 8999)           
        self.scmd2.SetValue(int(self.config['Misc']['berhigh']))
        self.scmd2.SetToolTip(wx.ToolTip('<= 8999'))
        self.Bind(wx.EVT_SPINCTRL, self.onHigh, self.scmd2)
        gbsz.Add(self.scmd2, wx.GBPosition(3, 3), wx.GBSpan(1, 1), LRT | wx.GROW, GAP) 

        st = wx.StaticText(self, -1, 'Check / uncheck:')
        gbsz.Add(st, wx.GBPosition(4, 0), wx.GBSpan(1, 1), LRT | wx.GROW, GAP) 
        self.cb = wx.CheckBox(self, -1, ' (if checked - mark all candidates)')
        gbsz.Add(self.cb, wx.GBPosition(4, 1), wx.GBSpan(1, 3), LRT | wx.GROW, GAP)
        self.Bind(wx.EVT_CHECKBOX, self.onCheck, self.cb)
        self.cb.SetValue(True)
   
        sbsz1.Add(gbsz, 0, wx.GROW | wx.ALL, GAP)        
        
        sizer.Add(sbsz1, 1, wx.GROW | LRT, BORDER)       
        sb = wx.StaticBox(self, -1, ' Replace candidates [0|0] ')
        sbsz2 = wx.StaticBoxSizer(sb, wx.VERTICAL)       
        self.candidates = CheckListCtrl(self, self.config, sb)
        self.candidates.InsertColumn(0, 'Id_Title')
        self.candidates.InsertColumn(1, 'Title')
        sbsz2.Add(self.candidates, 1, wx.GROW | wx.ALL, GAP)
        sizer.Add(sbsz2, 1, wx.GROW | LRT, BORDER)

        butrow = wx.BoxSizer(wx.HORIZONTAL)
        if not btnadj: butrow.AddStretchSpacer()
        butreplace = wx.Button(self, wx.ID_REPLACE)
        self.Bind(wx.EVT_BUTTON, self.onReplace, id = wx.ID_REPLACE)
        butrow.Add(butreplace, btnadj, wx.GROW | wx.ALL, GAP)
        butcancel = wx.Button(self, wx.ID_CANCEL)
        butrow.Add(butcancel, btnadj, wx.GROW | wx.ALL, GAP)
        sizer.Add(butrow, 0, wx.GROW | wx.ALL, GAP)
    
        self.SetSizer(sizer)
        self.Fit()
        self.hits.SetLabel('M: 0 [%s]' % (self.mumadb.getRecordCount('muma_title')))
    #-------------------------------------------------------------------------------------------

    def onCheck(self, event):
        status = self.cb.GetValue()
        for index in range(self.matchcount):
            self.candidates.CheckItem(index, status)
        self.Refresh()
    #-------------------------------------------------------------------------------------------

    def onHigh(self, event):
        value = event.GetInt()
        self.scmd1.SetRange(1000, value)
        self.scmd2.SetValue(value)   
    #-------------------------------------------------------------------------------------------

    def onLow(self, event):
        value = event.GetInt()
        self.scmd2.SetRange(value, 9900)
        self.scmd1.SetValue(value)
    #-------------------------------------------------------------------------------------------

    def onColumnCombo(self, event):
        self.candidates.DeleteAllItems()
        self.candidates.DeleteColumn(1)
        self.candidates.InsertColumn(1, self.co_column.GetValue())
        self.hits.SetLabel('M: 0 [%s]' % (self.mumadb.getRecordCount('muma_title')))
    #-------------------------------------------------------------------------------------------
        
    def onFind(self, event = None):
        columnstr = self.co_column.GetValue()
        self.candidates.DeleteAllItems()
        self.candidates.zeroCandidates()
        self.matchcount = 0
        status = self.cb.GetValue()
        lower = self.scmd1.GetValue()
        upper = self.scmd2.GetValue()
        total = self.mumadb.getRecordCount('muma_title')
        if self.tcsearch.GetValue() != '':
            query = 'SELECT count(*) FROM muma_title WHERE  %s LIKE "%%%s%%"' % \
                                          (columnstr, self.tcsearch.GetValue())
            records, self.matchcount = self.mumadb.getMatches(query)
            total = records[0][0]          
            query = 'SELECT Id_Title, %s FROM muma_title WHERE Id_Title >= "%i00" AND Id_Title <= "%i99" AND %s LIKE "%%%s%%" ORDER BY %s LIMIT 250' % \
                    (columnstr, lower, upper, columnstr, self.tcsearch.GetValue(), columnstr)
            records, self.matchcount = self.mumadb.getMatches(query)
            self.candidates.DeleteAllItems()
            if self.matchcount:
                index = 0
                for entrytuple in records:
                    str_id_title = str(entrytuple[0])
                    if str_id_title.startswith('99'):
                        self.matchcount -= 1
                        total -= 1
                    else:
                        item = self.candidates.InsertStringItem(index, str_id_title)
                        if index % 2 == 0:
                            self.candidates.SetItemBackgroundColour(item, self.altcolor)
                        if isSpecialId(str_id_title):
                             self.candidates.SetItemBackgroundColour(item, self.hlcolor)
                        self.candidates.SetStringItem(index, 1, entrytuple[1])
                        self.candidates.CheckItem(index, status)
                        index += 1
            else:
                self.candidates.InsertStringItem(0, '>>>')
                self.candidates.SetStringItem(0, 1, 'no matching record found')
                self.candidates.SetItemTextColour(0, BLUE)
    
            self.candidates.SetColumnWidth(0, wx.LIST_AUTOSIZE)
            if not LINUX: self.candidates.SetColumnWidth(0, self.candidates.GetColumnWidth(0) + 5)
            rect = self.candidates.GetRect()
            self.candidates.SetColumnWidth(1, rect[2] - 50 - self.candidates.GetColumnWidth(0))
            #rect = self.GetRect()
            #sizex = max(rect[2] / 2 - self.candidates.GetColumnWidth(0) - 50, self.candidates.GetColumnWidth(1))
            #self.candidates.SetColumnWidth(1, sizex)
        
        self.hits.SetLabel('M: %i [%i]' % (self.matchcount, total))
        self.Refresh()
    #-------------------------------------------------------------------------------------------

    def onCancel(self, event = None):
        self.windowPosition()
        self.EndModal(wx.ID_CANCEL)
    #-------------------------------------------------------------------------------------------

    def onReplace(self, event):
        repstr = self.tcreplace.GetValue()
        if repstr == '' or self.matchcount == 0:
            dlg = wx.MessageDialog(self, "Ooooops - here is nothing to replace!",
                                       TITLE + ' - Batch edit', wx.OK | wx.ICON_INFORMATION)
            dlg.ShowModal()
            dlg.Destroy()
            return

        checkcount = 0
        srcstr = self.tcsearch.GetValue()
        columnstr = self.co_column.GetValue()
        for index in range(self.candidates.GetItemCount()):
            if self.candidates.IsChecked(index):
                checkcount += 1
                itemstr = self.candidates.GetItem(index, 1).GetText()
                #newitemstr = itemstr.replace(srcstr, repstr)  # case sensitive replace
                # the next 2 line make it case insensitive
                index_l = itemstr.lower().index(srcstr.lower())
                newitemstr = itemstr[:index_l] + repstr + itemstr[index_l + len(srcstr):] 
                checkcount += 1
                query = 'UPDATE muma_title SET %s = "%s" WHERE Id_Title = %s' % (columnstr, newitemstr, self.candidates.GetItem(index, 0).GetText())
                self.mumadb.execQuery(query)

        self.writeToLogwin("Batch edit - %i record(s) modified" % (checkcount))
        self.onQuery(enterparam = False)                
        self.windowPosition()
        self.EndModal(wx.ID_OK)
    #-------------------------------------------------------------------------------------------

    def windowPosition(self):
        windowrect = self.GetRect()
        self.config['Window']['beposx'] = windowrect[0]
        self.config['Window']['beposy'] = windowrect[1]
        self.config['Misc']['besearch'] = self.tcsearch.GetValue()
        self.config['Misc']['becolindex'] = self.co_column.GetValue()
        self.config['Misc']['bereplace'] = self.tcreplace.GetValue()
        self.config['Misc']['berlow'] = str(self.scmd1.GetValue())
        self.config['Misc']['berhigh'] = str(self.scmd2.GetValue())
    #-------------------------------------------------------------------------------------------
        
    def onCloseWindow(self, event = None):
        self.windowPosition()
        self.Destroy()
# - class BatchEditDialog(wx.Dialog):-----------------------------------------------------------
    
