#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import wx
from .mumamisc import *


class CheckTitle(wx.Dialog):
    
    def __init__(self, parent, id_title, config, mumadb, onCheckMusicDir, onPlayTitle, writeToLogwin, plogger):
        windowpos = (wx.Point(int(config['Window']['cptposx']),
                              int(config['Window']['cptposy'])))
        windowsize = (wx.Size(int(config['Window']['cptsizex']),
                              int(config['Window']['cptsizey'])))        
        wx.Dialog.__init__(self, parent, -1, TITLE + ' - Check / Paste [/ Remove] title',
                           pos = windowpos, size = windowsize,
                           style = wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER)
        self.Bind(wx.EVT_CLOSE, self.onButtonCancel)
        self.parent = parent
        self.config = config
        btnadj = int(config['Misc']['norm'] == 'False')
        self.mumadb = mumadb
        self.onCheckMusicDir = onCheckMusicDir
        self.onPlayTitle = onPlayTitle
        self.writeToLogwin = writeToLogwin
        self.logger = plogger
        self.id_title = id_title
        self.selindex = -1
        self.sourceindex = -1
        self.targetindex = -1
        collist = config['Misc']['altcolor']
        self.altcolor = (int(collist[0]), int(collist[1]), int(collist[2]))

        sizer = wx.BoxSizer(wx.VERTICAL)
        sb = wx.StaticBox(self, -1, ' Select source (green) and target (red) title for pasting ')
        row1 = wx.StaticBoxSizer(sb, wx.HORIZONTAL)
        self.lic = wx.ListCtrl(self, -1, style = wx.LC_REPORT | \
                               wx.LC_SINGLE_SEL | wx.LC_HRULES | wx.BORDER_SUNKEN)
        imagelist = wx.ImageList(16, 16)
        #imagelist.Add(wx.ArtProvider.GetBitmap(wx.ART_NORMAL_FILE, wx.ART_OTHER, wx.Size(16, 16)))
        imagelist.Add(loadIcon('audiofile.png'))
        self.lic.AssignImageList(imagelist, wx.IMAGE_LIST_SMALL)

        self.Bind(wx.EVT_LIST_ITEM_SELECTED, self.onSelected)
        self.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.onPlayItemClicked, self.lic)
        self.Bind(wx.EVT_LIST_ITEM_RIGHT_CLICK, self.onShowContext, self.lic)
        self.lic.InsertColumn(0, 'Id_Title')
        self.lic.InsertColumn(1, 'Title')
        self.lic.InsertColumn(2, 'Interpreter')     
        row1.Add(self.lic, 1, wx.ALL | wx.GROW, GAP)
        
        sb = wx.StaticBox(self, -1, ' Remove target (red) from database / device ')
        row2 = wx.StaticBoxSizer(sb, wx.VERTICAL)
        self.cbre = wx.CheckBox(self, -1, ' (if checked - the target title will be removed from db)') 
        row2.Add(self.cbre, 0, wx.ALL, GAP)
        self.cbre.Enable(False)
        self.Bind(wx.EVT_CHECKBOX, self.onCheckRemove, self.cbre)
        self.cbmf = wx.CheckBox(self, -1, ' (if checked - the obsolete associated file will be deleted too)') 
        row2.Add(self.cbmf, 0, wx.ALL, GAP)
        self.cbmf.Enable(False)        

        self.butpaste = wx.Button(self, wx.ID_PASTE)
        butcancel = wx.Button(self, wx.ID_CANCEL)
        self.Bind(wx.EVT_BUTTON, self.onButtonCancel, id = wx.ID_CANCEL)   
        self.Bind(wx.EVT_BUTTON, self.onPaste, id = wx.ID_PASTE)
                
        row3 = wx.BoxSizer(wx.HORIZONTAL)
        if not btnadj: row3.AddStretchSpacer()
        row3.Add(self.butpaste, btnadj, wx.GROW | wx.RIGHT, GAP)
        row3.Add(butcancel, btnadj, wx.GROW | wx.LEFT, GAP)

        sizer.Add(row1, 1, wx.GROW | LRT, BORDER)
        sizer.Add(row2, 0, wx.GROW | LRT, BORDER)
        sizer.Add(row3, 0, wx.GROW | wx.ALL, BORDER)
        self.SetSizer(sizer)
        self.SetMinSize(wx.Size(480, 380))
        self.initContextMenu()
        self.findAlternatives()
        self.updateList(id_title)
    #-------------------------------------------------------------------------------------------

    def enableCheckboxes(self):
        self.selectstate = self.sourceindex >= 0 and self.targetindex >= 0
        self.cbre.Enable(self.selectstate)
        self.cbmf.Enable(self.selectstate and self.onCheckMusicDir() and self.cbre.GetValue())
    #-------------------------------------------------------------------------------------------

    def onCheckRemove(self, event):
        self.cbmf.Enable(self.onCheckMusicDir() and self.cbre.GetValue())
    #-------------------------------------------------------------------------------------------

    def onSelected(self, event):
        self.selindex = event.GetIndex()   
    #-------------------------------------------------------------------------------------------

    def initContextMenu(self):
        self.playpopupID0 = wx.NewId()  
        self.playpopupID1 = wx.NewId()          
        self.playpopupID2 = wx.NewId()
        self.Bind(wx.EVT_MENU, self.onSelectSource, id = self.playpopupID0)
        self.Bind(wx.EVT_MENU, self.onSelectTarget, id = self.playpopupID1)
        self.Bind(wx.EVT_MENU, self.onPaste, id = self.playpopupID2)
    #-------------------------------------------------------------------------------------------

    def onShowContext(self, event):
        """ context menu handler """
        menu = wx.Menu()       
        item = wx.MenuItem(menu, self.playpopupID0, 'Select paste source')
        item.SetBitmap(loadIcon('addlist.png')) 
        menu.AppendItem(item)       
        item = wx.MenuItem(menu, self.playpopupID1, 'Select paste target')
        item.SetBitmap(loadIcon('dellist.png')) 
        menu.AppendItem(item)
        menu.AppendSeparator() 
        item = wx.MenuItem(menu, self.playpopupID2, 'Paste source to target')
        item.SetBitmap(loadIcon('copy.png')) 
        menu.AppendItem(item)
        self.PopupMenu(menu)
        menu.Destroy()
    #-------------------------------------------------------------------------------------------      

    def onSelectSource(self, event):
        if len(self.matches) > 1:
            self.sourceindex = self.selindex
            if self.targetindex == self.sourceindex:
                self.targetindex = -1
            self.updateList()
    #-------------------------------------------------------------------------------------------      
    
    def onSelectTarget(self, event):
        if len(self.matches) > 1:        
            self.targetindex = self.selindex
            if self.targetindex == self.sourceindex:
                self.sourceindex = -1        
            self.updateList()
    #-------------------------------------------------------------------------------------------      

    def onPlayItemClicked(self, event):
        if not self.onCheckMusicDir():
            dlg = wx.MessageDialog(self, "The music device is not mounted / reachable!", TITLE + ' - Play title', wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
        
        index = event.GetIndex() 
        id_title = self.lic.GetItem(index, 0).GetText()
        if id_title != '>>>':
            self.onPlayTitle(id = id_title, top25 = False)
    #-------------------------------------------------------------------------------------------

    def onButtonCancel(self, event):
        self.windowPosition()
        self.EndModal(wx.ID_CANCEL)
    #-------------------------------------------------------------------------------------------
        
    def onPaste(self, event):
        if self.sourceindex < 0 or self.targetindex < 0:
            dlg = wx.MessageDialog(self, "Select a source and a target first!", TITLE + ' - Paste title', wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()
            return
        
        pssource = self.lic.GetItem(self.sourceindex).GetText()
        pstarget = self.lic.GetItem(self.targetindex).GetText()
        self.mumadb.patchTables(pstarget, pssource, self.cbre.GetValue())
        if self.cbre.GetValue():
            self.writeToLogwin("Patch db - entry '%s' has been removed from db" % (pstarget))
            self.logger.info("entry '%s' has been removed from db" % (pstarget))
        self.writeToLogwin("Patch db - entry '%s' patched with '%s'" % (pstarget, pssource))
        self.logger.info("entry '%s' patched with '%s'" % (pstarget, pssource))
        if self.onCheckMusicDir() and self.cbmf.IsEnabled() and self.cbmf.IsChecked():
            found, pathname = findTitleExt(self.config, pstarget)
            try:
                os.remove(pathname)
                self.writeToLogwin("Patch - obsolet file '%s' removed" % (pathname))
            except:
                self.writeToLogwin("Patch - could not remove obsolet file '%s'" % (pathname))
        
        self.windowPosition()
        self.EndModal(wx.ID_PASTE)
    #-------------------------------------------------------------------------------------------

    def windowPosition(self):
        windowrect = self.GetRect()
        self.config['Window']['cptposx'] = windowrect[0]
        self.config['Window']['cptposy'] = windowrect[1]
        self.config['Window']['cptsizex'] = windowrect[2]
        self.config['Window']['cptsizey'] = windowrect[3]
    #-------------------------------------------------------------------------------------------

    def findAlternatives(self):
        sql = 'SELECT Id_Title, Title, Interpreter FROM muma_title WHERE Id_Title = "%s"' % (self.id_title)
        matches, count = self.mumadb.getMatches(sql)
        match = matches[0]
        title = match[1]
        interpreter = match[2]
        sql = 'SELECT Id_Title, Title, Interpreter FROM muma_title WHERE Title LIKE "%%%s%%" AND Interpreter LIKE "%%%s%%"' % (title, interpreter)
        matches, count = self.mumadb.getMatches(sql)
        self.matches = []
        for match in matches:
            if match[0] % 100 != 0:
                self.matches.append(match)
    #-------------------------------------------------------------------------------------------
                
    def updateList(self, id_title = 0):
        self.enableCheckboxes()
        self.lic.DeleteAllItems()

        for match in self.matches:
            strid_title = str(match[0])
            index = self.lic.InsertImageStringItem(sys.maxsize, strid_title, 0)
            self.lic.SetStringItem(index, 1, match[1])
            self.lic.SetStringItem(index, 2, match[2])           
            if strid_title == id_title:
                self.lic.SetItemState(index, wx.LIST_STATE_SELECTED | wx.LIST_STATE_FOCUSED, wx.LIST_STATE_SELECTED | wx.LIST_STATE_FOCUSED)
            if index % 2 == 0:
                self.lic.SetItemBackgroundColour(index, self.altcolor)

        if self.sourceindex >= 0:
            self.lic.SetItemTextColour(self.sourceindex, GREEN)
        if self.targetindex >= 0:
            self.lic.SetItemTextColour(self.targetindex, RED)

        if len(self.matches) > 1:
            self.butpaste.Enable(self.selectstate)
        else:
            self.butpaste.Enable(False)
            self.lic.InsertStringItem(1, '>>>')
            self.lic.SetStringItem(1, 1, 'Nothing to do')
            self.lic.SetStringItem(1, 2, '- no alternative title found')
            self.lic.SetItemTextColour(1, BLUE)
                     
        self.lic.SetColumnWidth(0, wx.LIST_AUTOSIZE)
        if not LINUX: self.lic.SetColumnWidth(0, self.lic.GetColumnWidth(0) + 5)
        self.lic.SetColumnWidth(1, wx.LIST_AUTOSIZE)
        rect = self.GetRect()
        self.lic.SetColumnWidth(2, rect[2] - 60 - self.lic.GetColumnWidth(0) - self.lic.GetColumnWidth(1))
# - class CheckPlaylist(wx.Dialog):--------------------------------------------------------------

