#!/usr/bin/env python
# -*- coding: UTF-8 -*-

import sys
import subprocess
import stat
import os
import os.path
import datetime
import codecs
import cPickle as pickle
import shutil
import threading
import wx
import wx.html
import logging
import logging.handlers
import webbrowser
import tarfile
from   time import strftime, sleep
from   distutils.file_util import copy_file
from   .mumamisc import *
if not LINUX: import ctypes
    
try:
    from agw import aui
except ImportError: # if it's not there locally, try the wxPython lib.
    import wx.lib.agw.aui as aui

try:
    import wx.lib.evtmgr as evtmgr
    import wx.lib.intctrl
except:    
    class PanicApp(wx.App):
        def OnInit(self):
            print('\n< < <  P A N I C  > > >\n')
            print("Can not start MusicMan because the lib 'intctrl' is missing!")
            print('Check your installation. The system reports, that you are', )
            print(WXVERSION, '- but it has to be 2.8 or better!')
            print()
            print('Please install wxPython 2.8 (package python-wxgtk2.8 on')
            print('Ubuntu / Kubuntu).')
            return True
            
    app = PanicApp(redirect = True)
    app.MainLoop()
   
try:
    import psyco
except:
    pass

from . import mumavlist
from . import mumacustbar
from . import mumasystray
from . import mumaabout
from . import mumaprefs
from . import mumaqueryman
from . import mumaedit
from . import mumaparam
from . import mumasqlite
from . import mumapopup
from . import mumatreectrl
from . import mumaimport
from . import mumareplace
from . import mumatextentry
from . import mumasync
from . import mumadir
from . import mumainstall
from . import mumadelrec
from . import mumachecklist
from . import mumachecktitle
from . import mumabatchedit
from . import mumaupdate
from . import mumamove

from .configobj import ConfigObj
from .singleinstance import SingleInstance
try:
    from .cherrytemplate import renderTemplate
    cherryisthere = True
except:
    cherryisthere = False
    
try:
    import notify2 as pynotify
    pynotify.init(TITLE)
    notifyisthere = True
except:
    try:
        import pynotify
        pynotify.init(TITLE)
        notifyisthere = True
    except:
        notifyisthere = False  

try:
    import pyudev
    pyudevisthere = True
except:
    pyudevisthere = False

class MumaTextDropTarget(wx.TextDropTarget):
    def __init__(self, playlistwin, config, playlists, updatePlaylistWin):
        wx.TextDropTarget.__init__(self)
        self.playlistwin = playlistwin
        self.config = config
        self.playlists = playlists
        self.updatePlaylistWin = updatePlaylistWin
    #-------------------------------------------------------------------------------------------
        
    def OnDropText(self, x, y, data):
        selplname = self.config['View']['selplname']
        title = data[:-8]
        id_title = data[-6:]
        if id_title.startswith('99'): return
        if id_title in self.playlists[selplname]:
            dlg = wx.MessageDialog(self.playlistwin, "The selected title(s) is/are already in the playlist!\nAdd the title anyhow?", TITLE + ' - add to playlist', wx.YES | wx.NO | wx.NO_DEFAULT | wx.ICON_WARNING)
            if dlg.ShowModal() != wx.ID_YES: return                   
        self.playlists[selplname].append(title)
        self.playlists[selplname].append(id_title)
        itemid = self.updatePlaylistWin('%s, %s' % (title, id_title), selplname)
        self.playlistwin.SelectItem(itemid)            
# - class MumaTextDropTarget(wx.TextDropTarget):-------------------------------------------------
  
class TabData(object):
    # Query Notebook Tab related data
    def __init__(self, querywin, querydata, orderby, orderdir, curquery, matchcount):
        self.querywin = querywin
        self.querydata = querydata
        self.orderby = orderby
        self.orderdir = orderdir
        self.curquery = curquery
        self.matchcount = matchcount
# - class TabData(object):------------------------------------------------------------------------

           
class MusicManFrame(wx.Frame):   
    ID_ADDRECORD = wx.NewId()
    ID_EDITRECORD = wx.NewId()
    ID_REMOVERECORD = wx.NewId()
    ID_BATCHEDIT = wx.NewId()
    ID_QUERYWIN = wx.NewId()
    ID_QUERYTAB = wx.NewId()
    ID_REFRESH_QUERY = wx.NewId()
    ID_LOGWIN = wx.NewId()
    ID_HISTWIN = wx.NewId()
    ID_WINDOW_TLC = wx.NewId()
    ID_INI_PERSP = wx.NewId()
    ID_EXAMPLE = wx.NewId()
    ID_QUERY = wx.NewId()
    ID_QUERY_TOI = wx.NewId()
    ID_QUERYHIST = wx.NewId()
    ID_CURRENT = wx.NewId()
    ID_BACKONE = wx.NewId()
    ID_BACKTWO = wx.NewId()
    ID_QUICK = wx.NewId()
    ID_QUICK_TAB = wx.NewId()
    ID_OPENVOL = wx.NewId()
    ID_MANAGE_QUERIES = wx.NewId()
    ID_OPEN_DB = wx.NewId()
    ID_BACKUP_DB = wx.NewId()
    ID_RESTORE_DB = wx.NewId()
    ID_IMPORT_DB = wx.NewId()
    ID_COMPACT_DB = wx.NewId()
    ID_PLAY = wx.NewId()
    ID_QUEUE = wx.NewId()
    ID_PLAYLIST = wx.NewId()
    ID_QUEUELIST = wx.NewId()
    ID_ADDPLAY = wx.NewId()
    ID_NEWPLAY = wx.NewId()
    ID_SAVEPLAY = wx.NewId()
    ID_EXPORTPLAY = wx.NewId()
    ID_IMPORTPLAY = wx.NewId()    
    ID_REMOVEPLAY = wx.NewId()
    ID_REMOVEITEM = wx.NewId()
    ID_MOVEUP = wx.NewId()
    ID_MOVEDOWN = wx.NewId()
    ID_CHECKLIST = wx.NewId()
    ID_PLAYWIN = wx.NewId()
    ID_EXPORT = wx.NewId()
    ID_COPY = wx.NewId()
    ID_IMPORT = wx.NewId()
    ID_REPLACE = wx.NewId()
    ID_MOVE = wx.NewId()
    ID_PLAYLIST = wx.NewId()
    ID_PRINT = wx.NewId()
    ID_SYNC = wx.NewId()
    ID_CHECK = wx.NewId()
    ID_SYSTRAY = wx.NewId()
    ID_EXIT = wx.NewId()
    ID_WEBSITE = wx.NewId()
    ID_UPDATE = wx.NewId()
    ID_INSTALL = wx.NewId()
    ID_ABOUT = wx.NewId()
    ID_HELP = wx.NewId()
    ID_PREFS = wx.NewId()
    ID_PROPER = wx.NewId()
    ID_CBDIRNAME = wx.NewId()
    ID_CBFILENAME = wx.NewId()
    ID_TCFILESIZE = wx.NewId()
    ID_CBFILEDATE = wx.NewId()
    ID_RADIOBUTTON1 = wx.NewId()
    ID_RADIOBUTTON2 = wx.NewId()
    ID_BUTTON = wx.NewId()
    ID_LISTCTRL = wx.NewId()
    

    def __init__(self, parent, id, title, instance):
        self.instance = instance        # this object should live until termination
        
        try:
            self.handleConfigData('r')
            self.initconfig = False
        except:
            self.handleConfigData('i')
            self.initconfig = True
        
        windowpos = (wx.Point(int(self.config['Window']['posx']),
                               int(self.config['Window']['posy'])))
        windowsize = (wx.Size(int(self.config['Window']['sizex']),
                              int(self.config['Window']['sizey'])))
        wx.Frame.__init__(self, parent, id, title, pos = windowpos, size = windowsize)  
        self.Bind(wx.EVT_CLOSE, self.onCloseWindow)

        iconfile = ICONPATH + os.sep + APPICON
        if os.path.exists(iconfile):  
            self.icon = wx.Icon(iconfile, wx.BITMAP_TYPE_ICO)
            self.SetIcon(self.icon)
        else:
            self.icon = None

        logfilename = self.inifilename.replace('.ini', '-Action.log')
        logpath = os.path.join(HOMEDIR, logfilename)
        self.alogger = logging.getLogger('ActionLog')
        self.alogger.setLevel(logging.INFO)
        handler = logging.handlers.RotatingFileHandler(logpath, 'a', ACTION_LOG_SIZE, 4)
        formatter = logging.Formatter("%(asctime)s -- %(message)s", "%Y-%m-%d %H:%M:%S")
        handler.setFormatter(formatter)        
        self.alogger.addHandler(handler)
        self.alogger.info('%s started - %s' % (logfilename, VERSION))

        logfilename = self.inifilename.replace('.ini', '-Patch.log')
        logpath = os.path.join(HOMEDIR, logfilename)
        self.plogger = logging.getLogger('PatchLog')
        self.plogger.setLevel(logging.INFO)
        handler = logging.handlers.RotatingFileHandler(logpath, 'a', ACTION_LOG_SIZE, 4)
        formatter = logging.Formatter("%(asctime)s -- %(message)s", "%Y-%m-%d %H:%M:%S")
        handler.setFormatter(formatter)        
        self.plogger.addHandler(handler)
        self.plogger.info('%s started - %s' % (logfilename, VERSION))
        self.misslist = []   
        
        self.initMenu()                                      
        self.sb = mumacustbar.CustomStatusBar(self)
        self.SetStatusBar(self.sb)
        
        collist = self.config['Misc']['hlcolor']
        self.hlcolor = (int(collist[0]), int(collist[1]), int(collist[2]))
        collist = self.config['Misc']['99color']
        self.color99 = (int(collist[0]), int(collist[1]), int(collist[2]))
        collist = self.config['Misc']['altcolor']
        self.altcolor = (int(collist[0]), int(collist[1]), int(collist[2]))
        collist = self.config['Misc']['misscolor']
        self.misscolor = (int(collist[0]), int(collist[1]), int(collist[2]))        
        
        self.tabdata = []
        self.oldorderby = 'Id_Title'
        self.oldorderdir = 'ASC'
        self.config['Misc']['mountedbysm'] = []         # temp list
        
        self._mgr = aui.AuiManager()
        self._mgr.SetManagedWindow(self)

        self.querywin = mumatreectrl.TreeCtrlWithMixin(self, -1, size = wx.Size(200, -1),
                                                       style = wx.TR_LINES_AT_ROOT |
                                                               wx.TR_HAS_BUTTONS |
                                                               wx.TR_HIDE_ROOT |
                                                               wx.TR_FULL_ROW_HIGHLIGHT,
                                                       moveItem = None)
        self.Bind(wx.EVT_TREE_ITEM_ACTIVATED, self.onQuery, self.querywin)
        self.Bind(wx.EVT_TREE_ITEM_RIGHT_CLICK, self.onShowQueryInfo, self.querywin)
        self.updateQuerywin()
        
        self.playlistwin = mumatreectrl.TreeCtrlWithMixin(self, -1, size = wx.Size(200, -1),
                                                          style = wx.TR_LINES_AT_ROOT |
                                                                  wx.TR_HAS_BUTTONS |
                                                                  wx.TR_HIDE_ROOT |
                                                                  wx.TR_FULL_ROW_HIGHLIGHT,
                                                          moveItem = self.moveItem)
        self.Bind(wx.EVT_TREE_ITEM_ACTIVATED, self.onPlayList, self.playlistwin)
        self.Bind(wx.EVT_TREE_SEL_CHANGED, self.onSelChanged, self.playlistwin)
        evtmgr.eventManager.Register(self.onShowPlayContext, wx.EVT_RIGHT_DOWN, self.playlistwin)
        self.playlistwin.Bind(wx.EVT_CHAR, self.onChar)        
        dt = MumaTextDropTarget(self.playlistwin, self.config, self.playlists, self.updatePlaylistWin)
        self.playlistwin.SetDropTarget(dt)
        
        self.unpickleMyTop25()
        
        self.oldplname = self.config['View']['selplname']
        self.updatePlaylistWin()
        try:
            explist = [eval(item) for item in  self.config['Window']['expanstate']]
        except:
            explist = []
        self.playlistwin.SetExpansionState(explist)
        self.markActivePlaylist()
        
        self.logwin = wx.TextCtrl(self, -1, size = wx.Size(-1, 100), style = wx.TE_RICH | wx.TE_MULTILINE | wx.TE_DONTWRAP | wx.TE_READONLY)
        evtmgr.eventManager.Register(self.onContext, wx.EVT_RIGHT_DOWN, self.logwin)
                
        self._notebook_style = aui.AUI_NB_DEFAULT_STYLE | aui.AUI_NB_TAB_EXTERNAL_MOVE | aui.AUI_NB_CLOSE_ON_ALL_TABS | wx.NO_BORDER        
        self.notebook = aui.AuiNotebook(self, -1, agwStyle = self._notebook_style)
        self.Bind(aui.EVT_AUINOTEBOOK_PAGE_CLOSE, self.onQueryTabClose)
        self.Bind(aui.EVT_AUINOTEBOOK_PAGE_CHANGING, self.onQueryTabChange)
        self.onAddQueryTab()

        self.historywin = wx.ListBox(self, -1, size = wx.Size(200, -1), style = wx.LB_SINGLE | wx.LB_HSCROLL)
        self.Bind(wx.EVT_LISTBOX_DCLICK, self.onPlayAgain)
        self.Bind(wx.EVT_LISTBOX, self.onHistSelect)
        #self.Bind(wx.EVT, self.onHistProperties)

        self._mgr.AddPane(self.querywin, aui.AuiPaneInfo().Name('querywin').Caption('Queries').Left())
        self._mgr.AddPane(self.playlistwin, aui.AuiPaneInfo().Name('playwin').Caption('Playlists').Left())
        self._mgr.AddPane(self.notebook, aui.AuiPaneInfo().Name("listwin").Caption('Query results').
                          Center().CloseButton(False).MaximizeButton(True))              
        self._mgr.AddPane(self.historywin, aui.AuiPaneInfo().Name('histwin').Caption('Play history').Right())
        self._mgr.AddPane(self.logwin, aui.AuiPaneInfo().Name('logwin').Caption('Log messages').Bottom())
        gradient = aui.AUI_GRADIENT_HORIZONTAL
        self._mgr.GetArtProvider().SetMetric(aui.AUI_DOCKART_GRADIENT_TYPE, gradient)
        flags = self._mgr.GetAGWFlags()
        flags &= ~aui.AUI_MGR_TRANSPARENT_HINT
        flags &= ~aui.AUI_MGR_VENETIAN_BLINDS_HINT
        self._mgr.SetAGWFlags(flags)       
        flag = aui.AUI_MGR_RECTANGLE_HINT
        self._mgr.SetAGWFlags(self._mgr.GetAGWFlags() ^ flag)
        # self._mgr.Update()  # generates assertion
        self.Bind(aui.EVT_AUI_PANE_CLOSE, self.onPaneClose)
        defaultperspective = self._mgr.SavePerspective()
        if len(self.config['Window']['defaultperspective']) == 0:
            self.config['Window']['defaultperspective'] = defaultperspective       
        if len(self.config['Window']['perspective']):
            self._mgr.LoadPerspective(self.config['Window']['perspective'])
        self._mgr.GetPane('playwin').Caption('Playlists (00:00:00)[0]')
        self._mgr.GetPane('listwin').Caption('Query results (00:00:00)[0]')
        self._mgr.GetPane('histwin').Caption('Play history (00:00:00)[0]')
        self._mgr.Update()
                 
        self.mumadb = None
        self.connect2Database()
        self.playlistitem = None
        self.isplaylistitem = False
        self.updateHistorywin()
        self.param = self.param1 = self.param2 = ''
        self.lastvalidvolint = 1000
        self.devicename = ''
        
        self.tbicon = mumasystray.SysTrayIcon(self, self.icon, TITLE)          
        self.initContextMenu()
        self.initPlayContextMenu()
        
        # Printer init
        self.printdata = wx.PrintData()
        self.printdata.SetPaperId(wx.PAPER_LETTER)
        self.printdata.SetPrintMode(wx.PRINT_MODE_PRINTER)
        self.printhtml = wx.html.HtmlEasyPrinting(VERSION, self)

        try:
            import psyco
            psyco.cannotcompile(self.onPrint)
        except:
            pass

        try:
            import mutagen
            self.config['Misc']['mutagenpresent'] = 'True'
        except:
            self.config['Misc']['mutagenpresent'] = 'False'
        
        self.firstrun = True    
        self.musicdirtimer = wx.Timer(self, -1) 
        self.Bind(wx.EVT_TIMER, self.onCheckMusicDir, self.musicdirtimer) 
        self.musicdirtimer.Start(2000, oneShot = True)
        self.oldtitlecount = self.oldtimesum = 0
        self.timesumtimer = wx.Timer(self, -1) 
        self.Bind(wx.EVT_TIMER, self.onComputeDurationSum, self.timesumtimer) 
        self.timesumtimer.Start(500, oneShot = False)
        
        self.tabdata[self.notebook.GetPageIndex(self.notebook.GetCurrentPage())].curquery = self.config['Database']['currentquery']
        self.onQuery(enterparam = False)

        self.mainmenu.Enable(self.ID_INSTALL, False)
        if self.config['Misc']['update'] == 'True':
            self.updatecheck = True
            self.updatetimer = wx.Timer(self, -1) 
            self.Bind(wx.EVT_TIMER, self.onUpdate, self.updatetimer) 
            self.updatetimer.Start(5000, oneShot = True)
        else:
            self.updatecheck = False
        
        if self.initconfig:
            self.initimer = wx.Timer(self, -1) 
            self.Bind(wx.EVT_TIMER, self.onInitIni, self.initimer) 
            self.initimer.Start(1000, oneShot = True)
    #-------------------------------------------------------------------------------------------

    def initMenu(self):
        self.mainmenu = wx.MenuBar()         
        # File menu
        menu = wx.Menu()                     
        item = wx.MenuItem(menu, self.ID_EXPORT, '&Export matches\tSHIFT-CTRL-E', 'export matches to file')
        item.SetBitmap(loadIcon('export.png'))
        self.Bind(wx.EVT_MENU, self.onExport, id = self.ID_EXPORT)
        menu.AppendItem(item)
        item = wx.MenuItem(menu, self.ID_PRINT, '&Print matches', 'print matches')
        item.SetBitmap(loadIcon('print.png'))
        self.Bind(wx.EVT_MENU, self.onPrint, id = self.ID_PRINT)
        menu.AppendItem(item)
        item = wx.MenuItem(menu, self.ID_COPY, '&Copy file(s)\tSHIFT-CTRL-C', 'copy file(s)')
        item.SetBitmap(loadIcon('copy.png'))
        self.Bind(wx.EVT_MENU, self.onCopy, id = self.ID_COPY)
        menu.AppendItem(item)
        menu.AppendSeparator()
        item = wx.MenuItem(menu, self.ID_IMPORT, '&Import csv\tCTRL-I', 'import csv file')
        item.SetBitmap(loadIcon('import.png'))
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onImport, id = self.ID_IMPORT)
        item = wx.MenuItem(menu, self.ID_REPLACE, '&Replace title files', 'replace title files')
        item.SetBitmap(loadIcon('replace.png'))
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onReplace, id = self.ID_REPLACE)
        item = wx.MenuItem(menu, self.ID_MOVE, '&Move title files', 'move title files')
        item.SetBitmap(loadIcon('move.png'))
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onMove, id = self.ID_MOVE)
        item = wx.MenuItem(menu, self.ID_SYNC, '&Sync state\tCTRL-S', 'synchronize db and music volume')
        item.SetBitmap(loadIcon('sync.png'))
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onSynchronize, id = self.ID_SYNC)
        menu.AppendSeparator() 
        item = wx.MenuItem(menu, self.ID_EXIT, 'E&xit MusicMan\tCTRL-X', 'exit the application')
        item.SetBitmap(loadIcon('exit.png'))                 
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onCloseWindow, id = self.ID_EXIT)
        self.mainmenu.Append(menu, '&File')
        
        # Edit menu
        menu = wx.Menu()
        item = wx.MenuItem(menu, self.ID_ADDRECORD, '&Add record\tSHIFT-ALT-A', 'add new record')
        item.SetBitmap(loadIcon('additem.png'))
        menu.AppendItem(item)
        menu.SetLabel(self.ID_ADDRECORD, '%s\t%s'%('&Add record [F4]', 'SHIFT-ALT-A'))
        self.Bind(wx.EVT_MENU, self.onAddRecord, id = self.ID_ADDRECORD)
        item = wx.MenuItem(menu, self.ID_EDITRECORD, '&Edit record\tSHIFT+ALT+E', 'edit selected record')
        item.SetBitmap(loadIcon('edititem.png'))
        self.Bind(wx.EVT_MENU, self.onEditRecord, id = self.ID_EDITRECORD)
#        accel_tbl = wx.AcceleratorTable([(wx.ACCEL_SHIFT | wx.ACCEL_ALT,  ord('E'), self.ID_EDITRECORD),
#                                         (wx.ACCEL_NORMAL, wx.WXK_F2, self.ID_EDITRECORD)])
#        self.SetAcceleratorTable(accel_tbl)       
        menu.AppendItem(item)
        menu.SetLabel(self.ID_EDITRECORD, '%s\t%s'%('&Edit record [F2]', 'SHIFT-ALT-E'))    
        item = wx.MenuItem(menu, self.ID_REMOVERECORD, '&Remove record(s)\tSHIFT-ALT-R', 'remove selected record(s)')
        item.SetBitmap(loadIcon('removeitem.png'))
        self.Bind(wx.EVT_MENU, self.onRemoveRecord, id = self.ID_REMOVERECORD)
        menu.AppendItem(item)
        menu.AppendSeparator()
        item = wx.MenuItem(menu, self.ID_BATCHEDIT, 'Batch edit', 'rename one / more record(s)')
        item.SetBitmap(loadIcon('batchedit.png'))
        self.Bind(wx.EVT_MENU, self.onBatchEdit, id = self.ID_BATCHEDIT)
        menu.AppendItem(item)        
        self.mainmenu.Append(menu, '&Edit')

        # View menu
        menu = wx.Menu()
        menu.Append(self.ID_QUERYWIN, '&Queries\tSHIFT-ALT-Q', 'show queries list window', wx.ITEM_CHECK)
        self.Bind(wx.EVT_MENU, self.onQueryWin, id = self.ID_QUERYWIN)        
        menu.Append(self.ID_PLAYWIN, '&Playlist\tSHIFT-ALT-P', 'show play list window', wx.ITEM_CHECK)
        self.Bind(wx.EVT_MENU, self.onPlayWin, id = self.ID_PLAYWIN)        
        menu.Append(self.ID_LOGWIN, '&Log messages\tSHIFT-ALT-L', 'show log window', wx.ITEM_CHECK)
        self.Bind(wx.EVT_MENU, self.onLogWin, id = self.ID_LOGWIN)
        menu.Append(self.ID_HISTWIN, '&Play history\tSHIFT-ALT-H', 'show play history window', wx.ITEM_CHECK)
        self.Bind(wx.EVT_MENU, self.onHistWin, id = self.ID_HISTWIN)
        menu.AppendSeparator()
        menu.Append(self.ID_WINDOW_TLC, 'Windows to TLC\tF12', 'set windows to top left corner')
        self.Bind(wx.EVT_MENU, self.onWindowTLC, id = self.ID_WINDOW_TLC)
        menu.AppendSeparator()
        menu.Append(self.ID_INI_PERSP, '&Default perspective', 'set views to default perspective')
        self.Bind(wx.EVT_MENU, self.onPerspective, id = self.ID_INI_PERSP)
        self.mainmenu.Append(menu, '&View')

        # Play menu
        menu = wx.Menu()
        item = wx.MenuItem(menu, self.ID_PLAY, '&Play title(s)\tCTRL-P', 'play title with musicplayer')
        item.SetBitmap(loadIcon('play.png'))
        self.Bind(wx.EVT_MENU, self.onPlayTitle, id = self.ID_PLAY)
        menu.AppendItem(item)
        item = wx.MenuItem(menu, self.ID_QUEUE, '&Queue title(s)\tCTRL-Q', 'queue title in musicplayer')
        item.SetBitmap(loadIcon('play.png'))
        self.Bind(wx.EVT_MENU, self.onQueueTitle, id = self.ID_QUEUE)
        menu.AppendItem(item)
        item = wx.MenuItem(menu, self.ID_PLAYLIST, 'Play playlis&t (item)\tCTRL-SHIFT-P', 'add title to playerlist')
        item.SetBitmap(loadIcon('playlist.png'))
        self.Bind(wx.EVT_MENU, self.onPlayList, id = self.ID_PLAYLIST)
        menu.AppendItem(item)
        item = wx.MenuItem(menu, self.ID_QUEUELIST, 'Queue playlis&t (item)\tCTRL-SHIFT-Q', 'queue title to playerlist')
        item.SetBitmap(loadIcon('playlist.png'))
        self.Bind(wx.EVT_MENU, self.onQueueList, id = self.ID_QUEUELIST)
        menu.AppendItem(item)
        menu.AppendSeparator()
        item = wx.MenuItem(menu, self.ID_ADDPLAY, 'Add to playlist\tCTRL-A', 'add title to playlist')
        item.SetBitmap(loadIcon('addlist.png'))
        self.Bind(wx.EVT_MENU, self.onAddToPlaylist, id = self.ID_ADDPLAY)
        menu.AppendItem(item)
        item = wx.MenuItem(menu, self.ID_NEWPLAY, '&New playlist\tCTRL-N', 'create new playlist')
        item.SetBitmap(loadIcon('newlist.png'))
        self.Bind(wx.EVT_MENU, self.onNewPlaylist, id = self.ID_NEWPLAY)
        menu.AppendItem(item)
        item = wx.MenuItem(menu, self.ID_REMOVEPLAY, '&Remove playlist\tCTRL-SHIFT-R', 'remove selected playlist')
        item.SetBitmap(loadIcon('dellist.png'))
        self.Bind(wx.EVT_MENU, self.onRemovePlaylist, id = self.ID_REMOVEPLAY)
        menu.AppendItem(item)
        item = wx.MenuItem(menu, self.ID_SAVEPLAY, '&Save playlist\tCTRL-SHIFT-S', 'save playlist as csv')
        item.SetBitmap(loadIcon('savelist.png'))
        self.Bind(wx.EVT_MENU, self.onSavePlaylist, id = self.ID_SAVEPLAY)
        menu.AppendItem(item)
        item = wx.MenuItem(menu, self.ID_EXPORTPLAY, 'Export playlist', 'export selected playlist')
        item.SetBitmap(loadIcon('exportlist.png'))
        self.Bind(wx.EVT_MENU, self.onExportPlaylist, id = self.ID_EXPORTPLAY)
        menu.AppendItem(item)
        item = wx.MenuItem(menu, self.ID_IMPORTPLAY, 'Import playlist', 'import and add playlist')
        item.SetBitmap(loadIcon('importlist.png'))
        self.Bind(wx.EVT_MENU, self.onImportPlaylist, id = self.ID_IMPORTPLAY)
        menu.AppendItem(item)   
        menu.AppendSeparator()        
        item = wx.MenuItem(menu, self.ID_REMOVEITEM, 'Remove &item from list\tCTRL-R')
        item.SetBitmap(loadIcon('removefromlist.png'))
        self.Bind(wx.EVT_MENU, self.onRemovePlayItem, id = self.ID_REMOVEITEM)
        menu.AppendItem(item)
        item = wx.MenuItem(menu, self.ID_MOVEUP, 'Move item &up\tCTRL-U')
        item.SetBitmap(loadIcon('moveup.png'))
        self.Bind(wx.EVT_MENU, self.onMoveItemUp, id = self.ID_MOVEUP)
        menu.AppendItem(item)
        item = wx.MenuItem(menu, self.ID_MOVEDOWN, 'Move item &down\tCTRL-D')
        item.SetBitmap(loadIcon('movedown.png'))
        self.Bind(wx.EVT_MENU, self.onMoveItemDown, id = self.ID_MOVEDOWN)
        menu.AppendItem(item)
        menu.AppendSeparator()
        item = wx.MenuItem(menu, self.ID_CHECKLIST, 'Check playlist\tCTRL-SHIFT-C', 'check playlist')
        item.SetBitmap(loadIcon('checklist.png'))
        self.Bind(wx.EVT_MENU, self.onCheckList, id = self.ID_CHECKLIST)
        menu.AppendItem(item)       
        self.mainmenu.Append(menu, '&Play(list)')

        # Query menu
        menu = wx.Menu()
        item = wx.MenuItem(menu, self.ID_MANAGE_QUERIES, 'Manage Queries\tCTRL-M', 'manage queries')
        item.SetBitmap(loadIcon('managequeries.png'))
        menu.AppendItem(item)    
        self.Bind(wx.EVT_MENU, self.onManageQueries, id = self.ID_MANAGE_QUERIES)
        item = wx.MenuItem(menu, self.ID_QUERY, 'Execute &query\tCTRL-E', 'execute query')
        item.SetBitmap(loadIcon('query.png'))         
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onQuery, id = self.ID_QUERY)        
        item = wx.MenuItem(menu, self.ID_QUICK, 'Quic&k query\tCTRL-K', 'quick query')
        item.SetBitmap(loadIcon('quickquery.png'))
        menu.AppendItem(item)
        menu.SetLabel(self.ID_QUICK, '%s\t%s'%('Quic&k query [F3]', 'CTRL+K'))    
        self.Bind(wx.EVT_MENU, self.onQuick, id = self.ID_QUICK)
        item = wx.MenuItem(menu, self.ID_QUICK_TAB, 'Quic&k query (new tab)\tF9', 'quick query new tab')
        item.SetBitmap(loadIcon('quickquery.png'))
        menu.AppendItem(item)   
        self.Bind(wx.EVT_MENU, self.onQuick, id = self.ID_QUICK_TAB)
        item = wx.MenuItem(menu, self.ID_OPENVOL, 'Open volume\tF8', 'open volume')
        item.SetBitmap(loadIcon('openvol.png'))
        menu.AppendItem(item) 
        self.Bind(wx.EVT_MENU, self.onOpenVolume, id = self.ID_OPENVOL)    
        item = wx.MenuItem(menu, self.ID_QUERY_TOI, 'Query same TOI\tF6', 'query Title Or Interpreter')
        item.SetBitmap(loadIcon('quickquery.png'))         
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onQueryTOI, id = self.ID_QUERY_TOI)                
        menu.AppendSeparator()
        item = wx.MenuItem(menu, self.ID_REFRESH_QUERY, 'Refresh query\tF5', 'refresh last query')
        item.SetBitmap(loadIcon('refreshq.png'))         
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onRefreshQuery, id = self.ID_REFRESH_QUERY)            
        item = wx.MenuItem(menu, self.ID_QUERYTAB, 'Add query &tab\tCTRL-T', 'new query tab')
        item.SetBitmap(loadIcon('querytab.png'))         
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onAddQueryTab, id = self.ID_QUERYTAB)
        menu.AppendSeparator()  
        item = wx.MenuItem(menu, self.ID_OPEN_DB, '&Open database\tCTRL-O', 'open database')
        item.SetBitmap(loadIcon('dbman.png'))
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onOpenDb, id = self.ID_OPEN_DB)        
        item = wx.MenuItem(menu, self.ID_BACKUP_DB, '&Backup database\tCTRL-B', 'force database backup')
        item.SetBitmap(loadIcon('backupdb.png'))
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onBackupDb, id = self.ID_BACKUP_DB)
        item = wx.MenuItem(menu, self.ID_RESTORE_DB, 'Rest&ore database\tCTRL-R', 'restore database backup')
        item.SetBitmap(loadIcon('restoredb.png'))
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onRestoreDb, id = self.ID_RESTORE_DB)
        item = wx.MenuItem(menu, self.ID_IMPORT_DB, 'Import database', 'import database')
        item.SetBitmap(loadIcon('importdb.png'))
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onImportDb, id = self.ID_IMPORT_DB)
        item = wx.MenuItem(menu, self.ID_COMPACT_DB, '&Compact database\tCTRL-C', 'compact / optimize database backup')
        item.SetBitmap(loadIcon('compactdb.png'))
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onCompactDb, id = self.ID_COMPACT_DB)
        self.mainmenu.Append(menu, '&Query')
        
        # Options menu
        menu = wx.Menu()
        item = wx.MenuItem(menu, self.ID_PREFS, 'Pre&ferences\tCTRL-F', 'set preferences')
        item.SetBitmap(loadIcon('prefs.png'))
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onPreferences, id = self.ID_PREFS)        
        item = wx.MenuItem(menu, self.ID_PROPER, 'File &properties\tCTRL-SHIFT-F', 'music file show properties')
        item.SetBitmap(loadIcon('properties.png'))
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onProperties, id = self.ID_PROPER)        
        item = wx.MenuItem(menu, self.ID_CHECK, 'Check title [F7]\tCTRL-H', 'Check this title')
        item.SetBitmap(loadIcon('checktitle.png'))
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onCheckTitle, id = self.ID_CHECK)        
        accel_tbl = wx.AcceleratorTable([(wx.ACCEL_SHIFT | wx.ACCEL_ALT,  ord('A'), self.ID_ADDRECORD),
                                         (wx.ACCEL_NORMAL, wx.WXK_F4, self.ID_ADDRECORD),
                                         (wx.ACCEL_SHIFT | wx.ACCEL_ALT,  ord('E'), self.ID_EDITRECORD),
                                         (wx.ACCEL_NORMAL, wx.WXK_F2, self.ID_EDITRECORD),                                         
                                         (wx.ACCEL_CTRL,  ord('K'), self.ID_QUICK),
                                         (wx.ACCEL_NORMAL, wx.WXK_F3, self.ID_QUICK),
                                         (wx.ACCEL_CTRL,  ord('H'), self.ID_CHECK),
                                         (wx.ACCEL_NORMAL, wx.WXK_F7, self.ID_CHECK)])
        self.SetAcceleratorTable(accel_tbl)               
        menu.AppendSeparator()
        item = wx.MenuItem(menu, self.ID_SYSTRAY, 'S&ystray icon\tCTRL-Y', 'minimize app to systray')
        item.SetBitmap(loadIcon('systray.png'))
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onSysTray, id = self.ID_SYSTRAY)
        self.mainmenu.Append(menu, '&Options')
        
        # Help menu
        menu = wx.Menu()
        item = wx.MenuItem(menu, self.ID_HELP, '&Help (online)\tF1', 'html online help docs')
        item.SetBitmap(loadIcon('help.png'))  
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onHandbook, id = self.ID_HELP)
        menu.AppendSeparator()
        item = wx.MenuItem(menu, self.ID_WEBSITE, 'MusicMan Website\tCTRL-W', 'MusicMan Website')
        item.SetBitmap(loadIcon('website.png'))  
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onWebsite, id = self.ID_WEBSITE)
        menu.AppendSeparator()     
        item = wx.MenuItem(menu, self.ID_UPDATE, 'MusicMan Updates\tCTRL-SHIFT-U', 'Download newest version')
        item.SetBitmap(loadIcon('update.png'))  
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onUpdate, id = self.ID_UPDATE)                
        item = wx.MenuItem(menu, self.ID_INSTALL, 'install MusicMan Updates', 'install newest version')
        item.SetBitmap(loadIcon('update.png'))  
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onInstallUpdate, id = self.ID_INSTALL)                
        menu.AppendSeparator()        
        item = wx.MenuItem(menu, self.ID_ABOUT, 'About MusicMan\tSHIFT-CTRL-A', 'about the application')
        item.SetBitmap(loadIcon('about.png'))  
        menu.AppendItem(item)
        self.Bind(wx.EVT_MENU, self.onAbout, id = self.ID_ABOUT)
        self.mainmenu.Append(menu, '&Help')

        self.SetMenuBar(self.mainmenu)
        
        self.mainmenu.Check(self.ID_QUERYWIN, self.config['View']['querywin'] == 'True')
        self.mainmenu.Check(self.ID_PLAYWIN, self.config['View']['playwin'] == 'True')  
        self.mainmenu.Check(self.ID_LOGWIN, self.config['View']['logwin'] == 'True')
        self.mainmenu.Check(self.ID_HISTWIN, self.config['View']['histwin'] == 'True')
        isexpert = self.config['Misc']['expert'] == 'True'
        self.mainmenu.Enable(self.ID_IMPORT, isexpert)
        self.mainmenu.Enable(self.ID_MOVE, isexpert)
        self.mainmenu.Enable(self.ID_ADDRECORD, isexpert)
        self.mainmenu.Enable(self.ID_EDITRECORD, isexpert)
        self.mainmenu.Enable(self.ID_REMOVERECORD, isexpert)
        self.mainmenu.Enable(self.ID_BATCHEDIT, isexpert)
        self.mainmenu.Enable(self.ID_CHECK, isexpert)
        self.mainmenu.Enable(self.ID_QUERYTAB, self.config['ListCtrl']['newtab'] == 'False')
    #-------------------------------------------------------------------------------------------
        
    def initContextMenu(self):
        self.popupID0 = wx.NewId()  
        self.popupID1 = wx.NewId()          
        self.popupID2 = wx.NewId()
        self.popupID3 = wx.NewId()
        self.popupID4 = wx.NewId()
        self.popupID5 = wx.NewId()
        self.popupID6 = wx.NewId()
        self.popupID7 = wx.NewId()
        self.popupID8 = wx.NewId()
        self.popupID9 = wx.NewId()
        self.popupID10 = wx.NewId()
        self.popupID11 = wx.NewId()
        self.popupID12 = wx.NewId()
        self.popupID13 = wx.NewId()
        self.popupID14 = wx.NewId()
        self.popupID15 = wx.NewId()
        
        self.Bind(wx.EVT_MENU, self.onPlayTitle, id = self.popupID0)
        self.Bind(wx.EVT_MENU, self.onQueueTitle, id = self.popupID1)
        self.Bind(wx.EVT_MENU, self.onAddToPlaylist, id = self.popupID2)
        self.Bind(wx.EVT_MENU, self.onQuery, id = self.popupID3)
        self.Bind(wx.EVT_MENU, self.onOpenVolume, id = self.popupID15)
        self.Bind(wx.EVT_MENU, self.onAddQueryTab, id = self.popupID14)
        self.Bind(wx.EVT_MENU, self.onManageQueries, id = self.popupID4)
        self.Bind(wx.EVT_MENU, self.onAddRecord, id = self.popupID5)
        self.Bind(wx.EVT_MENU, self.onEditRecord, id = self.popupID6)
        self.Bind(wx.EVT_MENU, self.onOpenDb, id = self.popupID7)
        self.Bind(wx.EVT_MENU, self.onExport, id = self.popupID8)
        self.Bind(wx.EVT_MENU, self.onCopy, id = self.popupID12)
        self.Bind(wx.EVT_MENU, self.onPrint, id = self.popupID9)
        self.Bind(wx.EVT_MENU, self.onPreferences, id = self.popupID10)
        self.Bind(wx.EVT_MENU, self.onProperties, id = self.popupID13)
        self.Bind(wx.EVT_MENU, self.onSysTray, id = self.popupID11)
    #-------------------------------------------------------------------------------------------

    def onContext(self, event):
        """ context menu handler """
        menu = wx.Menu()
        item = wx.MenuItem(menu, self.popupID0, 'Play &title(s)\tCTRL-P')
        item.SetBitmap(loadIcon('play.png'))
        menu.AppendItem(item)
        item = wx.MenuItem(menu, self.popupID1, '&Queue title(s)\tCTRL-Q')
        item.SetBitmap(loadIcon('play.png'))
        menu.AppendItem(item)        
        item = wx.MenuItem(menu, self.popupID2, '&Add to playlist\tCTRL-A')
        item.SetBitmap(loadIcon('addlist.png'))
        menu.AppendItem(item)
        menu.AppendSeparator()        
        item = wx.MenuItem(menu, self.popupID3, '&Query database\tCTRL-Q')
        item.SetBitmap(loadIcon('query.png'))        
        menu.AppendItem(item)
        item = wx.MenuItem(menu, self.popupID15, 'Open volume\tF8')
        item.SetBitmap(loadIcon('openvol.png'))        
        menu.AppendItem(item)
        item = wx.MenuItem(menu, self.popupID14, 'Add query &tab\tCTRL-T')
        item.SetBitmap(loadIcon('querytab.png'))        
        menu.AppendItem(item)
        menu.Enable(self.popupID14, self.config['ListCtrl']['newtab'] == 'False') 
        item = wx.MenuItem(menu, self.popupID4, 'Manage &queries\tCTRL-M')
        item.SetBitmap(loadIcon('managequeries.png'))
        menu.AppendItem(item)
        menu.AppendSeparator()
        item = wx.MenuItem(menu, self.popupID5, '&Add record [F4]\tSHIFT-ALT-A')
        item.SetBitmap(loadIcon('additem.png'))
        menu.AppendItem(item)
        menu.Enable(self.popupID5, self.config['Misc']['expert'] == 'True') 
        item = wx.MenuItem(menu, self.popupID6, '&Edit record [F2]\tSHIFT-ALT-E')
        item.SetBitmap(loadIcon('edititem.png'))
        menu.AppendItem(item)
        menu.Enable(self.popupID6, self.config['Misc']['expert'] == 'True') 
        menu.AppendSeparator()        
        item = wx.MenuItem(menu, self.popupID7, '&Open database\tCTRL-O')
        item.SetBitmap(loadIcon('dbman.png'))         
        menu.AppendItem(item)
        menu.AppendSeparator()        
        item = wx.MenuItem(menu, self.popupID8, '&Export matches\tSHIFT-CTRL-E')
        item.SetBitmap(loadIcon('export.png'))
        menu.AppendItem(item)
        item = wx.MenuItem(menu, self.popupID9, '&Print matches')
        item.SetBitmap(loadIcon('print.png'))
        menu.AppendItem(item)       
        item = wx.MenuItem(menu, self.popupID12, '&Copy file(s)\tSHIFT-CTRL-C')
        item.SetBitmap(loadIcon('copy.png'))
        menu.AppendItem(item)
        menu.AppendSeparator()
        item = wx.MenuItem(menu, self.popupID10, 'Pre&ferences\tCTRL-F')
        item.SetBitmap(loadIcon('prefs.png'))
        menu.AppendItem(item)
        item = wx.MenuItem(menu, self.popupID13, 'File &properties\tCTRL-SHIFT-F')
        item.SetBitmap(loadIcon('properties.png'))
        menu.AppendItem(item)        
        item = wx.MenuItem(menu, self.popupID11, 'S&ystray icon\tCTRL-Y')
        item.SetBitmap(loadIcon('systray.png'))
        menu.AppendItem(item)
        self.PopupMenu(menu)
        menu.Destroy()
    #-------------------------------------------------------------------------------------------      

    def initPlayContextMenu(self):
        self.playpopupID0 = wx.NewId()  
        self.playpopupID1 = wx.NewId()          
        self.playpopupID2 = wx.NewId()
        self.playpopupID3 = wx.NewId()
        self.playpopupID4 = wx.NewId()
        self.playpopupID5 = wx.NewId()
        self.playpopupID6 = wx.NewId()
        self.playpopupID7 = wx.NewId()
        self.playpopupID8 = wx.NewId()
        self.playpopupID9 = wx.NewId()
        self.playpopupID10 = wx.NewId()
        self.playpopupID11 = wx.NewId()
        
        self.Bind(wx.EVT_MENU, self.onPlayList, id = self.playpopupID0)
        self.Bind(wx.EVT_MENU, self.onQueueList, id = self.playpopupID1)
        self.Bind(wx.EVT_MENU, self.onCopyList, id = self.playpopupID2)
        self.Bind(wx.EVT_MENU, self.onNewPlaylist, id = self.playpopupID3)
        self.Bind(wx.EVT_MENU, self.onRemovePlaylist, id = self.playpopupID4)
        self.Bind(wx.EVT_MENU, self.onSavePlaylist, id = self.playpopupID5)
        self.Bind(wx.EVT_MENU, self.onRemovePlayItem, id = self.playpopupID6)
        self.Bind(wx.EVT_MENU, self.onMoveItemUp, id = self.playpopupID7)
        self.Bind(wx.EVT_MENU, self.onMoveItemDown, id = self.playpopupID8)
        self.Bind(wx.EVT_MENU, self.onPlayProperties, id = self.playpopupID9)
        self.Bind(wx.EVT_MENU, self.onExportPlaylist, id = self.playpopupID10)
        self.Bind(wx.EVT_MENU, self.onImportPlaylist, id = self.playpopupID11)
    #-------------------------------------------------------------------------------------------

    def onShowPlayContext(self, event):
        """ context menu handler """
        menu = wx.Menu()       
        item = wx.MenuItem(menu, self.playpopupID0, 'Play playlis&t (item)\tCTRL-SHIFT-P')
        item.SetBitmap(loadIcon('playlist.png')) 
        menu.AppendItem(item)       
        item = wx.MenuItem(menu, self.playpopupID1, 'Queue playlis&t (item)\tCTRL-SHIFT-Q')
        item.SetBitmap(loadIcon('playlist.png')) 
        menu.AppendItem(item)       
        item = wx.MenuItem(menu, self.playpopupID2, '&Copy playlist (item)\tCTRL-SHIFT-C')
        item.SetBitmap(loadIcon('copy.png')) 
        menu.AppendItem(item)      
        menu.AppendSeparator() 
        item = wx.MenuItem(menu, self.playpopupID6, 'Remove &item from list\tCTRL-I')
        item.SetBitmap(loadIcon('removefromlist.png'))
        menu.AppendItem(item)
        item = wx.MenuItem(menu, self.playpopupID7, 'Move item &up\tCTRL-U')
        item.SetBitmap(loadIcon('moveup.png'))
        menu.AppendItem(item)
        item = wx.MenuItem(menu, self.playpopupID8, 'Move item &down\tCTRL-D')
        item.SetBitmap(loadIcon('movedown.png'))
        menu.AppendItem(item)
        menu.AppendSeparator()        
        item = wx.MenuItem(menu, self.playpopupID3, '&New playlist\tCTRL-N')
        item.SetBitmap(loadIcon('newlist.png'))
        menu.AppendItem(item)
        item = wx.MenuItem(menu, self.playpopupID4, '&Remove playlist\tCTRL-R')
        item.SetBitmap(loadIcon('dellist.png'))        
        menu.AppendItem(item)
        item = wx.MenuItem(menu, self.playpopupID5, '&Save playlist\tCTRL-S')
        item.SetBitmap(loadIcon('savelist.png'))
        menu.AppendItem(item)
        item = wx.MenuItem(menu, self.playpopupID10, 'Export playlist')
        item.SetBitmap(loadIcon('exportlist.png'))
        menu.AppendItem(item)
        item = wx.MenuItem(menu, self.playpopupID11, 'Import playlist')
        item.SetBitmap(loadIcon('importlist.png'))
        menu.AppendItem(item)        
        menu.AppendSeparator() 
        item = wx.MenuItem(menu, self.playpopupID9, 'File properties')
        item.SetBitmap(loadIcon('properties.png'))
        menu.AppendItem(item)        
        self.PopupMenu(menu)
        menu.Destroy()
    #-------------------------------------------------------------------------------------------      

    def onQueryTabClose(self, event):
        del self.tabdata[event.selection]
        self.notebook.SetCloseButton(self.notebook.GetPageIndex(self.tabdata[0].querywin), len(self.tabdata) > 1)
        self.notebook.HideAllTabs((self.config['ListCtrl']['hidetab'] == 'True') and (len(self.tabdata) == 1))
        self._mgr.Update()
    #-------------------------------------------------------------------------------------------
        
    def onQueryTabChange(self, event):
        evtmgr.eventManager.DeregisterListener(self.onContext)
        tabdata = self.tabdata[event.selection]
        self.listwin = tabdata.querywin
        self.querydata = tabdata.querydata
        self.oldorderby = tabdata.orderby
        self.oldorderdir = tabdata.orderdir
        self.Bind(wx.EVT_LIST_COL_END_DRAG, self.onColSize, self.listwin)
        self.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.onPlayTitle, self.listwin)
        self.Bind(wx.EVT_LIST_COL_CLICK, self.onColumnSort, self.listwin)
        self.Bind(wx.EVT_LIST_BEGIN_DRAG, self.onListwinDragInit, id=self.listwin.GetId())
        self.listwin.Bind(wx.EVT_CHAR, self.onVolume) 
        evtmgr.eventManager.Register(self.onContext, wx.EVT_RIGHT_DOWN, self.listwin)
        try:
            self.updateMatches(tabdata.matchcount)
        except:
            pass
        # in this handler event.selection gives the correct new index
        # while self.notebook.GetPageIndex(self.notebook.GetCurrentPage()) gives
        # the previously selected index
        # print event.selection, self.notebook.GetPageIndex(self.notebook.GetCurrentPage())
    #-------------------------------------------------------------------------------------------

    def onAddQueryTab(self, event = None):
        newwin = mumavlist.VirtualListCtrl(self.notebook, self.ID_LISTCTRL, self.config)
        self.myfont = wx.Font(int(self.config['ListCtrl']['fsize']), wx.DEFAULT, wx.NORMAL, wx.NORMAL, False, self.config['ListCtrl']['font'])
        newwin.SetFont(self.myfont)
        self.tabdata.append(TabData(newwin, self.querydata, self.querydata['columns'][0], 'ASC', '', 0))
        page_bmp = wx.ArtProvider.GetBitmap(wx.ART_NORMAL_FILE, wx.ART_OTHER, wx.Size(16, 16))
        self.notebook.AddPage(newwin, 'Query results', False, page_bmp)
        wincount = len(self.tabdata)                   
        self.notebook.SetCloseButton(self.notebook.GetPageIndex(self.tabdata[0].querywin), wincount > 1)
        self.notebook.HideAllTabs((self.config['ListCtrl']['hidetab'] == 'True') and (len(self.tabdata) == 1))
        self.notebook.DoSizing()    # Windows fix - if not active, tabs will be shown >= 3!                   
        self.notebook.SetSelection(wincount - 1)
        self._mgr.Update()
    #-------------------------------------------------------------------------------------------
            
    def onPerspective(self, event):
        self.setWindowDefaults()
        self.SetDimensions(int(self.config['Window']['posx']),
                           int(self.config['Window']['posy']),
                           int(self.config['Window']['sizex']),
                           int(self.config['Window']['sizey']))
        self.Refresh()
        self.config['View']['querywin'] = 'True'
        self.config['View']['logwin'] = 'True'
        self.config['View']['histwin'] = 'True'
        self.mainmenu.Check(self.ID_QUERYWIN, True)       
        self.mainmenu.Check(self.ID_LOGWIN, True)
        self.mainmenu.Check(self.ID_HISTWIN, True)
        self._mgr.LoadPerspective(self.config['Window']['defaultperspective'])
    #-------------------------------------------------------------------------------------------
        
    def onPaneClose(self, event):
        mb = self.GetMenuBar()
        name = event.GetPane().name
        if name == 'querywin':
            submenuitem = mb.FindItemById(self.ID_QUERYWIN)
            self.config['View']['querywin'] = 'False'
        elif name == 'playwin':
            submenuitem = mb.FindItemById(self.ID_PLAYWIN)
            self.config['View']['playwin'] = 'False'
        elif name == 'histwin':
            submenuitem = mb.FindItemById(self.ID_HISTWIN)
            self.config['View']['histwin'] = 'False'
        else:
            submenuitem = mb.FindItemById(self.ID_LOGWIN)
            self.config['View']['logwin'] = 'False'
        submenuitem.Check(False)    
    #-------------------------------------------------------------------------------------------

    def onQueryWin(self, event):
        mb = self.GetMenuBar()
        submenuitem = mb.FindItemById(self.ID_QUERYWIN)
        if submenuitem.IsChecked():
            self._mgr.GetPane('querywin').Show()
        else:
            self._mgr.GetPane('querywin').Hide()
        self.config['View']['querywin'] = str(submenuitem.IsChecked())            
        self._mgr.Update()
    #-------------------------------------------------------------------------------------------

    def onPlayWin(self, event):        
        mb = self.GetMenuBar()
        submenuitem = mb.FindItemById(self.ID_PLAYWIN)
        if submenuitem.IsChecked():
            self._mgr.GetPane('playwin').Show()
        else:
            self._mgr.GetPane('playwin').Hide()
        self.config['View']['playwin'] = str(submenuitem.IsChecked())            
        self._mgr.Update()
    #-------------------------------------------------------------------------------------------

    def onLogWin(self, event):
        mb = self.GetMenuBar()
        submenuitem = mb.FindItemById(self.ID_LOGWIN)
        if submenuitem.IsChecked():
            self._mgr.GetPane('logwin').Show()
        else:
            self._mgr.GetPane('logwin').Hide()
        self.config['View']['logwin'] = str(submenuitem.IsChecked())
        self._mgr.Update()
    #-------------------------------------------------------------------------------------------

    def onHistWin(self, event):
        mb = self.GetMenuBar()
        submenuitem = mb.FindItemById(self.ID_HISTWIN)
        if submenuitem.IsChecked():
            self._mgr.GetPane('histwin').Show()
        else:
            self._mgr.GetPane('histwin').Hide()
        self.config['View']['listwin'] = str(submenuitem.IsChecked())
        self._mgr.Update()
    #-------------------------------------------------------------------------------------------

    def onListwinDragInit(self, event):
        index = event.GetIndex()
        id_title = self.listwin.GetItemText(index)
        if id_title == '': return
        title = self.listwin.GetItem(index, 1).GetText()
        tdo = wx.TextDataObject('%s, %s' % (title, id_title))
        tds = wx.DropSource(self.listwin)
        tds.SetData(tdo)
        tds.DoDragDrop(True)
    #-------------------------------------------------------------------------------------------

    def updateHistorywin(self):
        history = self.config['Misc']['history']
        titles = []
        for index in range(0, len(history), 2):
            titles.append('%s, %s' % (history[index], history[index + 1]))
        self.historywin.Set(titles)
        if len(titles): self.historywin.EnsureVisible(len(titles) - 1)
        self._mgr.GetPane('histwin').Caption('Play history (00:00:00)[0]')
        self._mgr.Update()
    #-------------------------------------------------------------------------------------------

    def updateDbName(self, recordcount = None):
        self.sb.SetStatusText(self.config['Database']['pathname'], 4)
        if recordcount != None:
            self.updateRecords(recordcount)
        if self.mumadb != None:
            self.sb.setLed(GREEN, True)
        else:
            self.sb.setLed(RED, True)
    #-------------------------------------------------------------------------------------------

    def updateQuerywin(self):
        querynames = list(self.config['Queries'].keys())
        querynames.remove('QuickQuery')
        querynames.remove('AlbumTitles')
        querynames.remove('AddRecord')
        querynames.sort()
        self.querywin.DeleteAllItems()            
        root = self.querywin.AddRoot('All queries', 0)
        for index, queryname in enumerate(querynames):
            item = self.querywin.AppendItem(root, queryname, 1)
            self.querywin.SetItemImage(item, self.querywin.fileidx, wx.TreeItemIcon_Normal) #neu
      #  self.querywin.ExpandAll()
    #-------------------------------------------------------------------------------------------

    def updatePlaylistWin(self, selitemlabel = '', plname = ''):
        retval = None
        playlistnames = list(self.playlists.keys())
        playlistnames.sort()
        explist = self.playlistwin.GetExpansionState()
        
        self.playlistwin.DeleteAllItems()
        root = self.playlistwin.AddRoot("Playlists", 0)
        self.playlistwin.SetPyData(root, None) #neu
        self.playlistwin.SetItemImage(root, self.playlistwin.fldridx, wx.TreeItemIcon_Normal) #neu
        self.playlistwin.SetItemImage(root, self.playlistwin.fldropenidx, wx.TreeItemIcon_Expanded) #neu
                
        for playlistname in playlistnames:        
            item = self.playlistwin.AppendItem(root, playlistname, 0)            
            self.playlistwin.SetPyData(item, None) #neu
            self.playlistwin.SetItemImage(item, self.playlistwin.fldridx, wx.TreeItemIcon_Normal) #neu
            self.playlistwin.SetItemImage(item, self.playlistwin.fldropenidx, wx.TreeItemIcon_Expanded) #neu           
            playlist = self.playlists[playlistname]
            playlen = len(playlist)
            for i in range(0, playlen, 2):
                itemlabel = '%s, %s' % (playlist[i], playlist[i+1])
                itemid = self.playlistwin.AppendItem(item, itemlabel, 1)
                self.playlistwin.SetPyData(itemid, i)   # save index as data for identifikation
                self.playlistwin.SetItemImage(itemid, self.playlistwin.mfileidx, wx.TreeItemIcon_Normal) #neu
                if plname == playlistname and itemlabel == selitemlabel:
                    retval = itemid
        self.playlistwin.SetExpansionState(explist)
        self.markActivePlaylist()
        return retval
    #-------------------------------------------------------------------------------------------

    def markActivePlaylist(self):
        nothingselected = True
        selplname = self.config['View']['selplname']
        if len(selplname):
            root = self.playlistwin.GetRootItem()
            if len(self.oldplname):
                (child, cookie) = self.playlistwin.GetFirstChild(root)
                while True:
                    if self.oldplname == self.playlistwin.GetItemText(child):
                        self.playlistwin.SetItemBackgroundColour(child, wx.SystemSettings_GetColour(wx.SYS_COLOUR_WINDOW))
                        nothingselected = False
                        break
                    (child, cookie) = self.playlistwin.GetNextChild(root, cookie)
        
            self.oldplname = selplname
            (child, cookie) = self.playlistwin.GetFirstChild(root)
            while True:
                if selplname == self.playlistwin.GetItemText(child):
                    self.playlistwin.SetItemBackgroundColour(child, self.hlcolor)
                    nothingselected = False
                    break
                (child, cookie) = self.playlistwin.GetNextChild(root, cookie)
        if nothingselected:
            try:
                self._mgr.GetPane('playwin').Caption('Playlists (00:00:00)[0]')
                self._mgr.Update()
            except:
                pass            
    #-------------------------------------------------------------------------------------------

    def onRemovePlayItem(self, event):
        if self.playlistitem != None:
            itemlabel = self.playlistwin.GetItemText(self.playlistitem)
            parentitem = self.playlistwin.GetItemParent(self.playlistitem)
            parentlabel = self.playlistwin.GetItemText(parentitem)
            if parentlabel == MYTOP25: return
            try:
                playlist = self.playlists[parentlabel]
                title = itemlabel[0 : -8]
                id_title = itemlabel[-6:]
                index =  self.playlistwin.GetPyData(self.playlistitem)
                del playlist[index]
                del playlist[index]
                itemlabel = '%s, %s' % (playlist[0], playlist[1])
            except:
                itemlabel = ''
                pass
            itemid = self.updatePlaylistWin(itemlabel, parentlabel)
            if itemid != None: self.playlistwin.SelectItem(itemid)
    #-------------------------------------------------------------------------------------------

    def onPlayProperties(self, event):
        if self.playlistitem != None:
            itemlabel = self.playlistwin.GetItemText(self.playlistitem)
            id_title = itemlabel[-6:]           
            win = mumapopup.PropertyPopup(self, self.config, self.mumadb, id_title, self.playlists)
            pos = self.playlistwin.ClientToScreen((0,0))  # pos is the left top edge of the win
            sz = self.playlistwin.GetSize()               # sz is the size of the win
            win.Position((pos[0] + sz[0] / 2, pos[1] - sz[1] / 2), (0, sz[1]))       
            win.Popup()            
    #-------------------------------------------------------------------------------------------
        
    def onSelChanged(self, event):
        titlecount = 0
        timesum = 0
        self.playlistitem = event.GetItem()
        try:        # this try block fixes a win32 problem
            label = self.playlistwin.GetItemText(self.playlistitem)
            parent = self.playlistwin.GetItemParent(self.playlistitem)
            if parent != self.playlistwin.GetRootItem():
                timesum = toSeconds(self.mumadb.getDurationById(label[-6:]))
                label = self.playlistwin.GetItemText(parent)
                titlecount = 1
                self.isplaylistitem = True
            else:
                self.isplaylistitem = False
                playlist = self.playlists[label]
                if len(playlist):
                    for i in range(1, len(playlist), 2):
                        timesum += toSeconds(self.mumadb.getDurationById(playlist[i]))
                        titlecount += 1
            self.config['View']['selplname'] = label
            self.markActivePlaylist()
        except:
            return
        self._mgr.GetPane('playwin').Caption('Playlists (%s)[%i]' % (toString(timesum), titlecount))
        self._mgr.Update()        
    #-------------------------------------------------------------------------------------------

    def onMoveItemUp(self, event):
        if self.playlistitem != None:
            itemlabel = self.playlistwin.GetItemText(self.playlistitem)
            parentitem = self.playlistwin.GetItemParent(self.playlistitem)
            parentlabel = self.playlistwin.GetItemText(parentitem)
            if parentlabel == MYTOP25: return
            try:
                playlist = self.playlists[parentlabel]
                title = itemlabel[0 : -8]
                id = itemlabel[-6:]
                index =  self.playlistwin.GetPyData(self.playlistitem)
                if index > 1:
                    del playlist[index]
                    del playlist[index]
                    playlist.insert(index - 2, title)
                    playlist.insert(index - 1, id)
            except:
                pass
            itemid = self.updatePlaylistWin(itemlabel, parentlabel)
            self.playlistwin.SelectItem(itemid)
    #-------------------------------------------------------------------------------------------

    def onMoveItemDown(self, event):
        if self.playlistitem != None:
            itemlabel = self.playlistwin.GetItemText(self.playlistitem)
            parentitem = self.playlistwin.GetItemParent(self.playlistitem)
            parentlabel = self.playlistwin.GetItemText(parentitem)
            if parentlabel == MYTOP25: return
            try:
                playlist = self.playlists[parentlabel]
                title = itemlabel[0 : -8]
                id = itemlabel[-6:]
                index =  self.playlistwin.GetPyData(self.playlistitem)
                if index < len(playlist):
                    del playlist[index]
                    del playlist[index]
                    playlist.insert(index + 2, title)
                    playlist.insert(index + 3, id)
            except:
                pass
            itemid = self.updatePlaylistWin(itemlabel, parentlabel)
            self.playlistwin.SelectItem(itemid)
    #-------------------------------------------------------------------------------------------

    def moveItem(self, dragindex, dropindex):
        if dragindex == dropindex or len(dragindex) < 2 or len(dropindex) < 2: return
        
        draglistindex, dragitemindex = dragindex
        droplistindex, dropitemindex = dropindex
        dragitemindex *= 2
        dropitemindex *= 2
        
        playlistnames = list(self.playlists.keys())
        playlistnames.sort()       
        draglistname = playlistnames[draglistindex]
        droplistname = playlistnames[droplistindex]
        if droplistname == MYTOP25: return
        draglist = self.playlists[draglistname]
        droplist = self.playlists[droplistname]
        
        dragtitle = draglist[dragitemindex]
        dragid_title = draglist[dragitemindex + 1]
        del draglist[dragitemindex]
        del draglist[dragitemindex]
        
        if draglistindex == droplistindex and dropitemindex > dragitemindex:  
            droplist.insert(dropitemindex - 2, dragid_title)
            droplist.insert(dropitemindex - 2, dragtitle)
        else:  # drop to other playlist or same playlist before dragindex
            droplist.insert(dropitemindex, dragid_title)
            droplist.insert(dropitemindex, dragtitle)
        
        itemid = self.updatePlaylistWin('%s, %s' % (dragtitle, dragid_title), droplistname)
        self.playlistwin.SelectItem(itemid)        
    #-------------------------------------------------------------------------------------------

    def onCheckList(self, event):
        self.sb.setLed(GREY)
        explist = self.playlistwin.GetExpansionState()  
        dlg = mumachecklist.CheckPlaylist(self, self.config, self.mumadb, self.playlists, self.onCheckMusicDir, self.onPlayTitle)
        dlg.ShowModal()
        self.updatePlaylistWin(plname = self.config['View']['selplname'])
        self.playlistwin.SetExpansionState(explist)
        self.sb.restoreLed()
    #-------------------------------------------------------------------------------------------
    
    def onManagePlaylists(self, event):
        if self.mumadb != None:
            self.mumadb.commitTransaction()
        dlg = mumaplayman.PlaylistsManDialog(self, self.config, self.updateQuerywin, self.writeToLogwin)
        dlg.ShowModal()
    #-------------------------------------------------------------------------------------------

    def onCheckTitle(self, event):
        self.sb.setLed(GREY)
        if self.listwin.GetSelectedItemCount():
            index = -1
            state = wx.LIST_STATE_SELECTED            
            index = self.listwin.GetNextItem(index, wx.LIST_NEXT_ALL, state)
            id_title = self.listwin.getColumnText(index, 0)
            if not isSpecialId(id_title):
                dlg = mumachecktitle.CheckTitle(self, id_title, self.config, self.mumadb, self.onCheckMusicDir, self.onPlayTitle, self.writeToLogwin, self.plogger)
                dlg.ShowModal()
                self.reQuery()
                try:
                    self.listwin.SetItemState(index, wx.LIST_STATE_SELECTED | wx.LIST_STATE_FOCUSED, wx.LIST_STATE_SELECTED | wx.LIST_STATE_FOCUSED)
                except:
                    pass
        self.sb.restoreLed()
    #-------------------------------------------------------------------------------------------

    def nextVolume(self, keycode):
        try:
            volumeid = self.listwin.GetItem(0).GetText()
            volint = int(volumeid)
        except:
            volint = self.lastvalidvolint
            
        if volint % 100 == 0:
            if keycode == 314:           # '<-'
                volint -= 100
            elif keycode == 316:         # '->'
                volint += 100
            self.showAlbumTitles(str(volint))
            self.lastvalidvolint = volint
    #-------------------------------------------------------------------------------------------

    def writeToLogwin(self, info, color = None, onlyfile = False):
        try:
            if not onlyfile:
                start = self.logwin.GetInsertionPoint()
                self.logwin.WriteText('%s\n' % (info))
                stop = self.logwin.GetInsertionPoint() - 1
                if color != None:
                    self.logwin.SetStyle(start, stop, wx.TextAttr(color))
                self.logwin.ShowPosition(-1)
            if self.config['Misc']['actionlog'] == 'True':
                self.alogger.info('%s' % (info))
        except:
            pass
    #-------------------------------------------------------------------------------------------

    def taskbarNotify(self, message):
        if notifyisthere and self.config['Misc']['notify'] == 'True':
            imguri = ICONPATH + os.sep + 'musicman.png'
            try:    # workaround for Mint XFCE 17.2
                notification = pynotify.Notification('%s - Version %s' % (TITLE, VERNUM), message, imguri)
                notification.set_urgency(pynotify.URGENCY_NORMAL)
                notification.set_timeout(5000) 
                notification.show()
            except:    
                pynotify.init(TITLE)
    #-------------------------------------------------------------------------------------------

    def checkMissing(self, musicdir):
        if self.firstrun:
            sql = 'SELECT Id_Title FROM muma_title WHERE Id_Title >= "100000" AND Id_Title <= "899999"' 
            self.missrecords, self.missmatchcount = self.mumadb.getMatches(sql)
            self.missrecnum = 0
            self.firstrun = False
            return True
            
        while self.missrecnum < self.missmatchcount:
            title_id = self.missrecords[self.missrecnum][0]
            self.missrecnum += 1
            if title_id % 100 == 0:     # this is a dir
                dirnum = str(title_id)[0:4]
                try:
                    dirlist = os.listdir(os.path.join(musicdir, dirnum[:2], dirnum))
                except:
                    dirlist = []
                file_ids = []
                for item in dirlist:
                    file_ids.append(item[0:6])
                while self.missrecnum < self.missmatchcount:
                    title_id = self.missrecords[self.missrecnum][0]
                    if title_id % 100 == 0: break
                    self.missrecnum += 1
                    if str(title_id) not in file_ids:
                        self.misslist.append(title_id)
                break # this dir is done, so wait for the next timer event                    
        return self.missrecnum < self.missmatchcount
    #-------------------------------------------------------------------------------------------

    def onCheckMusicDir(self, event = None):
        self.musicdirtimer.Stop()
        musicdir = self.config['Misc']['musicbase']
        if os.path.isdir(musicdir):
            if LINUX:
                st = os.statvfs(musicdir)
                self.sb.setGauge((st.f_blocks - st.f_bfree) * 100 / st.f_blocks)
            else:
                _, total, free = ctypes.c_ulonglong(), ctypes.c_ulonglong(), ctypes.c_ulonglong()
                if isinstance(musicdir, unicode) or sys.version_info >= (3,):
                    fun = ctypes.windll.kernel32.GetDiskFreeSpaceExW
                else:
                    fun = ctypes.windll.kernel32.GetDiskFreeSpaceExA
                ret = fun(musicdir, ctypes.byref(_), ctypes.byref(total), ctypes.byref(free))
                self.sb.setGauge((total.value - free.value) * 100 / total.value)
            
            if self.config['Misc']['missing'] == 'True' and self.checkMissing(musicdir):
                self.musicdirtimer.Start(250, oneShot = False)
            else:
                self.musicdirtimer.Start(30000, oneShot = False)
            return True
        else:
            if pyudevisthere and self.config['Misc']['automount'] == 'True':
                context = pyudev.Context()
                devicename = ''
                for device in context.list_devices(subsystem = 'block', DEVTYPE = 'partition'):
                    devicelabel = device.get('ID_FS_LABEL')
                    if devicelabel == self.config['Misc']['devlabel']:
                        devicename = device.get('DEVNAME')
                        break
                if len(devicename):
                    try:
                        cmdline = 'pmount -w %s %s' % (devicename, devicelabel)
                        process = subprocess.Popen(cmdline, shell = True, stdin = subprocess.PIPE, stdout = subprocess.PIPE, stderr = subprocess.PIPE)
                        process.wait()
                        self.devicename = devicename
                        if os.path.isdir(musicdir):
                            if LINUX:
                                st = os.statvfs(musicdir)
                                self.sb.setGauge((st.f_blocks - st.f_bfree) * 100 / st.f_blocks)
                            else:
                                _, total, free = ctypes.c_ulonglong(), ctypes.c_ulonglong(), ctypes.c_ulonglong()
                                if isinstance(musicdir, unicode) or sys.version_info >= (3,):
                                    fun = ctypes.windll.kernel32.GetDiskFreeSpaceExW
                                else:
                                    fun = ctypes.windll.kernel32.GetDiskFreeSpaceExA
                                ret = fun(musicdir, ctypes.byref(_), ctypes.byref(total), ctypes.byref(free))
                                self.sb.setGauge((total.value - free.value) * 100 / total.value)
                            if self.config['Misc']['missing'] == 'True':
                                self.musicdirtimer.Start(2000, oneShot = False)
                            else:
                                self.musicdirtimer.Start(30000, oneShot = False)
                            return True
                    except:
                        pass
            self.sb.setGauge(0)
            self.misslist = []
            self.musicdirtimer.Start(5000, oneShot = False)
            return False
    #-------------------------------------------------------------------------------------------

    def onOpenDb(self, event = None):
        """  """
        self.sb.setLed(GREY)
        wildcard = 'SQLite file (*.db3)|*.db3|' \
                   'All files (*.*)|*.*'
        dlg = mumadir.DirDialog(self, TITLE + ' - open database', self.config, 
                                self.config['Database']['pathname'], mode = 'openf', filters = wildcard)
        if dlg.ShowModal() == wx.ID_OPEN:
            path = os.path.normpath(dlg.GetFilePath())
            self.config['Database']['pathname'] = path
        dlg.Destroy()
        self.connect2Database()
    #-------------------------------------------------------------------------------------------

    def onBackupDb(self, event = None):        
        """  """
        self.sb.setLed(GREY)
        currentbkupstate = self.config['Misc']['multibkup']
        self.config['Misc']['multibkup'] = 'True'
        self.makeDbBackup()
        self.config['Misc']['multibkup'] = currentbkupstate
        self.sb.restoreLed()
    #-------------------------------------------------------------------------------------------

    def onRestoreDb(self, event):
        """  """
        wildcard = 'DB backup file (*.tar.gz)|*.tar.gz|' \
                   'All files (*.*)|*.*'
        
        self.sb.setLed(GREY)
        dbfname = self.config['Database']['pathname']
        head, tail = os.path.split(dbfname)        
        restname = self.config['Database']['bkuppath']
        dlg = mumadir.DirDialog(self, TITLE + ' - open database backup', self.config, 
                                restname, mode = 'openf', filters = wildcard)
        if dlg.ShowModal() == wx.ID_OPEN:
            tgzfilename = os.path.normpath(dlg.GetFilePath())
            
            try:
                tf = tarfile.open(tgzfilename, 'r:gz')
                member = tf.getmembers()[0]
                tf.extract(member, head)
                tf.close()
                self.writeToLogwin("DB-restore - file '%s' restored" % (dbfname))
                self.connect2Database()
            except:
                self.writeToLogwin("DB-restore - '%s' could not be restored" % (dbfname))
        dlg.Destroy()
        self.reQuery()
    #-------------------------------------------------------------------------------------------

    def onImportDb(self, event):
        """  """
        wildcard = 'DB file (*.db3)|*.db3|' \
                   'All files (*.*)|*.*'
        
        self.sb.setLed(GREY)
        dbfname = self.config['Database']['pathname']
        dbfhead, dbftail = os.path.split(dbfname)
        dlg = mumadir.DirDialog(self, TITLE + ' - open database to import', self.config, 
                                self.config['Database']['dbimportpath'], mode = 'openf', filters = wildcard)
        if dlg.ShowModal() == wx.ID_OPEN:
            currentlastupdate = self.mumadb.getLastUpdate()
            importfilename = os.path.normpath(dlg.GetFilePath())
            localdb = mumasqlite.MusicManDb(importfilename)
            newlastupdate = localdb.getLastUpdate()
            
            if currentlastupdate >= newlastupdate:
                msgdlg = wx.MessageDialog(self, "The selected database is NOT newer than the current db!\n%s \t\t\t %s\n\nImport selected database anyhow?" \
                                          % (newlastupdate, currentlastupdate),
                                          TITLE + ' - import database', wx.NO | wx.YES | wx.NO_DEFAULT | wx.ICON_INFORMATION)
                retval = msgdlg.ShowModal()
                msgdlg.Destroy()
                if retval != wx.ID_YES:
                    dlg.Destroy()
                    self.writeToLogwin("DB-import - '%s' is not imported" % (importfilename))
                    self.sb.restoreLed()
                    self.reQuery()
                    return
            
            self.onBackupDb() 
            self.config['Database']['dbimportpath'] = importfilename
            imphead, imptail = os.path.split(importfilename)
            self.mumadb.closeAll()          # release DB-file, if not Windows error
            self.mumadb = None
            self.sb.setLed(RED)
            try:
                newdbfname = os.path.join(dbfhead, imptail)
                copy_file(importfilename, newdbfname)
                self.config['Database']['pathname'] = newdbfname
                self.writeToLogwin("DB-import - file '%s' imported" % (importfilename))
                self.connect2Database()
            except:
                self.writeToLogwin("DB-import - '%s' could not be imported" % (importfilename))
        dlg.Destroy()
        self.reQuery()
    #-------------------------------------------------------------------------------------------

    def connect2Database(self):
        self.writeToLogwin('DB-connect - try to connect to database')
        self.prepareDisplay()

        if self.mumadb != None:
            self.mumadb.commitTransaction()
            self.mumadb = None
            self.sb.setLed(YELLOW)
        
        try:
            dbfilename = self.config['Database']['pathname']
            if not os.path.exists(dbfilename) and dbfilename.endswith('musicman.db3'):
                f = file(dbfilename, 'wb')    # create db file 
                f.close()
            try:
                os.chmod(dbfilename, stat.S_IREAD | stat.S_IWRITE)
            except:
                pass         
            self.mumadb = mumasqlite.MusicManDb(dbfilename)
            if not self.mumadb.checkDatabase()[0]:
                self.mumadb = None
                self.writeToLogwin("DB-connect - could connect to database '%s', but its not a MusicMan db" % (dbfilename), RED)
                self.updateDbName(None)
                dlg = wx.MessageDialog(self, "This is a db3 but not a MusicMan database!", TITLE + ' - open database', wx.OK | wx.ICON_ERROR)
                dlg.ShowModal()
                dlg.Destroy()
            else:
                self.updateDbName(self.mumadb.getRecordCount(self.mumadb.TAB_TITLE))
                self.writeToLogwin("DB-connect - database '%s' opened" % (dbfilename))
        except:
            self.writeToLogwin("DB-connect - could not connect to database '%s'" % (dbfilename), RED)
            self.sb.SetStatusText('not connected', 4)
            self.sb.setLed(RED, True)
    #-------------------------------------------------------------------------------------------

    def onManageQueries(self, event):
        self.sb.setLed(GREY)
        if self.mumadb != None:
            self.mumadb.commitTransaction()
        dlg = mumaqueryman.QueriesManDialog(self, self.config, self.mumadb, self.updateQuerywin, self.writeToLogwin)
        dlg.ShowModal()
        self.sb.restoreLed()
    #-------------------------------------------------------------------------------------------

    def updateRecords(self, recordcount = 0):
        self.sb.SetStatusText('R: %s' % (format3P(recordcount)), 3)
        self.sb.rePosition()
        # wx.SafeYield(self)  # assertion
    #-------------------------------------------------------------------------------------------

    def onColSize(self, event):
        for i in range(self.listwin.GetColumnCount()):
            self.querydata['colwidth'][i] = self.listwin.GetColumnWidth(i)
        self.config['Queries'][self.querydata['name']] = self.querydata
    #-------------------------------------------------------------------------------------------

    def prepareDisplay(self, columns = None, colwidth = None):
        if self.querydata != None:
            if columns == None:
                columns = self.querydata['columns']
                colwidth = self.querydata['colwidth']
            self.listwin.ClearAll()
        if columns != None:
            columncount = len(columns)
            for i in range(columncount):
                self.listwin.InsertColumn(i, columns[i], width = int(colwidth[i]))
            self.listwin.SetColumnCount(columncount)
    #-------------------------------------------------------------------------------------------

    def updateMatches(self, matchesread = 0, plus = False):
        if plus:
            self.sb.SetStatusText('M: 500+ [%s]' % (format3P(self.mumadb.getRecordCount(self.mumadb.TAB_TITLE))), 3)
        else:
            self.sb.SetStatusText('M: %s [%s]' % (format3P(matchesread),
                                                  format3P(self.mumadb.getRecordCount(self.mumadb.TAB_TITLE))), 3)
        self.sb.rePosition()
    #-------------------------------------------------------------------------------------------

    def onPreferences(self, event):
        """  """
        self.sb.setLed(GREY)
        self.handleConfigData('w')
        prefdlg = mumaprefs.PrefsDialog(self, self.mumadb, self.hlcolor, self.color99, self.altcolor, self.misscolor, self.inifilename)
        if prefdlg.ShowModal() == wx.ID_OK:
            self.config = prefdlg.getNewConfig()
            self.config.write()
            self.writeToLogwin('Preferences - new values set')
        else:
            self.config.reload()
        prefdlg.Destroy()
        self.myfont = wx.Font(int(self.config['ListCtrl']['fsize']),
                              wx.DEFAULT, wx.NORMAL, wx.NORMAL, False,
                              self.config['ListCtrl']['font'])
        self.listwin.SetFont(self.myfont)
        collist = self.config['Misc']['hlcolor']
        self.hlcolor = (int(collist[0]), int(collist[1]), int(collist[2]))
        collist = self.config['Misc']['99color']
        self.color99 = (int(collist[0]), int(collist[1]), int(collist[2]))
        collist = self.config['Misc']['altcolor']
        self.altcolor = (int(collist[0]), int(collist[1]), int(collist[2]))
        collist = self.config['Misc']['misscolor']
        self.misscolor = (int(collist[0]), int(collist[1]), int(collist[2]))
        self.listwin.setNewColor(self.config)        
        
        isexpert = self.config['Misc']['expert'] == 'True'
        self.mainmenu.Enable(self.ID_IMPORT, isexpert)
        self.mainmenu.Enable(self.ID_MOVE, isexpert)
        self.mainmenu.Enable(self.ID_ADDRECORD, isexpert)
        self.mainmenu.Enable(self.ID_EDITRECORD, isexpert)
        self.mainmenu.Enable(self.ID_REMOVERECORD, isexpert)
        self.mainmenu.Enable(self.ID_BATCHEDIT, isexpert)
        self.mainmenu.Enable(self.ID_CHECK, isexpert)
        self.mainmenu.Enable(self.ID_QUERYTAB, self.config['ListCtrl']['newtab'] == 'False')
        self.notebook.HideAllTabs((self.config['ListCtrl']['hidetab'] == 'True') and (len(self.tabdata) == 1))
        self._mgr.Update()
        self.reQuery()
        self.sb.restoreLed()
    #-------------------------------------------------------------------------------------------

    def onProperties(self, event):
        if self.listwin.GetSelectedItemCount():
            index = -1
            state = wx.LIST_STATE_SELECTED            
            index = self.listwin.GetNextItem(index, wx.LIST_NEXT_ALL, state)
            id_title = self.listwin.getColumnText(index, 0)
            if not isSpecialId(id_title):
                win = mumapopup.PropertyPopup(self, self.config, self.mumadb, id_title, self.playlists)
                pos = self.listwin.ClientToScreen((0,0))  # pos is the left top edge of the win
                sz = self.listwin.GetSize()               # sz is the size of the win
                win.Position((pos[0] + sz[0] / 2, pos[1] - sz[1] / 2), (0, sz[1]))       
                win.Popup()
    #-------------------------------------------------------------------------------------------

    def onQuick(self, event):
        if event.Id == self.ID_QUICK_TAB or \
           self.config['ListCtrl']['newtab'] == 'True': self.onAddQueryTab()
        
        self.Unbind(aui.EVT_AUINOTEBOOK_PAGE_CHANGING)
        self.querydata = self.config['Queries']['QuickQuery']      
        if self.mumadb != None:
            self.updateDbName()
            dlg = mumaparam.QuickParamDialog(self, self.config, self.mumadb, self.onCheckMusicDir, self.listwin, self.updateMatches, self.onPlayTitle, self.misslist)
            dlg.ShowModal()
            tabindex = self.notebook.GetPageIndex(self.notebook.GetCurrentPage())
            results = dlg.getValues()
            self.tabdata[tabindex].curquery = results[0]          
            self.config['Queries']['QuickQuery']['fuzzyhist'] = results[1]
            self.config['Queries']['QuickQuery']['titlehist'] = results[2]
            self.config['Queries']['QuickQuery']['interhist'] = results[3]
            self.querydata['sql'] = results[0] 
            self.tabdata[tabindex].querydata = self.querydata
            self.tabdata[tabindex].matchcount = results[5]
            self.updateMatches(results[5])
            
            dlg.Destroy()
            index = results[4]
            if index != None:
                self.listwin.SetItemState(index, wx.LIST_STATE_SELECTED | wx.LIST_STATE_FOCUSED, wx.LIST_STATE_SELECTED | wx.LIST_STATE_FOCUSED)
                self.listwin.EnsureVisible(index)
        self.Bind(aui.EVT_AUINOTEBOOK_PAGE_CHANGING, self.onQueryTabChange)
    #-------------------------------------------------------------------------------------------

    def onColumnSort(self, event):
        oldorderby = 'Id_Title'
        orderdir = 'ASC'
        col = event.Column
        neworderby = self.querydata['columns'][col]         
        currentquery = self.tabdata[self.notebook.GetPageIndex(self.notebook.GetCurrentPage())].curquery 
        
        if 'ORDER BY' in currentquery:
            newquery = currentquery[0 : currentquery.index('ORDER BY')]
            oldorderby = currentquery[currentquery.index('ORDER BY')+9:]
        elif 'LIMIT' in currentquery:
            newquery = currentquery[0 : currentquery.index('LIMIT')]
        else:
            newquery = currentquery
            
        if oldorderby.endswith('ASC'):
            orderdir = 'ASC'
            oldorderby = oldorderby[0:-4]
        elif oldorderby.endswith('DESC'):
            orderdir = 'DESC'
            oldorderby = oldorderby[0:-5]
            
        if neworderby == oldorderby:
            if orderdir == 'ASC':
                self.oldorderdir = 'DESC'
            else:
                self.oldorderdir = 'ASC'
        else:
            self.oldorderdir = 'ASC'
        self.oldorderby = neworderby
        newquery = '%sORDER BY %s %s' % (newquery, self.oldorderby, self.oldorderdir)
        self.writeToLogwin('Query - %s' % (newquery))
        self.listwin.UpdateImages(col, self.oldorderdir == 'ASC')
        
        self.sb.setLed(YELLOW)
        self.records, matchcount = self.mumadb.getMatches(newquery)        
        self.tabdata[self.notebook.GetPageIndex(self.notebook.GetCurrentPage())].curquery = newquery
        self.updateMatches(matchcount)
        self.listwin.setDataSource(self.records, matchcount, self.misslist)
        self.sb.setLed(GREEN)        
    #-------------------------------------------------------------------------------------------

    def showAlbumTitles(self, id_title):
        self.querydata = self.config['Queries']['AlbumTitles']
        self.querydata['sql'] = '%sLIKE "%s%%" ORDER BY Id_Title' % (self.querydata['sql'][0:self.querydata['sql'].index('LIKE')], id_title[0:4])
        self.onQuery(enterparam = False, showalbumtitles = True)
    #-------------------------------------------------------------------------------------------

    def onRefreshQuery(self, event):
        if self.listwin.GetColumnCount():
            self.reQuery() #onQuery(enterparam = False)
    #-------------------------------------------------------------------------------------------

    def onQuery(self, event = None, enterparam = True, showalbumtitles = False, volume_id = None):
        if event != None and self.config['ListCtrl']['newtab'] == 'True':
            self.onAddQueryTab()
        
        if event and not showalbumtitles:          
            try:
                self.querydata = self.config['Queries'][self.querywin.GetItemText(self.querywin.GetSelection())]
            except:
                self.querydata = None
        elif volume_id != None:
            self.querydata = self.config['Queries']['AddRecord']
            self.querydata['id_title'] = volume_id
        if self.mumadb != None:
            if self.querydata != None:
                self.tabdata[self.notebook.GetPageIndex(self.notebook.GetCurrentPage())].querydata = self.querydata
                self.updateDbName()
                query = self.querydata['sql']
                if event == None:
                    try:
                        ordquery = query[0 : query.index('ORDER')]
                    except:
                        try:
                            ordquery = query[0 : query.index('LIMIT')]
                        except:
                            ordquery = ''
                    query = '%sORDER BY %s %s' % (ordquery, self.oldorderby, self.oldorderdir)

                if '%param' in query:
                    if enterparam:
                        self.param = self.param1 = self.param2 = ''
                        if '%param%' in query:
                            dlg = mumaparam.ParamDialog(self, self.querydata['freewhere'], self.querydata['paramhist'])
                            if dlg.ShowModal() == wx.ID_OK:
                                self.querydata['paramhist'] = dlg.getValue()
                                self.param = self.querydata['paramhist'][0]
                                query = query.replace("%param%", self.param)
                                query = query.replace("%param%", self.param)
                            else:
                                self.writeToLogwin('Query - no parameter entered')
                                return
                        elif '%param1%' in query:
                            #if not self.querydata.has_key('paramhist2'):    # (Python 2)
                            if 'paramhist2' not in self.querydata:           # (Python 3)
                                self.querydata['paramhist2'] = ['']
                            dlg = mumaparam.ParamDialog(self, self.querydata['freewhere'], self.querydata['paramhist'], self.querydata['paramhist2'])
                            if dlg.ShowModal() == wx.ID_OK:
                                self.querydata['paramhist'] = dlg.getValue()
                                self.param1 = self.querydata['paramhist'][0]
                                self.param2 = self.querydata['paramhist2'][0]
                                query = query.replace("%param1%", self.param1)
                                query = query.replace("%param2%", self.param2)
                            else:
                                self.writeToLogwin('Query - no parameter entered')
                                return
                        else:
                            self.writeToLogwin('Query - parameter syntax error')
                            return
                        if len(self.param):
                            self.sb.SetStatusText("Parameter: %%param%% = '%s'" % (self.param), 4)
                        else:
                            self.sb.SetStatusText("Parameter: %%param1%% = '%s', %%param2%% = '%s'" % (self.param1, self.param2), 4)
                    else:
                        try:
                            self.param = self.querydata['paramhist'][0]
                        except:
                            pass
                        query = query.replace("%param%", self.param)
                        query = query.replace("%param%", self.param)
                        query = query.replace("%param1%", self.param1)
                        query = query.replace("%param2%", self.param2)
                elif '%volume_id%' in query and volume_id != None:
                    query = query.replace("%volume_id%", volume_id)
               
                self.writeToLogwin('Query - %s' % (query))
                self.sb.setLed(YELLOW)
                # wx.SafeYield(self)  # assertion
                self.records = None
                self.prepareDisplay()
                try:
                    if 'ORDER BY' in query:
                        orderby = query[query.find('ORDER BY')+9:]
                        if orderby.endswith('ASC'):
                            self.oldorderdir = 'ASC'
                            self.oldorderby = orderby[0:-4]
                        elif orderby.endswith('DESC'):
                            self.oldorderdir = 'DESC'
                            self.oldorderby = orderby[0:-5]
                        else:
                            self.oldorderby = orderby
                            self.oldorderdir = 'ASC'                    
                    
                    self.records, matchcount = self.mumadb.getMatches(query)
                    self.updateMatches(matchcount)
                    if not matchcount and showalbumtitles:
                        index = query.index('LIKE') + 6
                        id00 = int(query[index : index + 4])
                        record = [(id00 * 100, 'empty volume - no db entries', '', '', '', '', '')]
                        self.listwin.setDataSource(record, 1, self.misslist)
                    else:
                        self.listwin.setDataSource(self.records, matchcount, self.misslist)
                    self.sb.setLed(GREEN)
                    tabindex = self.notebook.GetPageIndex(self.notebook.GetCurrentPage())
                    self.tabdata[tabindex].curquery = query
                    self.tabdata[tabindex].matchcount = matchcount

                    if matchcount:
                        self.writeToLogwin('Query - query matches %i database entries' % matchcount)
                    else:
                        self.writeToLogwin('Query - query matches no database entries')
                except:
                    dlg = wx.MessageDialog(self, query + "\n\nis not a valid Sqlite statement. Please correct it.", TITLE + ' - Query', wx.OK | wx.ICON_ERROR)
                    self.sb.setLed(RED)
                    dlg.ShowModal()
            else:
                self.writeToLogwin("Query - can't query (nothing selected)")
        else:
            self.writeToLogwin("Query - can't query (no database connection)")
    #-------------------------------------------------------------------------------------------

    def reQuery(self):
        if self.mumadb != None:
            tabindex = self.notebook.GetPageIndex(self.notebook.GetCurrentPage())
            currentquery = self.tabdata[tabindex].curquery 
            self.writeToLogwin('Query - %s' % (currentquery))
            self.sb.setLed(YELLOW)
            wx.SafeYield(self)
            self.records = None
            try:
                self.records, matchcount = self.mumadb.getMatches(currentquery)
                self.updateMatches(matchcount)
                self.tabdata[tabindex].matchcount = matchcount
                self.listwin.setDataSource(self.records, matchcount, self.misslist)
                self.sb.setLed(GREEN)
                if matchcount:
                    self.writeToLogwin('Query - query matches %i database entries' % matchcount)
                else:
                    self.writeToLogwin('Query - query matches no database entries')
            except:
                dlg = wx.MessageDialog(self, currentquery + "\n\nis not a valid Sqlite statement. Please correct it.", TITLE + ' - Query', wx.OK | wx.ICON_ERROR)
                self.sb.setLed(RED)
                dlg.ShowModal()
        else:
            self.writeToLogwin("Query - can't query (no database connection)")
    #-------------------------------------------------------------------------------------------

    def onShowQueryInfo(self, event):
        ''' click in querywin '''
        item = event.GetItem()
        if item:
            label = self.querywin.GetItemText(item)
            win = mumapopup.QueryInfoPopup(self.querywin, self.config, label)
            
            pos = self.querywin.ClientToScreen((0,0))  # pos is the left top edge of the Backup task win
            sz = self.querywin.GetSize()               # sz is the size of the Backup task win
            win.Position((pos[0] + sz[0] / 2, pos[1] - sz[1] / 2), (0, sz[1]))       
            win.Popup()
    #-------------------------------------------------------------------------------------------

    def specialQueryWithNewTab(self, query):
        self.onAddQueryTab()
        self.querydata = self.config['Queries']['QuickQuery']
        oldquerydata = self.querydata
        self.querydata['sql'] = query
        self.prepareDisplay()
        self.onQuery(enterparam = False, showalbumtitles = False, volume_id = None)
        self.querydata = oldquerydata
    #-------------------------------------------------------------------------------------------
        
    def onQueryTOI(self, event):
        if self.listwin.GetSelectedItemCount():        
            index = self.listwin.GetNextItem(-1, wx.LIST_NEXT_ALL, wx.LIST_STATE_SELECTED)
            try:            
                title = self.listwin.getColumnText(index, self.querydata['columns'].index('Title'))
            except:
                title = ''
            try:
                interpreter = self.listwin.getColumnText(index, self.querydata['columns'].index('Interpreter'))
            except:
                interpreter = ''
          
            if title == '':
                query = 'SELECT * FROM muma_title WHERE Interpreter LIKE "%s" \
                         LIMIT 500' % (interpreter)
            elif interpreter == '':
                query = 'SELECT * FROM muma_title WHERE Title LIKE "%s" \
                         LIMIT 500' % (title)
            else:
                query = 'SELECT * FROM muma_title WHERE Title LIKE "%s" OR Interpreter LIKE "%s" \
                         LIMIT 500' % (title, interpreter)
            self.specialQueryWithNewTab(query)
    #-------------------------------------------------------------------------------------------

    def onOpenVolume(self, event):
        if self.listwin.GetSelectedItemCount():        
            index = self.listwin.GetNextItem(-1, wx.LIST_NEXT_ALL, wx.LIST_STATE_SELECTED)            
            try:            
                idtitle = self.listwin.getColumnText(index, self.querydata['columns'].index('Id_Title'))[:4]
            except:
                dlg = wx.MessageDialog(self, "Unable to open volume - no volume\ninfo column 'Id_Title' detected!", TITLE + ' - open volume', wx.OK | wx.ICON_INFORMATION)
                dlg.ShowModal()
                return
                        
            query = 'SELECT * FROM muma_title WHERE ID_Title LIKE "%s%%" ORDER BY Id_Title' % (idtitle)
            self.specialQueryWithNewTab(query)
    #-------------------------------------------------------------------------------------------
        
    def makeDbBackup(self, compact = False):
        maxbkupcount = int(self.config['Misc']['bkupcount'])
        if maxbkupcount == 0 and not compact: return
        
        nomultibkup = self.config['Misc']['multibkup'] == 'False'
        timestamp = strftime('%Y%m%d%H%M')
        sourcepath = self.config['Database']['pathname']  
        head, tail = os.path.split(sourcepath)
        head = self.config['Database']['bkuppath']
        basename, ext = os.path.splitext(tail)
        targetfname = os.path.join(head, '%s-%s.tar.gz'% (basename, timestamp))
        dbfname, dummy = os.path.splitext(tail)
        
        bkuplist = []
        try:
            dirlist = os.listdir(head)
        except:
            dlg = wx.MessageDialog(self, "Can't make DB backup - unreachable directory\n\n%s" % (head), TITLE + ' - backup Database', wx.OK | wx.ICON_WARNING)
            dlg.ShowModal()
            dlg.Destroy()
            dirlist = []
        
        for filename in dirlist:
            if filename.startswith(dbfname) and filename.endswith('tar.gz'):
                bkuplist.append(filename)
        bkuplist.sort(reverse = True)
        
        # do we have to delete old backups
        if len(bkuplist) >= maxbkupcount:
            for i in range(maxbkupcount - 1, len(bkuplist)):
                filepath = os.path.join(head, bkuplist[i])
                try:
                    os.remove(filepath)
                    self.writeToLogwin("DB-backup - old backup file '%s' removed" % (filepath))
                except:
                    self.writeToLogwin("DB-backup - could not remove old backup file '%s'" % (filepath))
        
        if nomultibkup and not compact:    
            for filename in bkuplist:
                filepath = os.path.join(head, filename)
                if filepath.startswith(targetfname[:-11]) and filepath.endswith('.tar.gz'):
                    return      # no new backup, if we made one today
        
        dlg = mumaparam.BackupModeless(self, compact)
        dlg.Show(True)
        wx.SafeYield(self)
        
        try:
            tf = tarfile.open(targetfname, 'w:gz')                           
            tf.add(sourcepath, tail)
            tf.close()
            self.writeToLogwin("DB-backup - file '%s' created" % (targetfname))
        except:
            self.writeToLogwin("DB-backup - could not create file '%s'" % (targetfname))
        
        dlg.Destroy()
    #-------------------------------------------------------------------------------------------
   
    def onCompactDb(self, event):
        self.sb.setLed(GREY)
        if self.mumadb != None:
            self.makeDbBackup(True)
            self.mumadb.compactDatabase()
            self.writeToLogwin("Compact DB - old database saved and compacted DB generated")
        self.sb.restoreLed()
    #-------------------------------------------------------------------------------------------

    def onAddRecord(self, event = None, editmode = False):
        if self.mumadb != None:
            self.Unbind(aui.EVT_AUINOTEBOOK_PAGE_CHANGING)
            self.makeDbBackup()
            old_querydata = self.querydata
            dlg = mumaedit.EditRecordDialog(self, self.config, self.listwin, self.mumadb, self.onCheckMusicDir, self.writeToLogwin, self.onQuery, self.onPlayTitle, self.plogger, edit = editmode)
            dlg.ShowModal()
            index, volume_id = dlg.getIndex()
            self.querydata = old_querydata
            self.reQuery()
            try:
                self.listwin.SetItemState(index, wx.LIST_STATE_SELECTED | wx.LIST_STATE_FOCUSED, wx.LIST_STATE_SELECTED | wx.LIST_STATE_FOCUSED)
                self.listwin.EnsureVisible(index)
            except:
                pass
            self.Bind(aui.EVT_AUINOTEBOOK_PAGE_CHANGING, self.onQueryTabChange)
    #-------------------------------------------------------------------------------------------

    def onEditRecord(self, event):
        if self.listwin.GetSelectedItemCount():
            self.onAddRecord(editmode = True)
    #-------------------------------------------------------------------------------------------

    def onRemoveRecord(self, event):    
        if self.listwin.GetSelectedItemCount() and self.mumadb != None:
            self.Unbind(aui.EVT_AUINOTEBOOK_PAGE_CHANGING)
            self.makeDbBackup()
            if self.config['ListCtrl']['confirm'] == 'True':
                dlg = mumadelrec.DeleteRecordDialog(self, self.config['ListCtrl']['filedel'] == 'True')
                if dlg.ShowModal() != wx.ID_REMOVE:
                    return
        
            filedel = dlg.getCheckBoxState()        
            index = -1
            while True:            
                index = self.listwin.GetNextItem(index, wx.LIST_NEXT_ALL, wx.LIST_STATE_SELECTED)
                if index < 0: break
                id_title = self.listwin.getColumnText(index, 0)
                self.mumadb.deleteRecordById(id_title)
                self.maintainTop25(id_title, remove = True)
                self.writeToLogwin("Remove - record with Id_Title = '%s' removed from db" % (id_title))
                if filedel:
                    found, pathname = findTitleExt(self.config, id_title)
                    try:
                        os.remove(pathname)
                        self.writeToLogwin("Remove - file '%s' removed from device" % (pathname))
                    except:
                        self.writeToLogwin("Remove - file '%s' could not be removed from device" % (pathname))
            # now update the new timesum
            query = 'SELECT Id_Title, Title, Duration FROM muma_title WHERE Id_Title LIKE "%s%%" ORDER BY Title' % (id_title[0:4])
            records, self.matchcount = self.mumadb.getMatches(query)
            timesum = 0
            for index, entrytuple in enumerate(records):
                str_id_title = str(entrytuple[0])
                if entrytuple[0] % 100: timesum += toSeconds(entrytuple[2])
            query = 'UPDATE muma_title SET Duration = "%s" WHERE Id_Title = "%s00"' % (toString(timesum), id_title[0:4])
            self.mumadb.execQuery(query)

            self.reQuery()
            self.Bind(aui.EVT_AUINOTEBOOK_PAGE_CHANGING, self.onQueryTabChange)            
    #-------------------------------------------------------------------------------------------

    def onBatchEdit(self, event):
        self.sb.setLed(GREY)
        if self.mumadb != None:
            self.makeDbBackup()
            tabindex = self.notebook.GetPageIndex(self.notebook.GetCurrentPage())
            currentquery = self.tabdata[tabindex].curquery            
            dlg = mumabatchedit.BatchEditDialog(self, self.config, self.mumadb, self.writeToLogwin, self.onQuery)
            dlg.ShowModal()
            self.tabdata[tabindex].curquery = currentquery
            self.reQuery()
        self.sb.restoreLed()
    #-------------------------------------------------------------------------------------------

    def addToHistory(self, title, id_title, update = True):
        history = self.config['Misc']['history']
        history.append(title)
        history.append(id_title)
        while len(history) > int(self.config['Misc']['maxhistlen']) * 2:
            del history[0]
            del history[0]
        if update: self.updateHistorywin()
    #-------------------------------------------------------------------------------------------      

    def copyConvert(self, filename, counter = 0, notify = True):        
        # what are source and target exts?
        id_title = filename[-10:-4]
        targetexts = ['mp3', 'ogg', 'wav', 'unchanged']
        targetext = targetexts[int(self.config['Misc']['musicformat'])]
        targetdir = self.config['Misc']['copydir']
        source, sourceext = os.path.splitext(filename)
        sourceext = sourceext[1:]        
        dummy, sname = os.path.split(filename)
        sname = sname.replace(sourceext, '')
        sourceext = sourceext.lower()  
        if self.config['Misc']['fileformat'] == '0':    # 0::= Id_Title
            targetname = sname + targetext
        else:
            if targetext != 'unchanged':
                targetname = '%s.%s' % (self.mumadb.getInterTitleById(id_title), targetext)
            else:
                targetname = '%s.%s' % (self.mumadb.getInterTitleById(id_title), sourceext)
                
        if self.config['Misc']['fileformat'] == '2':
            targetname = '%02d-%s' % (counter + 1, targetname)
        targetpath = os.path.join(targetdir, targetname)

        if os.path.exists(targetpath):
            if not self.overwriteall and not self.skipall:
                dlg = mumaparam.SkipOrOverwrite(self, targetpath)
                retval = dlg.ShowModal()                
                dlg.Destroy()
                if retval == 0:     # ID_SKIP
                    return ''
                elif retval == 1:   # ID_SKIPALL
                    self.skipall = True
                    return ''
                else:               # 2 == ID_OVER, 3 == ID_OVERALL
                    self.overwriteall = retval == 3
                    os.remove(targetpath)
            else:
                if self.skipall:
                    return ''
                else:
                    os.remove(targetpath)            

        # copy or convert?
        if notify:
            self.taskbarNotify('Copy or convert File(s) via avconv / ffmpeg\nthis background action can take some time ...')
   
        if sourceext == targetext or targetext == 'unchanged':        
            try:
                self.writeToLogwin('Copying file %s' % (filename))
                wx.SafeYield(self)
                if self.config['Misc']['fileformat'] == '0':    # 0::= Id_Title
                    shutil.copy(filename, targetdir)
                else:
                    shutil.copy(filename, targetpath)
            except:
                self.writeToLogwin('could not copy file %s' % (filename), RED)
        else:        
            # is ffmpeg present?
            ffmpeg = self.config['Misc']['ffmpeg']
            if not LINUX and not ffmpeg.lower().endswith('.exe'):
                ffmpeg = ffmpeg + '.exe'
            ffmpegfound, ffmpegpath = searchFile(ffmpeg)
            if not ffmpegfound:
                if notify:
                    dlg = wx.MessageDialog(self, "unable to find program 'avconv' / 'ffmeg'.\nConverter possibly not in searchpath!", TITLE + ' - start musicplayer', wx.OK | wx.ICON_ERROR)
                    dlg.ShowModal()
                return ''
           
            if (sourceext == 'ogg' or sourceext == 'wav') and targetext == 'mp3':
                convparam = FFMPEG2MP3
            elif (sourceext == 'mp3' or sourceext == 'wav') and targetext == 'ogg':
                convparam = FFMPEG2OGG
            else:
                convparam = ''
            if self.config['Misc']['dquotation'] == 'True':
                cmdline = '"%s" -i "%s" %s "%s"' % (ffmpegpath, filename, convparam, targetpath)
            else:
                cmdline = '%s -i "%s" %s "%s"' % (ffmpegpath, filename, convparam, targetpath)
            self.writeToLogwin('Converting file "%s" - this background action can take some time' % (filename))
            self.writeToLogwin('Command line = %s' % cmdline)
            
            process = subprocess.Popen(cmdline, shell = True, stdin = subprocess.PIPE, stdout = subprocess.PIPE, stderr = subprocess.PIPE)
            process.wait()
            stdoutdata, stderrdata = process.communicate()
            #if len(stderrdata):
            #    self.writeToLogwin('The media converter reports:\n' + stderrdata)
            #    wx.SafeYield(self)            
            meta, ismp3, bitrate, dummyduration = getTagInfo(self.config, mfilepath=filename) 
            if not targetext in ['mp3', 'ogg']: return ''            
        return targetname
    #-------------------------------------------------------------------------------------------      

    def onComputeDurationSum(self, event):
        index = -1
        timesum = 0
        titlecount = 0
        while True:
            index = self.listwin.GetNextItem(index, wx.LIST_NEXT_ALL, wx.LIST_STATE_SELECTED)
            if index < 0: break
            id_title = self.listwin.getColumnText(index, 0)
            if not isSpecialId(id_title):
                timesum += toSeconds(self.mumadb.getDurationById(id_title))
                titlecount += 1
  
        if titlecount != self.oldtitlecount or timesum != self.oldtimesum:
            self._mgr.GetPane('listwin').Caption('Query results (%s)[%i]' % (toString(timesum), titlecount))
            self._mgr.Update()
            self.oldtitlecount = titlecount
            self.oldtimesum = timesum
    #-------------------------------------------------------------------------------------------

    def onHistSelect(self, event):
        index = self.historywin.GetSelection() * 2
        id_title = self.config['Misc']['history'][index + 1]
        self._mgr.GetPane('histwin').Caption('Play history (%s)[1]' % (self.mumadb.getDurationById(id_title)))
        self._mgr.Update()
        
        win = mumapopup.PropertyPopup(self, self.config, self.mumadb, id_title, self.playlists)
        pos = self.historywin.ClientToScreen((0,0))  # pos is the left top edge of the win
        sz = self.historywin.GetSize()               # sz is the size of the win
        win.Position((pos[0] + sz[0] / 2, pos[1] - sz[1] / 2), (0, sz[1]))       
        win.Popup()                  
    #-------------------------------------------------------------------------------------------

    def onPlayAgain(self, event):
        index = self.historywin.GetSelection() * 2
        self.onPlayTitle(id = self.config['Misc']['history'][index + 1])
    #-------------------------------------------------------------------------------------------      

    def onQueueTitle(self, event):
        self.onPlayTitle(event = event, queue = True)
    #-------------------------------------------------------------------------------------------      

    def showDeviceHint(self, info):                    
        mb = self.GetMenuBar()
        submenuitem = mb.FindItemById(self.ID_LOGWIN)
        if not submenuitem.IsChecked():
            popupwin = mumapopup.UpdatePopup(self, 'Requested action is not possible!\nReason: ' + info)
            pos = self.notebook.ClientToScreen((0,0))  # pos is the left top edge of the win
            sz = self.notebook.GetSize()               # sz is the size of the win
            popupwin.Position((pos[0] + sz[0] / 2 - 150, pos[1] - 50 - sz[1] / 2), (0, sz[1]))
            popupwin.Popup()
    #-------------------------------------------------------------------------------------------      

    def onPlayTitle(self, event = None, id = None, copy = False, queue = False, top25 = True):
        if self.listwin.GetSelectedItemCount() or id != None:
            index = -1
            playlist = []
            titlelist = []
            timelist= []
            again = True
            state = wx.LIST_STATE_SELECTED
            
            while again:
                if id == None:                    
                    index = self.listwin.GetNextItem(index, wx.LIST_NEXT_ALL, state)
                    if index < 0: break
                    id_title = self.listwin.getColumnText(index, 0)
                    if id_title.endswith('00'):
                        if copy:
                            continue
                        elif index == 0 and type(event) == wx.CommandEvent:
                            state = wx.LIST_STATE_DONTCARE
                            continue
                        else:
                            self.showAlbumTitles(id_title)
                            return
                else:
                    again = False
                    id_title = id
                if id_title.startswith('99'): continue
                if not self.onCheckMusicDir():
                    self.writeToLogwin('Play title - music device not present')
                    self.showDeviceHint('music device not present')
                    return
                found, path = findTitleExt(self.config, id_title)
                if not found:
                    dlg = wx.MessageDialog(self, "'%s'\nhas not supported fileext or file does not exist" % (path) , TITLE + ' - find music file', wx.OK | wx.ICON_ERROR)
                    dlg.ShowModal()
                    continue
                elif int(id_title) in self.misslist:
                    self.misslist.remove(int(id_title))
                    self.reQuery()
                
                playlist.append('"%s"' % (path))
                titlelist.append(self.mumadb.getTitleById(id_title))
                timelist.append(self.mumadb.getDurationById(id_title))

            if len(titlelist) == 0: return
            if copy:
                self.overwriteall = False               # | wx.PD_APP_MODAL 
                self.skipall = False
                dlg = wx.ProgressDialog(TITLE + ' - copying / converting', 'please be patient, this operation can take some time ...',
                                        len(playlist), self, style = wx.PD_CAN_ABORT | wx.PD_APP_MODAL | wx.PD_AUTO_HIDE | wx.PD_ELAPSED_TIME)                                       
                for index, item in enumerate(playlist):                    
                    wx.SafeYield(dlg)
                    tname = self.copyConvert(item.strip('"'), index, index == 0)
                    if tname == '':
                        (keepgoing, skip) = dlg.Update(index + 1, 'work in progress ...')
                    else:
                        (keepgoing, skip) = dlg.Update(index + 1, "'%s' completed ..." % tname)
                    if keepgoing == False: break
                wx.SafeYield(dlg)
                dlg.Destroy()
                self.taskbarNotify('copying / converting of files finished ...')
            else:
                titlespath = ' '.join(playlist)
                playerlist = self.config['Misc']['player'][0].split(' -')
                player = playerlist[0]
                if not LINUX and not player.lower().endswith('.exe'):
                    player = player + '.exe'                    
                playerfound, playerpath = searchFile(player)
                if not playerfound:
                    dlg = wx.MessageDialog(self, "unable to execute command\n'" + player + "'\nMusicplayer possibly not in searchpath!", TITLE + ' - start musicplayer', wx.OK | wx.ICON_ERROR)
                    dlg.ShowModal()
                    return
               
                try:
                    playerpath = '%s -%s' % (playerpath, playerlist[1])
                except:
                    pass
                self.writeToLogwin("Play - playing title(s):")
                for i in range(len(playlist)):
                    self.writeToLogwin('Duration = %s - Title = "%s" - File = %s' % (timelist[i], titlelist[i], playlist[i]))
                    self.addToHistory(titlelist[i], playlist[i][-11:-5], i == len(playlist) - 1)
                    if top25 and self.config['Misc']['top25'] == 'True':
                        self.maintainTop25(playlist[i][-11:-5])
                                    
                msg = 'Now playing title(s):'
                for i in range(len(playlist)):
                    msg = '%s\n%s' % (msg, titlelist[i])                               
                self.taskbarNotify(msg)
                if self.config['Misc']['dquotation'] == 'True':
                    if queue:
                        cmdline = '"%s" -e %s' % (playerpath, titlespath) 
                    else:
                        cmdline = '"%s" %s' % (playerpath, titlespath)
                else:
                    if queue:
                        cmdline = '%s -e %s' % (playerpath, titlespath) 
                    else:
                        cmdline = '%s %s' % (playerpath, titlespath)                    
                self.writeToLogwin('Command line = %s' % cmdline)
                proc = subprocess.Popen(cmdline, shell = True, stdin = subprocess.PIPE, stdout = subprocess.PIPE, stderr = subprocess.PIPE)
        else:
            self.writeToLogwin('Play title - nothing selected')
    #-------------------------------------------------------------------------------------------

    def onQueueList(self, event):
        self.onPlayList(queue = True)
    #-------------------------------------------------------------------------------------------      

    def onPlayList(self, event = None, m3upath = '', copy = False, queue = False):    # play the List or Title(s) from a List        
        if len(m3upath) == 0:
            if self.playlistitem == None: return
            itemlabel = self.playlistwin.GetItemText(self.playlistitem)
            parentitem = self.playlistwin.GetItemParent(self.playlistitem)            
            try:
                if self.playlistwin.GetItemText(parentitem) != 'Playlists': # this is a title not a list
                    playlist = self.playlists[self.playlistwin.GetItemText(parentitem)]
                    title = itemlabel[0 : -8]
                    id_title = itemlabel[-6:]
                    self.onPlayTitle(id = id_title, copy = copy, queue = queue)
                    return
            except:     # .. self.playlistwin.GetItemText(parentitem) throws an exception on Windows
                pass    # if parentitem is the rootitem and the control has the style HIDE_ROOT
            
        # this is a list not a title
        selplname = self.config['View']['selplname']
        playlist = []
        titlelist = []
        timelist = []
        if self.onCheckMusicDir() and selplname != '':
            playlist = self.playlists[selplname]
            playlen = len(playlist)
            if len(m3upath):
                plfilename = os.path.normpath(m3upath)
            else:
                plfilename = os.path.normpath(HOMEDIR + '/MusicMan.m3u')        
            plfile = codecs.open(plfilename, 'w', 'utf8')

            for i in range(1, playlen, 2):
                id_title = playlist[i]
                found, path = findTitleExt(self.config, id_title)
                if found:
                    dbtitle = self.mumadb.getTitleById(id_title)
                    dbtime = self.mumadb.getDurationById(id_title)
                    if len(dbtitle):
                        title = 'Duration = %s - Title = "%s" - File = "%s"' % (dbtime, dbtitle, path)
                        plfile.write('%s\n' % path)
                        titlelist.append(title)
            plfile.close
            if len(m3upath): return
                
            if copy:
                self.overwriteall = False
                self.skipall = False
                dlg = wx.ProgressDialog(TITLE + ' - copying / converting', 'please be patient, this operation can take some time ...',
                                        len(titlelist), self.playlistwin, style = wx.PD_CAN_ABORT | wx.PD_AUTO_HIDE | wx.PD_ELAPSED_TIME)
                for index, item in enumerate(titlelist):
                    wx.SafeYield(dlg)
                    item = item[item.index(' - File') + 10 :]
                    tname = self.copyConvert(item.strip('"'), index, index == 0)
                    if tname == '':
                        (keepgoing, skip) = dlg.Update(index + 1, 'work in progress ...')
                    else:
                        (keepgoing, skip) = dlg.Update(index + 1, "working on '%s' completed ..." % tname)
                    if keepgoing == False: break
                wx.SafeYield(dlg)
                dlg.Destroy()
            else:
                playerlist = self.config['Misc']['player'][0].split(' -')
                player = playerlist[0]
                if not LINUX and not player.lower().endswith('.exe'):
                    player = player + '.exe'                    
                playerfound, playerpath = searchFile(player)                                           
                if not playerfound:
                    dlg = wx.MessageDialog(self, "unable to execute command\n'" + player + "'\nMusicplayer possibly not in searchpath!", TITLE + ' - start musicplayer', wx.OK | wx.ICON_ERROR)
                    dlg.ShowModal()
                    return

                try:
                    playerpath = '%s -%s' % (playerpath, playerlist[1])
                except:
                    pass                       
                self.writeToLogwin("Play - playing playlist file '%s' containing %s titles" % (plfilename, len(playlist) / 2))
                self.writeToLogwin("Play - playing title(s):")
           
                msg = 'Now playing playlist-titles:'
                lentitlelist = len(titlelist)
                if lentitlelist > 16:
                    dlg = wx.ProgressDialog(TITLE + " - play list", "collecting titles from playlist '%s'" % selplname,
                                            lentitlelist, self,
                                            style = wx.PD_ELAPSED_TIME | wx.PD_AUTO_HIDE)                                                       
                    for i in range(lentitlelist):
                        dlg.Update(i)
                        msg = '%s\n%s' % (msg, playlist[2 * i])
                        self.writeToLogwin(titlelist[i])
                        self.addToHistory(playlist[2 * i], playlist[2 * i + 1], i == lentitlelist - 1)
                        if self.config['Misc']['top25'] == 'True': self.maintainTop25(playlist[2 * i + 1])
                    dlg.Update(lentitlelist)
                    dlg.Destroy()
                else:
                    for i in range(lentitlelist):
                        msg = '%s\n%s' % (msg, playlist[2 * i])
                        self.writeToLogwin(titlelist[i])
                        self.addToHistory(playlist[2 * i], playlist[2 * i + 1], i == lentitlelist - 1)
                        if self.config['Misc']['top25'] == 'True': self.maintainTop25(playlist[2 * i + 1])
                self.taskbarNotify(msg)
                if self.config['Misc']['dquotation'] == 'True':
                    if queue:
                        cmdline = '"%s" -e "%s"' % (playerpath, plfilename) 
                    else:
                        cmdline = '"%s" "%s"' % (playerpath, plfilename)
                else:
                    if queue:
                        cmdline = '%s -e "%s"' % (playerpath, plfilename) 
                    else:
                        cmdline = '%s "%s"' % (playerpath, plfilename) 
                self.writeToLogwin('Command line = %s' % cmdline)
                proc = subprocess.Popen(cmdline, shell = True, stdin = subprocess.PIPE, stdout = subprocess.PIPE, stderr = subprocess.PIPE)    
        else:
            self.writeToLogwin('Play list - nothing selected or music device not present')
            self.showDeviceHint('nothing selected or music device not present')
    #-------------------------------------------------------------------------------------------

    def onNewPlaylist(self, event):
        dlg = mumatextentry.TextEntryDialog(self, 'Enter playlist name to add')
        if dlg.ShowModal() == wx.ID_OK:
            plname = dlg.getValue()
            if plname in list(self.playlists.keys()):
                dlg = wx.MessageDialog(self, "the choosen playlist name already exists!", TITLE + ' - new playlist', wx.OK | wx.ICON_ERROR)
                dlg.ShowModal()            
            elif len(plname):
                self.playlists[plname] = []
                self.config['View']['selplname'] = plname
                self.updatePlaylistWin(plname = plname)
    #-------------------------------------------------------------------------------------------

    def onAddToPlaylist(self, event = None, listwin = None):
        """ """
        if listwin == None: listwin = self.listwin
        selplname = self.config['View']['selplname']
        if selplname == MYTOP25: return
        if listwin.GetSelectedItemCount() and selplname != '':
            index = -1
            state = wx.LIST_STATE_SELECTED
            while True:
                index = listwin.GetNextItem(index, wx.LIST_NEXT_ALL, state)
                if index < 0: break
                id_title = listwin.getColumnText(index, 0)
                if id_title.endswith('00'):
                    if index == 0 and type(event) == wx.CommandEvent:
                        state = wx.LIST_STATE_DONTCARE
                        continue
                if id_title.startswith('99'): continue
                if id_title in self.playlists[selplname]:
                    dlg = wx.MessageDialog(self, "The selected title(s) is/are already in the playlist!\nAdd the title anyhow?", TITLE + ' - add to playlist', wx.YES | wx.NO | wx.NO_DEFAULT | wx.ICON_WARNING)
                    if dlg.ShowModal() != wx.ID_YES: return   
                title = listwin.getColumnText(index, 1)
                self.playlists[selplname].append(title)
                self.playlists[selplname].append(id_title)
                self.writeToLogwin("Playlist - entry '%s', '%s' added to playlist '%s'" % (id_title,
                                   title, selplname))
            if not id_title.startswith('99'):
                itemid = self.updatePlaylistWin('%s, %s' % (title, id_title), selplname)
                self.playlistwin.SelectItem(itemid)
        else:
            dlg = wx.MessageDialog(self, "you have no active playlist and/or no title selected", TITLE + ' - add to playlist', wx.OK | wx.ICON_WARNING)
            dlg.ShowModal()            
    #-------------------------------------------------------------------------------------------

    def unpickleMyTop25(self):        
        self.mytop25 = {}       # {id_title1: [count, date], id_title2: [count, date] ...}
        if MYTOP25 not in list(self.playlists.keys()):
            self.playlists[MYTOP25] = []
        else:            
            try:
                self.mytop25 = pickle.load(open(self.inifilename.replace('.ini', '.t25'), 'rb'))
                self.skipTimePeriode()
            except:
                self.playlists[MYTOP25] = []
        try:
            self.updatePlaylistWin()
        except:
            pass
    #-------------------------------------------------------------------------------------------

    def skipTimePeriode(self):
        sd = self.config['Version']['shutdown']
        notexec = datetime.date.today() - datetime.date(int(sd[0:4]), int(sd[5:7]), int(sd[8:10]))
        if notexec >= datetime.timedelta(7):    
            # change dates, because MusicMan is not executed for a long while
            for key in self.mytop25.keys():
                ds = self.mytop25[key][1]       # add notexec days
                self.mytop25[key][1] = (datetime.date(int(ds[0:4]), int(ds[5:7]), int(ds[8:])) + datetime.timedelta(notexec)).isoformat()
    #-------------------------------------------------------------------------------------------

    def maintainTop25(self, id_title, remove = False):
        if not remove:
            if id_title in self.mytop25:
                self.mytop25[id_title][0] += 2
                self.mytop25[id_title][1] = strftime('%Y-%m-%d')
            else:
                self.mytop25[id_title] = [2, strftime('%Y-%m-%d')]
        elif id_title in self.mytop25:
            del self.mytop25[id_title]
        
        # decrement counter of aged entries (not played the last 30 days) if there are more then 25 items
        if len(self.mytop25) > 25:
            today_30 = (datetime.date.today() - datetime.timedelta(30)).isoformat()
            for key in self.mytop25.keys():
                if self.mytop25[key][1] < today_30:     # entry older than a month?
                    if self.mytop25[key][0] == 1:       
                        del self.mytop25[key]           # delete if counter == 1
                    else:
                        self.mytop25[key][0] -= 1       # give a chance
                        ds = self.mytop25[key][1]       # add 30 days
                        self.mytop25[key][1] = (datetime.date(int(ds[0:4]), int(ds[5:7]), int(ds[8:])) + datetime.timedelta(30)).isoformat()

        # now sort the mytop25 dict by value of count 
        smytop25 = sorted(self.mytop25.iteritems(), key = lambda (k,v): (v[0],k), reverse = True)
        
        # fill playlist My Top 25
        playlist25 = []
        count = 0
        for item in smytop25:
            title = self.mumadb.getTitleById(item[0])
            if len(title) == 0: continue                # title no longer exists
            playlist25.append(title)
            playlist25.append(item[0])
            count += 1
            if count == 25: break
        
        self.playlists[MYTOP25] = playlist25
        if self.playlistitem != None:    
            itemlabel = self.playlistwin.GetItemText(self.playlistitem)
            parentid = self.playlistwin.GetItemParent(self.playlistitem)
            try:
                parentlabel = self.playlistwin.GetItemText(parentid)
                if parentlabel != 'Playlists':
                    itemid = self.updatePlaylistWin(itemlabel, parentlabel)
                    if itemid != None: self.playlistwin.SelectItem(itemid)
            except:
                pass
        else:
            self.updatePlaylistWin()
    #-------------------------------------------------------------------------------------------      

    def onSavePlaylist(self, event):
        selplname = self.config['View']['selplname']
        if selplname != '':
            dlg = mumadir.DirDialog(self, TITLE + ' - save Matches as csv file', self.config,                                   
                        self.config['Misc']['m3upath'], selplname + '.m3u',
                        mode = 'save', filters = WILDM3U)        
            if dlg.ShowModal() == wx.ID_SAVE:                             
                m3upath = os.path.normpath(dlg.GetSaveFilePath())
                dirpart, dummy = os.path.split(m3upath)
                self.config['Misc']['m3upath'] = dirpart
                self.onPlayList(m3upath = m3upath)
                self.writeToLogwin("Playlist - saved to file '%s'" % (m3upath))
        else:
            dlg = wx.MessageDialog(self, "you have no playlist selected", TITLE + ' - save playlist', wx.OK | wx.ICON_WARNING)
            dlg.ShowModal()                            
    #-------------------------------------------------------------------------------------------

    def onExportPlaylist(self, event):
        selplname = self.config['View']['selplname']
        if selplname != '':
            wildcard = 'Playlist file (*.play)|*.play|' \
                       'All files (*.*)|*.*'
            fname = selplname + '.play'
            dlg = mumadir.DirDialog(self, TITLE + ' - export playlist', self.config, 
                                self.config['Misc']['expplpath'], fname,
                                mode = 'save', filters = wildcard)
            if dlg.ShowModal() == wx.ID_SAVE:
                path = os.path.normpath(dlg.GetSaveFilePath())
                playlist = ', '.join(self.playlists[selplname])
                playlist = '%s = %s' % (selplname, playlist)
                try:
                    plfile = codecs.open(path, 'w', 'utf8')
                    plfile.write(playlist)
                    plfile.close()
                    head, tail = os.path.split(path)
                    self.config['Misc']['expplpath'] = head
                except:
                    msgdlg = wx.MessageDialog(self, "unable to create file", TITLE + ' - export playlist', wx.OK | wx.ICON_ERROR)
                    msgdlg.ShowModal()                            
                    msgdlg.Destroy()                   
            dlg.Destroy()
    #-------------------------------------------------------------------------------------------

    def onImportPlaylist(self, event):
        wildcard = 'Playlist file (*.play)|*.play|' \
                   'All files (*.*)|*.*'
        dlg = mumadir.DirDialog(self, TITLE + ' - import and add playlist', self.config, 
                                self.config['Misc']['impplpath'], mode = 'openf',
                                filters = wildcard)
        if dlg.ShowModal() == wx.ID_OPEN:
            path = os.path.normpath(dlg.GetFilePath())
            plbuf = None
            try:
                plfile = codecs.open(path, 'r', 'utf8')
                playlist = plfile.read()
                plfile.close()
                head, tail = os.path.split(path)
                self.config['Misc']['impplpath'] = head
            except:
                msgdlg = wx.MessageDialog(self, "unable to read file", TITLE + ' - import and add playlist', wx.OK | wx.ICON_ERROR)
                msgdlg.ShowModal()                            
                msgdlg.Destroy()
            if playlist != None:
                plsplit = playlist.split(' = ')
                plname = plsplit[0]
                items = plsplit[1]
                if not plname in self.playlists:
                    newplist = []
                    itemlist = items.split(', ')
                    for item in itemlist:
                        newplist.append(item)
                    self.playlists[plname] = newplist
                else:
                    msgdlg = wx.MessageDialog(self, "unable to import - a list with this name exists", TITLE + ' - import and add playlist', wx.OK | wx.ICON_ERROR)
                    msgdlg.ShowModal()                            
                    msgdlg.Destroy()
                self.updatePlaylistWin()        
        dlg.Destroy()               
    #-------------------------------------------------------------------------------------------

    def onRemovePlaylist(self, event):
        selplname = self.config['View']['selplname']
        if selplname == MYTOP25: return
        playlen = len(self.playlists[selplname]) / 2
        dlg = wx.MessageDialog(self, "do you really want to delete the entire playlist '%s' with all (%i) collected titles?" % (selplname, playlen), TITLE + ' - remove playlist', wx.NO | wx.YES | wx.NO_DEFAULT | wx.ICON_QUESTION)
        if dlg.ShowModal() != wx.ID_YES:
            return
        
        if selplname != '':
            del self.playlists[selplname]
            self.writeToLogwin("Playlist - removed playlist '%s'" % (selplname))
            self.config['View']['selplname'] = ''
            self.oldplname = ''
            self.updatePlaylistWin()
        else:
            dlg = wx.MessageDialog(self, "you have no playlist selected", TITLE + ' - remove playlist', wx.OK | wx.ICON_WARNING)
            dlg.ShowModal()                                   
    #-------------------------------------------------------------------------------------------

    #def onMountMan(self, event):
    #    dlg = mumamountman.MountManDialog(self, self.config, self.writeToLogwin)
    #    dlg.ShowModal()
    #-------------------------------------------------------------------------------------------

    def onAbout(self, event):
        self.records, matchcount = self.mumadb.getMatches('SELECT sqlite_Version()')
        try:
            sqliteversion = self.records[0][0]
        except:
            sqliteversion = '???'
        dlg = mumaabout.AboutDialog(self, sqliteversion)
        dlg.ShowModal()
    #-------------------------------------------------------------------------------------------

    def onCopy(self, event = None, listwin = None):
        """  """
        self.sb.setLed(GREY)
        dlg = mumadir.DirDialog(self, TITLE + ' - create / select destination dir', self.config, 
                                self.config['Misc']['copydir'], mode = 'copy')
        if dlg.ShowModal() != wx.ID_CANCEL:
            self.config['Misc']['copydir'] = dlg.GetPath()
            oldlistwin = self.listwin
            if listwin != None: self.listwin = listwin
            self.onPlayTitle(copy = True)
            self.listwin = oldlistwin
        self.sb.restoreLed()
    #-------------------------------------------------------------------------------------------

    def onCopyList(self, event):
        """  """
        self.sb.setLed(GREY)
        dlg = mumadir.DirDialog(self, TITLE + ' - select / create destination dir', self.config, 
                                self.config['Misc']['copydir'], mode = 'copy')
        if dlg.ShowModal() != wx.ID_CANCEL:
            self.config['Misc']['copydir'] = dlg.GetPath()
            self.onPlayList(copy = True)
        self.sb.restoreLed()
    #-------------------------------------------------------------------------------------------

    def onExport(self, event):
        """  """
        from . import csvwriter
        
        self.sb.setLed(GREY)
        csvpath = self.config['Misc']['csvname']
        dirpart, filepart = os.path.split(csvpath)
        dlg = mumadir.DirDialog(self, TITLE + ' - save Matches as csv file', self.config, 
                        dirpart, filepart, mode = 'save', filters = WILDCSV)        
        if dlg.ShowModal() != wx.ID_SAVE:
            dlg.Destroy()
            self.sb.restoreLed()
            return                    
        csvpath = os.path.normpath(dlg.GetSaveFilePath())
        if os.path.exists(csvpath):
            dlg = wx.MessageDialog(self, "You have selected an existing csv file. Are you sure\n you want to replace it with an new csv file?", TITLE + ' - save csv file', wx.NO | wx.YES | wx.ICON_WARNING)
            if dlg.ShowModal() != wx.ID_YES: return
            os.remove(csvpath)    # remove old csv file
        dlg.Destroy()
        self.config['Misc']['csvname'] = csvpath
        self.sb.setLed(YELLOW)
        try:
            outfile = file(csvpath, 'w') 
        except:
            dlg = wx.MessageDialog(self, "The selected path is incorrect or check your rights!", TITLE + ' - save csv file', wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()
            self.sb.restoreLed()
            return
        writer = csvwriter.UnicodeCsvWriter(outfile)
        itemcount = self.listwin.GetItemCount()
        for index in range(itemcount):
            item = self.listwin.GetItem(index, 0)        
            row = [item.GetText()]
            for colnum in range(1, len(self.querydata['columns'])):
                item = self.listwin.GetItem(index, colnum)        
                row.append(item.GetText())
            writer.writerow(row)
        outfile.close()
        self.writeToLogwin("Export - %i records written to file '%s'" % (itemcount, csvpath))
        self.sb.restoreLed()
    #-------------------------------------------------------------------------------------------

    def onImport(self, event):
        """  """
        self.sb.setLed(GREY)
        self.makeDbBackup()
        dlg = mumaimport.ImportDialog(self, self.writeToLogwin, self.config, self.mumadb)
        dlg.ShowModal()
        dlg.Destroy()
        self.reQuery()
    #-------------------------------------------------------------------------------------------

    def onReplace(self, event):
        self.sb.setLed(GREY)
        if self.onCheckMusicDir():
            dlg = mumareplace.ReplaceDialog(self, self.config, self.writeToLogwin)
            dlg.ShowModal()
        else:
            info = 'Music basedir is not reachable'
            self.writeToLogwin('Replace - ' + info)
            self.showDeviceHint(info)
        self.sb.restoreLed()
    #-------------------------------------------------------------------------------------------
    def onMove(self, event):
        self.sb.setLed(GREY)
        if self.mumadb != None:
            self.onBackupDb()
        else:
            dlg = wx.MessageDialog(self, "No database selected / available!\nPlease correct the problem.", TITLE + ' - Move files', wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()
            return
        
        if self.onCheckMusicDir():
            dlg = mumamove.MoveDialog(self, self.writeToLogwin, self.config, self.mumadb)
            dlg.ShowModal()
        else:
            info = 'Music basedir is not reachable'
            self.writeToLogwin('Move - ' + info)
            self.showDeviceHint(info)
        self.reQuery()
    #-------------------------------------------------------------------------------------------

    def onSynchronize(self, event):
        """  """
        self.sb.setLed(GREY)
        dlg = mumasync.SyncDialog(self, self.config, self.writeToLogwin, self.makeDbBackup, self.mumadb)
        dlg.ShowModal()
        self.sb.restoreLed()
    #-------------------------------------------------------------------------------------------

    def onPrint(self, event):
        if cherryisthere:
            self.sb.setLed(GREY)
            self.writeToLogwin('Print - preparing page(s)')
            headerfile = codecs.open(os.path.join(APPPATH, 'mumaheader.html'), 'w', 'utf8')
            headerfile.write('''<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN" "http://www.w3.org/TR/REC-html40/strict.dtd">
<html>
    <head>
        <title>MusicMan Query results</title>
        <LINK rel="SHORTCUT ICON" href="musicmanimages/musicman.ico">
        <meta http-equiv="Content-Type" content="text/html; charset=iso-8859-1">
    </head>

    <body>
        <table  width="100%" border="1">
            <tr>''')
            for colname in self.querydata['columns']:                
                headerfile.write('<th align="left">%s</th>\n' % (colname))            
            headerfile.write('</tr>\n')
            headerfile.close()                 
            templatefile = codecs.open(os.path.join(APPPATH, 'mumatemplate.html'), 'w', 'utf-8')
            templatefile.write('<tr>\n')
            for colname in self.querydata['columns']:
                templatefile.write('<td><py-eval="%s"></td>\n' % (colname)) 
            templatefile.write('<tr>\n')
            templatefile.close()

            template = renderTemplate(file = os.path.join(APPPATH, 'mumaheader.html')) 

            itemcount = self.listwin.GetItemCount()
            for rownum in range(itemcount):
                for colnum in range(len(self.querydata['columns'])):
                    value = self.listwin.GetItem(rownum, colnum).GetText()
                    colname = self.querydata['columns'][colnum]
                    if colname == 'Id_Title':
                        Id_Title = value
                    elif colname == 'Title':
                        Title = value
                    elif colname == 'Interpreter':
                        Interpreter = value
                    elif colname == 'Duration':
                        Duration = value
                    elif colname == 'Date':
                        Date = value
                    elif colname == 'Code':
                        Code = value
                    else:
                        Composer = value
                template += renderTemplate(file = os.path.join(APPPATH, 'mumatemplate.html'))

            template += renderTemplate(file = os.path.join(APPPATH, 'mumafooter.html'))
            printData = self.printhtml.GetPrintData()
            printData.SetOrientation(wx.LANDSCAPE)
            self.printhtml.PageSetup()
            self.printhtml.SetHeader(VERSION)
            self.printhtml.SetFooter('Page @PAGENUM@ of @PAGESCNT@')
            self.printhtml.PreviewText(template)
        else:
            dlg = wx.MessageDialog(self, "Printing is impossible. Please install\nthe 'cherrytemplate' module first.", TITLE + ' - Print matches', wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()
        self.sb.restoreLed()
    #-------------------------------------------------------------------------------------------

    def onWindowTLC(self, event):
        self.config['Window']['posx'] = 0
        self.config['Window']['posy'] = 0
        self.config['Window']['aerposx'] = 0
        self.config['Window']['aerposy'] = 0
        self.config['Window']['beposx'] = 0
        self.config['Window']['beposy'] = 0
        self.config['Window']['impposx'] = 0
        self.config['Window']['impposy'] = 0
        self.config['Window']['synposx'] = 0
        self.config['Window']['synposy'] = 0
        self.config['Window']['qmaposx'] = 0
        self.config['Window']['qmaposy'] = 0
        self.config['Window']['cptposx'] = 0
        self.config['Window']['cptposy'] = 0
        self.config['Window']['cplposx'] = 0
        self.config['Window']['cplposy'] = 0
        self.config['Window']['qqposx'] = 0
        self.config['Window']['qqposy'] = 0
        self.config['Window']['reposx'] = 0
        self.config['Window']['reposy'] = 0
        self.config['Window']['dirposx'] = 0
        self.config['Window']['dirposy'] = 0
        windowrect = self.GetRect()
        windowrect[0] = 0
        windowrect[1] = 0
        self.SetRect(windowrect)
    #-------------------------------------------------------------------------------------------

    def setWindowDefaults(self):
        self.config['Window']['posx'] = 0
        self.config['Window']['posy'] = 0
        self.config['Window']['sizex'] = 1024
        self.config['Window']['sizey'] = 768
        self.config['Window']['aerposx'] = 0
        self.config['Window']['aerposy'] = 0
        self.config['Window']['beposx'] = 0
        self.config['Window']['beposy'] = 0
        self.config['Window']['impposx'] = 0
        self.config['Window']['impposy'] = 0
        self.config['Window']['impsizex'] = 520
        self.config['Window']['impsizey'] = 500                
        self.config['Window']['synposx'] = 0
        self.config['Window']['synposy'] = 0
        self.config['Window']['qmaposx'] = 0
        self.config['Window']['qmaposy'] = 0
        self.config['Window']['qmasizex'] = 700
        self.config['Window']['qmasizey'] = 500
        self.config['Window']['cptposx'] = 0
        self.config['Window']['cptposy'] = 0
        self.config['Window']['cptsizex'] = 480
        self.config['Window']['cptsizey'] = 380
        self.config['Window']['cplposx'] = 0
        self.config['Window']['cplposy'] = 0
        self.config['Window']['cplsizex'] = 800
        self.config['Window']['cplsizey'] = 520
        self.config['Window']['qqposx'] = 0
        self.config['Window']['qqposy'] = 0
        self.config['Window']['qqsizex'] = 450
        self.config['Window']['qqsizey'] = 480
        self.config['Window']['reposx'] = 0
        self.config['Window']['reposy'] = 0
        self.config['Window']['resizex'] = 500
        self.config['Window']['resizey'] = 500
        self.config['Window']['dirposx'] = 0
        self.config['Window']['dirposy'] = 0
        self.config['Window']['dirsizex'] = 450
        self.config['Window']['dirsizey'] = 450
        self.config['Window']['movposx'] = 0
        self.config['Window']['movposy'] = 0
    #-------------------------------------------------------------------------------------------
        
    def handleConfigData(self, rw = 'r'):
        if rw == 'r' or rw == 'i':               # read  / init ini file
            inipath = HOMEDIR
            if len(sys.argv) > 1:
                argument = sys.argv[1].lower()
                if argument.startswith('-i='):
                    inipath = sys.argv[1][3:]
            self.inifilename = os.path.join(inipath, TITLE + '.ini')
            self.config = ConfigObj(self.inifilename, encoding = 'UTF8')
#           if not self.config.has_key('Version'):              # new, empty ini file (Python 2)
            if rw == 'i' or 'Version' not in self.config:       # new, empty ini file (Python 3)
                # initialize all values
                self.config.initial_comment = ['MusicMan preferences file']
                self.config['Version'] = {}
                self.config['Version']['vernum'] = VERNUM
                self.config['Version']['startup'] = strftime('%Y-%m-%d %H:%M:%S')
                self.config['Version']['shutdown'] = strftime('%Y-%m-%d %H:%M:%S')
                self.config['Window'] = {}
                self.setWindowDefaults()
                self.config['Window']['qqcolwidth'] = ['65', '200', '200', '150']
                self.config['Window']['perspective'] = ''
                self.config['Window']['defaultperspective'] = ''
                self.config['Window']['tcexpstate'] = []
                self.config['Window']['expanstate'] = []
                self.config['View'] = {}                
                self.config['View']['querywin'] = 'True'
                self.config['View']['playwin'] = 'True'
                self.config['View']['histwin'] = 'True'
                self.config['View']['logwin'] = 'True'
                self.config['View']['selplname'] = ''
                self.config['Database'] = {}
                self.config['Database']['pathname'] = os.path.normpath(HOMEDIR + '/musicman.db3')
                self.config['Database']['dbimportpath'] = os.path.normpath(HOMEDIR)
                self.config['Database']['bkuppath'] = os.path.normpath(HOMEDIR)
                self.config['Database']['lastedit'] = ['', '', '', '', '', '', '' ]
                self.config['Database']['addwarn'] = 'True'
                self.config['Database']['emptydt'] = 'False'
                self.config['Database']['currentquery'] = ''
                self.config['Database']['lastqname'] = 'AlbumTitles'
                self.config['Database']['qqlimit'] = '500'
                self.config['ListCtrl'] = {}
                self.config['ListCtrl']['grid'] = 'True'
                self.config['ListCtrl']['font'] = wx.SystemSettings_GetFont(wx.SYS_ANSI_VAR_FONT).GetFaceName()
                self.config['ListCtrl']['fsize'] = '9'
                self.config['ListCtrl']['confirm'] = 'True'
                self.config['ListCtrl']['filedel'] = 'False'
                self.config['ListCtrl']['newtab'] = 'True'
                self.config['ListCtrl']['hidetab'] = 'True'
                self.config['Queries'] = initQueryExample()
                self.config['Misc'] = {}
                self.config['Misc']['splash'] = 'True'
                self.config['Misc']['instance'] = 'True'
                self.config['Misc']['notify'] = 'True'
                self.config['Misc']['multibkup'] = 'True'
                self.config['Misc']['bkupcount'] = '10'
                self.config['Misc']['actionlog'] = 'False'
                self.config['Misc']['update'] = 'True'
                self.config['Misc']['expert'] = 'False'
                self.config['Misc']['norm'] = 'False'
                self.config['Misc']['hlcolor'] = (194, 226, 194)
                self.config['Misc']['99color'] = (210, 210, 210)
                self.config['Misc']['altcolor'] = (226, 228, 230)
                self.config['Misc']['misscolor'] = (235, 192, 178)
                self.config['Misc']['mfileexts'] = ['mp3', 'ogg', 'wav']
                self.config['Misc']['musicbase'] = os.path.normpath(HOMEDIR)
                self.config['Misc']['tmusicbase'] = os.path.normpath(HOMEDIR)
                self.config['Misc']['player'] = ['qmmp', 'xmms -p', 'MusicMan', 'winamp.exe', 'foobar.exe']
                self.config['Misc']['ffmpeg'] = 'avconv'
                self.config['Misc']['csvname'] = os.path.normpath(HOMEDIR + '/MusicMan.csv')
                self.config['Misc']['copydir'] = os.path.normpath(HOMEDIR)
                self.config['Misc']['replacedir'] = os.path.normpath(HOMEDIR) 
                self.config['Misc']['titlecsv'] = os.path.normpath(HOMEDIR+ '/title.csv')
                self.config['Misc']['placecsv'] = os.path.normpath(HOMEDIR+ '/place.csv')
                self.config['Misc']['m3upath'] = os.path.normpath(HOMEDIR)
                self.config['Misc']['expplpath'] = os.path.normpath(HOMEDIR)
                self.config['Misc']['impplpath'] = os.path.normpath(HOMEDIR)
                self.config['Misc']['history'] = []
                self.config['Misc']['maxhistlen'] = '25'
                self.config['Misc']['musicformat'] = '0'        # 0 ::= mp3
                self.config['Misc']['fileformat'] = '0'         # 0 ::= ID_Title.ext
                self.config['Misc']['dquotation'] = 'True'
                self.config['Misc']['top25'] = 'True'
                self.config['Misc']['hiddendirs'] = 'False'
                self.config['Misc']['devlabel'] = 'MyDrive'
                self.config['Misc']['automount'] = 'True'
                self.config['Misc']['besearch'] = ''
                self.config['Misc']['becolindex'] = 'Title'
                self.config['Misc']['bereplace'] = ''
                self.config['Misc']['rangelow'] = '1000'
                self.config['Misc']['rangehigh'] = '8999'
                self.config['Misc']['thresht'] = '85'
                self.config['Misc']['threshi'] = '92'
                self.config['Misc']['berlow'] = '1000'
                self.config['Misc']['berhigh'] = '8999'
                self.config['Misc']['tabselect'] = '0'
                self.config['Misc']['missing'] = 'False'
            elif self.config['Version']['vernum'] < '6.47-3':                       # added in 6.45  (Python 3)
                self.config['Misc']['thresht'] = '85'
                self.config['Misc']['threshi'] = '92'
                if self.config['Version']['vernum'] < '6.45-3':                     # added in 6.45  (Python 3)
                    self.config['Database']['qqlimit'] = '500'
                if self.config['Version']['vernum'] < '6.27-3':                     # added in 6.27  (Python 3)
                    self.config['Misc']['missing'] = 'False'
                if self.config['Version']['vernum'] < '6.26-3':                     # added in 6.26  (Python 3)
                    self.config['Misc']['misscolor'] = (235, 192, 178)
                if self.config['Version']['vernum'] < '6.18-3':                     # added in 6.18  (Python 3) 
                    self.config['Misc']['tmusicbase'] = os.path.normpath(HOMEDIR)            
                if self.config['Version']['vernum'] < '6.15-3': 
                    self.config['Database']['lastqname'] = 'AlbumTitles'
                    self.config['Window']['movposx'] = 0
                    self.config['Window']['movposy'] = 0
                if self.config['Version']['vernum'] < '6.14-3':                     # added in 6.15  (Python 3)                
                    self.config['Misc']['rangelow'] = '1000'
                    self.config['Misc']['rangehigh'] = '8999'
                    self.config['Misc']['berlow'] = '1000'
                    self.config['Misc']['berhigh'] = '8999'
                    self.config['Misc']['tabselect'] = '0'
                if self.config['Version']['vernum'] < '6.07-3':                     # added in 6.07  (Python 3)                
                    self.config['Misc']['altcolor'] = (226, 228, 230)
                self.writeToLogwin('Config - ini file updated to a newer release')
                self.config['Version']['vernum'] = VERNUM
            else:                                                       # and so on
                self.writeToLogwin('Config - ini file (re)loaded')

            self.config['Version']['startup'] = strftime('%Y-%m-%d %H:%M:%S')
            self.config['Version']['inifilename'] = self.inifilename
            self.querydata = self.config['Queries'][self.config['Database']['lastqname']]   
            self.plfilename = os.path.join(inipath, TITLE + '.play')
            self.playlists = ConfigObj(self.plfilename, encoding = 'UTF8')
            self.playlists.initial_comment = ['MusicMan playlists file']
            if 'My Top 25' not in self.playlists:   # .ini exist, .play not / no more
                self.config['View']['selplname'] = ''
                self.config['Window']['expanstate'] = []
        else:                                   # write ini file
            self.config['Version']['vernum'] = VERNUM
            self.config['Window']['expanstate'] = self.playlistwin.GetExpansionState()
            self.config['Window']['perspective'] = self._mgr.SavePerspective()
            windowrect = self.GetRect()
            self.config['Window']['posx'] = windowrect[0]
            self.config['Window']['posy'] = windowrect[1]
            self.config['Window']['sizex'] = windowrect[2]
            self.config['Window']['sizey'] = windowrect[3]
            
            if self.querydata != None:
                self.config['Database']['lastqname'] = self.tabdata[0].querydata['name']
                self.config['Database']['currentquery'] = self.tabdata[0].curquery 
            self.config['Version']['shutdown'] = strftime('%Y-%m-%d %H:%M:%S')            
            
            self.config.write()
            self.playlists.write()
    #-------------------------------------------------------------------------------------------

    def onVolume(self, event):
        keycode = event.GetKeyCode()
        if keycode == 314:           # '->'
            self.nextVolume(keycode)
        elif keycode == 316:         # '<-'
            self.nextVolume(keycode)
        else:
            event.Skip()
    #-------------------------------------------------------------------------------------------------------

    def onChar(self, event):
        keycode = event.GetKeyCode()
        if keycode == 43:           # '+'
            self.playlistwin.ExpandAll()
        elif keycode == 45:         # '-'
            self.playlistwin.CollapseAll()
        event.Skip()
    #-------------------------------------------------------------------------------------------------------

    def readUrl(self):
        if sys.version < '3':
            import urllib2
        else:
            import urllib.request, urllib.error, urllib.parse

        try:
            if sys.version < '3':
                f = urllib2.urlopen(URL_DOWNLOADWEBPAGE, timeout = 5)
            else:
                f = urllib.request.urlopen(URL_DOWNLOADWEBPAGE, timeout = 5)
            html = f.read()
            f.close()
        except:
            return            
        
        versions = []
        fstring = TITLE + '-'
        while len(html):
            fpos = html.find(fstring)
            html = html[fpos:]
            version = html[9:13]
            if html[13:17] == '.zip':
                versions.append(version)
            html = html[24:]
            
        versions.sort(reverse = True)     
        self.versions = versions
    #-------------------------------------------------------------------------------------------

    def onUpdate(self, event):
        self.sb.setLed(GREY)
        # check for new program release
        self.versions = -1
        self.writeToLogwin('Update - try to connect to %s website' % (TITLE))
        dlg = mumaupdate.UpdateDialog(self, '... try to connect to %s website' % (TITLE))
        dlg.Show()
        wx.SafeYield(self)

        updatethread = threading.Thread(target = self.readUrl, args = ())
        updatethread.start()
        while updatethread.isAlive():
            try:
                wx.SafeYield(self)
            except:
                pass
        
        if type(self.versions) == int:
            dlg.changeLabel('unable to connect to the %s website!' % (TITLE))
            self.sb.restoreLed()
            return            
        
        self.writeToLogwin('Update - successfully connected to %s website' % (TITLE))
        dlg.changeLabel('... successfully connected to %s website' % (TITLE))
        if len(self.versions):
            version = self.versions[0]
            if version > VERNUM[:4]:
                dlg.changeLabel("... a newer version (%s) of %s is available!" % (version, TITLE))
                dlg.enableButton()
                doit = dlg.ShowModal()
                self.mainmenu.Enable(self.ID_INSTALL, True)
                self.newrelease = '%s/%s-%s.zip' % (URL_DOWNLOADFOLDER, TITLE, version)
                if doit == wx.ID_OK:
                    self.onInstallUpdate()
                    self.writeToLogwin('Update - process terminated')
                self.sb.restoreLed()
                return
        else:
            self.writeToLogwin("Update check - cannot read information from %s website" % (TITLE))
            dlg.changeLabel('... cannot read information from %s website' % (TITLE))
            dlg.ShowModal()
            self.sb.restoreLed()
            return
               
        dlg.changeLabel('... the installed version of %s is up to date!' % (TITLE))
        if self.updatecheck:
            dlg.Show()
            wx.SafeYield(self)
            self.updatecheck = False
            self.sb.SetStatusText('MusicMan is up to date', 0)
            sleep(1)
            dlg.Close()
        else:
            dlg.ShowModal()
        self.writeToLogwin("Update check - the installed version of %s is up to date" % (TITLE))
        self.sb.restoreLed()
    #-------------------------------------------------------------------------------------------

    def onInstallUpdate(self, event = None):
        # install new program release
        dlg = mumainstall.InstallDialog(self, self.config, self.alogger.info, self.newrelease)
        if dlg.ShowModal() == wx.ID_OK:
            self.writeToLogwin('Update - new release installed - restart %s' % TITLE)
            self.mainmenu.Enable(self.ID_INSTALL, False)    
    #-------------------------------------------------------------------------------------------

    def onWebsite(self, event):
        try:
            webbrowser.open(URL_WEBSITE)
        except:
            dlg = wx.MessageDialog(self, 'cannot connect to the MusicMan website!', TITLE + ' - browse website', wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()
    #-------------------------------------------------------------------------------------------

    def onHandbook(self, event):
        try:
            webbrowser.open(URL_HELPWEBPAGE)
        except:
            dlg = wx.MessageDialog(self, 'cannot connect to the MusicMan website!', TITLE + ' - browse help page', wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()       
    #-------------------------------------------------------------------------------------------

    def onInitIni(self, event):
        info = TITLE + '.ini file settings were corrupted.\nA new .ini file with default \
settings has been created.\nThe old .ini file is saved as ' + \
self.config['Version']['inifilename'].replace('.ini', '.ini.update')        
        convertwin = mumapopup.UpdatePopup(self, info)
        pos = self.notebook.ClientToScreen((0,0))  # pos is the left top edge of the win
        sz = self.notebook.GetSize()               # sz is the size of the win
        convertwin.Position((pos[0] + sz[0] / 2 - 150, pos[1] - 50 - sz[1] / 2), (0, sz[1]))
        convertwin.Popup()
    #-------------------------------------------------------------------------------------------

    def onSysTray(self, event):
        self.handleConfigData('w')          # save window position
        self.tbicon.minimizeToTaskbar()
    #-------------------------------------------------------------------------------------------

    def OnRestoreWindow(self, event):
        self.Show(True)
        self.Iconize(False)
        self.tbicon.RemoveIcon()
        self.Raise()       
    #-------------------------------------------------------------------------------------------------------
    
    def onCloseWindow(self, event = None):
        self.handleConfigData('w')
        pickle.dump(self.mytop25, open(self.inifilename.replace('.ini', '.t25'), 'wb'), pickle.HIGHEST_PROTOCOL)
        self.alogger.info('MusicMan shutdown\n')
        self.plogger.info('MusicMan shutdown\n')
        logging.shutdown()
#        try:
#            os.chmod(self.config['Database']['pathname'], stat.S_IREAD)
#        except:
#            pass
        if len(self.devicename):
            try:
                cmdline = 'pumount %s' % (self.devicename)
                process = subprocess.Popen(cmdline, shell = True, stdin = subprocess.PIPE, stdout = subprocess.PIPE, stderr = subprocess.PIPE)
                process.wait()
            except:
                pass
        evtmgr.eventManager.DeregisterListener(self.onShowPlayContext)
        evtmgr.eventManager.DeregisterListener(self.onContext)
        self._mgr.UnInit()          # deinitialize the frame manager
        self.Destroy()              # switch the light off
# - class MusicManFrame(wx.Frame):----------------------------------------------------------------


class MusicManSplashScreen(wx.SplashScreen):
    
    def __init__(self, instance):
        self.instance = instance
        bmp = wx.Image(ICONPATH + os.sep + 'splash.png').ConvertToBitmap()
        wx.SplashScreen.__init__(self, bmp, wx.SPLASH_CENTRE_ON_PARENT | wx.SPLASH_TIMEOUT,
                                 2000, None, -1)
        self.Bind(wx.EVT_CLOSE, self.OnClose)
        self.fc = wx.FutureCall(1000, self.ShowMain)
    #-------------------------------------------------------------------------------------------

    def OnClose(self, evt):
        # Make sure the default handler runs too so this window gets
        # destroyed
        evt.Skip()
        self.Hide()
        
        # if the timer is still running then go ahead and show the
        # main frame now
        if self.fc.IsRunning():
            self.fc.Stop()
            self.ShowMain()
    #-------------------------------------------------------------------------------------------

    def ShowMain(self):
        frame = MusicManFrame(None, -1, VERSION, self.instance)
        frame.Show()
        try:
            if self.fc.IsRunning():
                self.Raise()
        except:
            pass
# - class MusicManSplashScreen(wx.SplashScreenFrame):-----------------------------------------


class MusicManApp(wx.App):
    def OnInit(self):
        inipath = HOMEDIR
        if len(sys.argv) > 1:
            argument = sys.argv[1].lower()
            if argument.startswith('-i='):
                inipath = sys.argv[1][3:]
        inifilename = os.path.join(inipath, TITLE + '.ini')
        config = ConfigObj(inifilename, encoding = 'UTF8')
        try:
            splash = config['Misc']['splash'] == 'True'
        except:
            splash = True

        try:
            instance = config['Misc']['instance'] == 'False'
        except:
            instance = True
        
        if instance:
            try:
                si = SingleInstance(TITLE)
            except:
                return True
        else:
            si = None
                   
        if splash:
            frame = MusicManSplashScreen(si)
        else:
            frame = MusicManFrame(None, -1, VERSION, si)
        frame.Show()
        return True
    #-------------------------------------------------------------------------------------------

    def OnExit(self):
        try:
            self.lock.release()
        except:
            pass                    # we did our best

# - class MusicManApp(wx.App):--------------------------------------------------------------------
