#!/usr/bin/env python
#-*- coding: UTF-8 -*-

import wx
import time
import os.path
import shutil
from . import mumadir
from   .mumamisc import *

class MoveDialog(wx.Dialog):
    
    ID_START = wx.NewId()
    ID_BTN_TITLE = wx.NewId()
    ID_BTN_PLACE = wx.NewId()
    
    def __init__(self, parent, writeToLogwin, config, database):        
        """   """
        windowpos = (wx.Point(int(config['Window']['movposx']),
                              int(config['Window']['movposy'])))
        wx.Dialog.__init__(self, parent, -1, TITLE + ' - Move files / change DB entries',
                           pos = windowpos, style = wx.DEFAULT_DIALOG_STYLE)
        self.Bind(wx.EVT_CLOSE, self.onButtonClose)
        self.parent = parent
        self.writeToLogwin = writeToLogwin
        self.config = config
        btnadj = int(parent.config['Misc']['norm'] == 'False')
        self.database = database
        self.background_color = wx.SystemSettings_GetColour(READONLY_COLOR)

        sizer = wx.BoxSizer(wx.VERTICAL)
        sb = wx.StaticBox(self, -1, ' Movement action log ')
        row1 = wx.StaticBoxSizer(sb, wx.VERTICAL)                                          
        self.tclog = wx.TextCtrl(self, -1,
                                 style = wx.TE_RICH | wx.TE_MULTILINE | wx.TE_READONLY | wx.TE_DONTWRAP)      
        if LINUX and wx.VERSION_STRING < '3': self.tclog.Enable(False)
        self.tclog.SetBackgroundColour(self.background_color)
        row1.Add(self.tclog, 3, wx.EXPAND | wx.ALL, GAP)
        sizer.Add(row1, 3, wx.GROW | LRT, BORDER)        
        
        sb = wx.StaticBox(self, -1, ' Select from, to and destination Id_Title for movement ')
        row2 = wx.StaticBoxSizer(sb, wx.VERTICAL)
        fgsz = wx.FlexGridSizer(2, 4, 0, 0)
        
        st = wx.StaticText(self, -1, 'From Id_Title:')
        fgsz.Add(st, 0, wx.ALL, GAP)
        self.sctstart = wx.SpinCtrl(self, -1)
        self.sctstart.SetRange(100000, 899999)           
        self.sctstart.SetValue(100000)
        self.sctstart.SetToolTip(wx.ToolTip('>= xxxx00'))
        self.Bind(wx.EVT_SPINCTRL, self.onStart, self.sctstart)
        self.Bind(wx.EVT_TEXT, self.onStart, self.sctstart) 
        fgsz.Add(self.sctstart, 0, wx.ALL, GAP)
        
        st = wx.StaticText(self, -1, ' to (including):')
        fgsz.Add(st, 0, wx.ALL, GAP)
        self.sctend = wx.SpinCtrl(self, -1)
        self.sctend.SetRange(100000, 100099)           
        self.sctend.SetValue(100099)
        self.sctend.SetToolTip(wx.ToolTip('<= xxxx99'))
        self.Bind(wx.EVT_SPINCTRL, self.onStop, self.sctend)
        self.Bind(wx.EVT_TEXT, self.onStop, self.sctend)  
        fgsz.Add(self.sctend, 0, wx.ALL, GAP)        
                
        st = wx.StaticText(self, -1, 'Destination Id_Title:')
        fgsz.Add(st, 0, wx.ALL, GAP)
        self.sctdest = wx.SpinCtrl(self, -1)
        self.sctdest.SetRange(100000, 899999)           
        self.sctdest.SetValue(100000)
        self.sctdest.SetToolTip(wx.ToolTip('>= yyyy00'))
        self.Bind(wx.EVT_SPINCTRL, self.onDestination, self.sctdest)
        self.Bind(wx.EVT_TEXT, self.onDestination, self.sctdest)
        fgsz.Add(self.sctdest, 0, wx.ALL, GAP)

        st = wx.StaticText(self, -1, ' to (including):')
        fgsz.Add(st, 0, wx.ALL, GAP)
        
        self.ctrldend = wx.TextCtrl(self, -1, '100099', style = wx.TE_READONLY)
        self.ctrldend.SetBackgroundColour(self.background_color)
        fgsz.Add(self.ctrldend, 0, wx.ALL | wx.GROW, GAP)
        
        row2.Add(fgsz, 0, wx.EXPAND | wx.ALL, GAP)
        sizer.Add(row2, 1, wx.GROW | LRT, BORDER)
                
        self.butstart = wx.Button(self, self.ID_START, '&Start Move')
        butclose = wx.Button(self, wx.ID_CLOSE)
        self.Bind(wx.EVT_BUTTON, self.onButtonStart, id = self.ID_START)
        self.Bind(wx.EVT_BUTTON, self.onButtonClose, id = wx.ID_CLOSE)     

        row3 = wx.BoxSizer(wx.HORIZONTAL)
        if not btnadj: row3.AddStretchSpacer()
        row3.Add(self.butstart, btnadj, wx.GROW | wx.RIGHT, GAP)
        row3.Add(butclose, btnadj, wx.GROW | wx.LEFT, GAP)
        sizer.Add(row3, 0, wx.GROW | wx.ALL, BORDER)
        
        self.SetSizer(sizer)
        self.Fit()
        self.checkDestAndBorder()
    #-------------------------------------------------------------------------------------------

    def checkDestAndBorder(self):
        self.start_id = self.sctstart.GetValue()
        startisvolume = self.start_id % 100 == 0
        self.end_id = self.sctend.GetValue()
        self.dest_id = self.sctdest.GetValue()
        destisvolume = self.dest_id % 100 == 0
        enddest = self.dest_id + self.end_id - self.start_id
        self.dest_dir = os.path.join(self.config['Misc']['musicbase'], str(self.dest_id)[:2], str(self.dest_id)[:4])
        
        if os.path.isdir(self.dest_dir) and os.listdir(self.dest_dir):
            self.ctrldend.SetBackgroundColour(LRED)     # titles in dir
            retval = 1
        else:
            self.ctrldend.SetBackgroundColour(self.background_color) 
            retval = 0

        if (startisvolume and not destisvolume) or (destisvolume and not startisvolume):
            self.ctrldend.SetBackgroundColour(RED)
            retval = 3           
        elif self.dest_id / 100 != enddest / 100:
            self.ctrldend.SetBackgroundColour(RED)      # range exceeds limit
            retval = 2
        return retval
    #-------------------------------------------------------------------------------------------

    def onStart(self, event):
        startval = event.GetInt()
        endval = startval / 100 * 100 + 99
        self.sctend.SetRange(startval, endval)
        self.sctend.SetValue(endval)        
        destval = self.sctdest.GetValue()
        enddest = destval + endval - startval
        self.ctrldend.SetValue(str(enddest))
        self.checkDestAndBorder()
    #-------------------------------------------------------------------------------------------
             
    def onStop(self, event):
        endval = event.GetInt()
        startval = self.sctstart.GetValue()
        destval = self.sctdest.GetValue()
        enddest = destval + endval - startval
        self.ctrldend.SetValue(str(enddest))
        self.checkDestAndBorder()
    #-------------------------------------------------------------------------------------------

    def onDestination(self, event):
        startval = self.sctstart.GetValue()
        endval = self.sctend.GetValue()
        destval = event.GetInt()
        enddest = destval + endval - startval
        self.ctrldend.SetValue(str(enddest))
        self.checkDestAndBorder()
    #-------------------------------------------------------------------------------------------

    def doTheMove(self, source, destin):
        src = str(source)
        srctimesum = toSeconds(self.database.getDurationById('%s00' % src[0:4]))
        dest = str(destin)
        specialid = isSpecialId(src)
        if specialid:
            desttimesum = 0
        else:
            desttimesum = toSeconds(self.database.getDurationById('%s00' % dest[0:4]))
        info = '... moving title with Id: ' + src + ' to Id: ' + dest
        success, srcfile = findTitleExt(self.config, src)
        if success or specialid:
            destfile = os.path.join(self.config['Misc']['musicbase'], dest[:2], dest[:4], dest) + srcfile[-4:]
            try:
                if not specialid: shutil.move(srcfile, destfile)
                query = 'UPDATE muma_title SET Id_Title = "%s" WHERE Id_Title = "%s"' % (dest, src)
                self.database.execQuery(query)
                self.writeToLog('%s - success' % (info))
                if specialid:
                    titletime = 0
                else:
                    titletime = toSeconds(self.database.getDurationById(dest))
                query = 'UPDATE muma_title SET Duration = "%s" WHERE Id_Title = "%s00"' % (toString(srctimesum - titletime), src[0:4])
                self.database.execQuery(query)
                query = 'UPDATE muma_title SET Duration = "%s" WHERE Id_Title = "%s00"' % (toString(desttimesum + titletime), dest[0:4])
                self.database.execQuery(query)
                try:
                    self.mumadb.execQuery(query)    # possibly no volume header
                except:
                    pass
                return 0
            except:
                pass
        self.writeToLog('%s - failed' % (info))
        if source % 4 == 0: wx.SafeYield(self)
        return 1
    #-------------------------------------------------------------------------------------------
    
    def writeToLog(self, info):
        self.tclog.WriteText('%s\n' % (info))
        self.tclog.ShowPosition(-1)
        self.writeToLogwin("Move file - %s" % (info), onlyfile = True)
    #-------------------------------------------------------------------------------------------
        
    def onButtonStart(self, event):
        retval = self.checkDestAndBorder()
        if retval == 3:
            dlg = wx.MessageDialog(self, "Source or destination is a Volume!\nThey both have to be Volumes or not.", TITLE + ' - Move files', wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()
            return
            
        if retval == 2:
            dlg = wx.MessageDialog(self, "Destination range crosses volume border!\nPlease correct range(s).", TITLE + ' - Move files', wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()
            return
        
        if retval == 1:
            dirlist = os.listdir(self.dest_dir)
            dirlist.sort()
            if dirlist:
                info = 'Destination dir is not empty and contains these title files:\n\n' \
                       + ', '.join(dirlist) + '\n\nPress YES to continue anyhow, NO to stop'
                dlg = wx.MessageDialog(self, info, TITLE + ' - Move files', wx.YES | wx.NO | wx.NO_DEFAULT | wx.ICON_WARNING)
                if dlg.ShowModal() != wx.ID_YES: return 
    
        self.butstart.Enable(False)
        errorcount = 0
        self.writeToLog('starting at %s' % (time.strftime('%Y-%m-%d %H:%M:%S')))
        
        self.src_dir = os.path.join(self.config['Misc']['musicbase'], str(self.start_id)[:2], str(self.start_id)[:4])
        self.dest_dir = os.path.join(self.config['Misc']['musicbase'], str(self.dest_id)[:2], str(self.dest_id)[:4])
        if not os.path.exists(self.dest_dir):
            os.makedirs(self.dest_dir)
            self.writeToLog("created destination dir: '" + self.dest_dir)
            
        if self.start_id <= self.dest_id and self.dest_id <= self.end_id:
            count = self.end_id - self.start_id
            while count >= 0:
                errorcount += self.doTheMove(self.start_id + count, self.dest_id + count)
                count -= 1
        else:
            count = 0
            while count <= self.end_id - self.start_id:
                errorcount += self.doTheMove(self.start_id + count, self.dest_id + count)
                count += 1
        
        self.writeToLog('%i detected errors in this process' % (errorcount))
        self.writeToLog('done at %s' % (time.strftime('%Y-%m-%d %H:%M:%S')))
        self.writeToLogwin("Move file - %i detected errors in this process (see Movement action log for details)" % (errorcount))
            
        self.parent.sb.setLed(GREEN)
        self.butstart.Enable(True)
    #-------------------------------------------------------------------------------------------

    def windowPosition(self):
        windowrect = self.GetRect()
        self.config['Window']['movposx'] = windowrect[0]
        self.config['Window']['movposy'] = windowrect[1]
    #-------------------------------------------------------------------------------------------

    def onButtonClose(self, event):
        self.windowPosition()
        self.Destroy()
# - class QueriesManDialog(wx.Dialog):----------------------------------------------------------

