#!/usr/bin/env python
#-*- coding: UTF-8 -*-

import os
import os.path
import shutil
import wx
from wx.lib.mixins.listctrl import CheckListCtrlMixin
from . import mumadir
from . import mumapopup
from   .mumamisc import *


class CheckListCtrl(wx.ListCtrl, CheckListCtrlMixin):
    def __init__(self, parent, config, sbox):
        style = wx.LC_REPORT | wx.BORDER_SUNKEN
        self.sbox = sbox
        if config ['ListCtrl']['grid'] == 'True':
            style |= wx.LC_HRULES | wx.LC_VRULES
        wx.ListCtrl.__init__(self, parent, -1, style = style)
        CheckListCtrlMixin.__init__(self)
        self.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.OnItemActivated)
        self.checkcounter = 0
    #-------------------------------------------------------------------------------------------

    def resetCheckCounter(self):
        self.checkcounter = 0
    ##-------------------------------------------------------------------------------------------
        
    def OnItemActivated(self, evt):
        self.ToggleItem(evt.m_itemIndex)
    #-------------------------------------------------------------------------------------------

    # this is called by the base class when an item is checked/unchecked
    def OnCheckItem(self, index = 0, flag = None):
        if flag != None:
            if flag:
                self.checkcounter += 1
            else:
                self.checkcounter -= 1
        self.sbox.SetLabel(' Copy / Replacement candidates [%d|%d] ' % (self.checkcounter,
                                                                        self.GetItemCount()))
      #  data = self.GetItemData(index)
        self.Refresh()
# - class CheckListCtrl(wx.ListCtrl):-----------------------------------------------------------


class ReplaceDialog(wx.Dialog):
    ID_TC_SOURCE = wx.NewId()
    ID_TC_MUBASE = wx.NewId()
    ID_BTN_SOURCE = wx.NewId()
    ID_CB_DELETE = wx.NewId()
    ID_CB_DELDIR = wx.NewId()
    
    def __init__(self, parent, config, writeToLogwin):
        windowpos = (wx.Point(int(config['Window']['reposx']),
                              int(config['Window']['reposy'])))
        windowsize = (wx.Size(int(config['Window']['resizex']),
                              int(config['Window']['resizey'])))                
        wx.Dialog.__init__(self, parent, -1, TITLE + ' - Copy / Replace title files',
                           pos = windowpos, size = windowsize,
                           style = wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER)
        self.Bind(wx.EVT_CLOSE, self.onCloseWindow)
        self.readonly_color = wx.SystemSettings_GetColour(READONLY_COLOR)
        self.config = config
        self.writeToLogwin = writeToLogwin
        collist = config['Misc']['hlcolor']
        btnadj = int(config['Misc']['norm'] == 'False')
        self.hlcolor = (int(collist[0]), int(collist[1]), int(collist[2]))
         
        sizer = wx.BoxSizer(wx.VERTICAL)
        sb = wx.StaticBox(self, -1, ' Source and destination directories ')
        sbsz1 = wx.StaticBoxSizer(sb, wx.VERTICAL)
        fgsz = wx.FlexGridSizer(4, 3, 0, 0)
        fgsz.AddGrowableCol(1)         
        st = wx.StaticText(self, -1, 'Source dir:')
        fgsz.Add(st, 0, wx.ALL, GAP)
        self.tcsource = wx.TextCtrl(self, self.ID_TC_SOURCE, style = wx.TE_READONLY)
        self.tcsource.SetBackgroundColour(self.readonly_color)
        fgsz.Add(self.tcsource, 0, wx.GROW | wx.ALL, GAP)
        self.btnsource = wx.Button(self, self.ID_BTN_SOURCE, 'Browse')
        self.Bind(wx.EVT_BUTTON, self.onSourceDir, id = self.ID_BTN_SOURCE)
        fgsz.Add(self.btnsource, 0, wx.ALL | wx.GROW, GAP)     
        st = wx.StaticText(self, -1, 'Music basedir:')
        fgsz.Add(st, 0, wx.ALL, GAP)
        tcmusicbd = wx.TextCtrl(self, self.ID_TC_MUBASE, value = self.config['Misc']['musicbase'],
                                style = wx.TE_READONLY)
        tcmusicbd.SetBackgroundColour(self.readonly_color)
        fgsz.Add(tcmusicbd, 0, wx.GROW | wx.ALL, GAP)        
        fgsz.AddStretchSpacer()       
        st = wx.StaticText(self, -1, 'Delete source file:')
        fgsz.Add(st, 0, wx.ALL, GAP)
        self.cbd = wx.CheckBox(self, self.ID_CB_DELETE, ' (if checked - delete file(s) when copied)')
        fgsz.Add(self.cbd, 0, wx.ALL, GAP)
        fgsz.AddStretchSpacer()       
        st = wx.StaticText(self, -1, 'Delete source dir:')
        fgsz.Add(st, 0, wx.ALL, GAP)
        self.cbdd = wx.CheckBox(self, self.ID_CB_DELDIR, ' (if checked - delete source dir when empty)')
        fgsz.Add(self.cbdd, 0, wx.ALL, GAP)
        self.Bind(wx.EVT_CHECKBOX, self.onCheckDeldir, id = self.ID_CB_DELDIR)
        fgsz.AddStretchSpacer()       
        sbsz1.Add(fgsz, 0, wx.GROW | wx.ALL, GAP)
        sizer.Add(sbsz1, 0, wx.GROW | LRT, BORDER)
        
        self.sb1 = wx.StaticBox(self, -1, ' Copy / Replacement candidates [0|0] ')
        sbsz2 = wx.StaticBoxSizer(self.sb1, wx.VERTICAL)        
        self.cball = wx.CheckBox(self, -1, 'do not change selection / check / uncheck all candidates',
                                 style = wx.CHK_3STATE | wx.CHK_ALLOW_3RD_STATE_FOR_USER)
        self.Bind(wx.EVT_CHECKBOX, self.onCheckAll, self.cball)
        sbsz2.Add(self.cball, 0, wx.GROW | wx.ALL, GAP)
        self.candidates = CheckListCtrl(self, self.config, self.sb1)
        self.candidates.InsertColumn(0, 'use this file')
        self.candidates.InsertColumn(1, 'to replace this file')
        sbsz2.Add(self.candidates, 1, wx.GROW | wx.ALL, GAP)
        sizer.Add(sbsz2, 1, wx.GROW | LRT, BORDER)

        self.sb = wx.StaticBox(self, -1, ' Progress ')
        sbsz3 = wx.StaticBoxSizer(self.sb, wx.VERTICAL)
        self.gauge = wx.Gauge(self, -1)
        sbsz3.Add(self.gauge, 0, wx.EXPAND | wx.ALL, GAP)
        sizer.Add(sbsz3, 0, wx.GROW | LRT, BORDER)

        butrow = wx.BoxSizer(wx.HORIZONTAL)
        if not btnadj: butrow.AddStretchSpacer()
        self.butreplace = wx.Button(self, wx.ID_REPLACE, 'Copy / &Replace selected')
        self.Bind(wx.EVT_BUTTON, self.onReplace, id = wx.ID_REPLACE)
        butrow.Add(self.butreplace, btnadj, wx.GROW | wx.ALL, GAP)
        butclose = wx.Button(self, wx.ID_CLOSE)
        self.Bind(wx.EVT_BUTTON, self.onClose, id = wx.ID_CLOSE)
        butrow.Add(butclose, btnadj, wx.GROW | wx.ALL, GAP)
        sizer.Add(butrow, 0, wx.GROW | wx.ALL, GAP)
    
        self.SetSizer(sizer)
        self.SetMinSize(wx.Size(500, 500))
    #-------------------------------------------------------------------------------------------

    def onCheckDeldir(self, event):
        if self.cbdd.GetValue():
            self.cbd.SetValue(True)
            self.cbd.Enable(False)
        else:
            self.cbd.Enable(True)
    #-------------------------------------------------------------------------------------------
            
    def onSourceDir(self, event = None):
        self.matchcount = 0
        self.sortlist = []
        self.gauge.SetValue(0)
        self.candidates.resetCheckCounter()
        self.candidates.DeleteAllItems()
        self.cball.Set3StateValue(0)
        
        self.tcsource.Clear()
        dlg = mumadir.DirDialog(self, TITLE + ' - select dir with copy / replacement files', self.config, 
                                self.config['Misc']['replacedir'], mode = 'opend')
        if dlg.ShowModal() == wx.ID_OPEN:
            self.basechanged = False
            self.sourcepath = os.path.normpath(dlg.GetPath())
            self.tcsource.SetValue(self.sourcepath)
            self.config['Misc']['replacedir'] = self.sourcepath
            dlg.Destroy()
        else:
            dlg.Destroy()
            return
        self.butreplace.Enable()

        musicdir = self.config['Misc']['musicbase']
        if os.path.isdir(musicdir):
            try:
                sourcelist = os.listdir(self.sourcepath)
            except:
                return            
            sourcelist.sort()
            self.sortlist = []
            self.checklist = []
            for item in sourcelist:
                try:
                    i = int(item[0:6])              # is this 000000.ext .. 999999.ext
                    self.sortlist.append(item)
                except:
                    pass      
            
            self.replacelist = []        
            self.candidates.DeleteAllItems()
            self.sb.SetLabel(' Check candidates progress ')
            self.gauge.SetRange(len(self.sortlist))
            for index, item in enumerate(self.sortlist):
                listitem = self.candidates.InsertStringItem(index, '%s/%s' % (self.sourcepath, item))
                found, destfile = findTitleExt(self.config, item[0:6])
                self.replacelist.append(destfile)
                if found:
                    self.candidates.SetStringItem(index, 1, '%s' % (destfile))
                    self.candidates.CheckItem(index, True)
                    self.checklist.append(True)
                else:
                    self.candidates.SetItemTextColour(listitem, RED)
                    self.checklist.append(False)
                self.gauge.SetValue(index + 1)

            self.candidates.SetColumnWidth(0, wx.LIST_AUTOSIZE)
            if not LINUX: self.candidates.SetColumnWidth(0, self.candidates.GetColumnWidth(0) + 5)
            self.candidates.SetColumnWidth(1, wx.LIST_AUTOSIZE)
            rect = self.GetRect()
            sizex = max(rect[2] / 2 - self.candidates.GetColumnWidth(0) - 50, self.candidates.GetColumnWidth(1))
            self.candidates.SetColumnWidth(1, sizex)
            self.candidates.OnCheckItem()  # update check / lines counter
            self.Refresh()
        else:            
            dlg = wx.MessageDialog(self, "Music basedir is not reachable!            ",
                                       TITLE + ' - Copy / Replace title files', wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()
            dlg.Destroy()
    #-------------------------------------------------------------------------------------------

    def onReplace(self, event):
        copycount = 0
        notchecked = 0
        itemcount = self.candidates.GetItemCount()
        delsource = self.cbd.GetValue()
        delsourcedir = self.cbdd.GetValue()
        self.sb.SetLabel(' Copy / Replacement progress ')
        self.Refresh()
        if itemcount:
            for index in range(itemcount):
                self.gauge.SetRange(itemcount)
                if self.candidates.IsChecked(index):
                    sourcestr = os.path.join(self.sourcepath, self.sortlist[index])
                    deststr = self.replacelist[index]
                    dirpart = os.path.dirname(deststr)
                    if not os.path.exists(dirpart):
                        os.makedirs(dirpart)
                    try:
                        if not deststr.endswith('???'):
                            os.remove(deststr)
                        deststr = deststr[:-3] + sourcestr[-3:]
                        if delsource:
                            shutil.move(sourcestr, deststr)
                        else:
                            shutil.copy(sourcestr, deststr)
                        self.writeToLogwin("Replace title files - '%s' copied to '%s'" % (sourcestr, deststr))
                    except:
                        pass                    
                    copycount += 1
                    self.gauge.SetValue(index + 1)
                    wx.SafeYield(self)
                else:
                    notchecked += 1
            if self.cbdd.GetValue():
                try:
                    os.rmdir(self.sourcepath)
                except:
                    delsourcedir = False
            self.gauge.SetValue(index + 1)
            self.Refresh()
            wx.SafeYield(self)
            if delsource:
                self.writeToLogwin("Replace title files - source title files deleted")
            if delsourcedir:
                self.writeToLogwin("Replace title files - source dir deleted")
        else:
            dlg = wx.MessageDialog(self, "Ooooops - here is nothing to replace!",
                                       TITLE + ' - Replace title files', wx.OK | wx.ICON_INFORMATION)
            dlg.ShowModal()
            dlg.Destroy()
                    
        self.writeToLogwin("Replace title files - %i file(s) replaced / copied" % (copycount))
        nocopy = itemcount - copycount - notchecked
        if nocopy:
            self.writeToLogwin("Replace title files - %i file(s) could not be replaced!" % (nocopy))

        self.candidates.resetCheckCounter()
        self.candidates.DeleteAllItems()
        self.cball.Set3StateValue(0)
        self.sb1.SetLabel(' Copy / Replacement candidates [0|0] ')
        self.Refresh()

        self.sb.SetLabel(' Copy / Replacement finished (%i files copied / replaced)' % (copycount))
        self.butreplace.Disable()
    #-------------------------------------------------------------------------------------------

    def onCheckAll(self, event = None):
        cb = event.GetEventObject()
        cbstate = cb.Get3StateValue() 
        for index in range(len(self.sortlist)):
            if cbstate == 0:
                self.candidates.CheckItem(index, self.checklist[index])
            elif cbstate == 1:
                self.candidates.CheckItem(index, True)
            else:
                self.candidates.CheckItem(index, False)
        self.Refresh()
    #-------------------------------------------------------------------------------------------
    
    def onClose(self, event = None):
        self.windowPosition()
        self.EndModal(wx.ID_CLOSE)
    #-------------------------------------------------------------------------------------------

    def windowPosition(self):
        windowrect = self.GetRect()
        self.config['Window']['reposx'] = windowrect[0]
        self.config['Window']['reposy'] = windowrect[1]
        self.config['Window']['resizex'] = windowrect[2]
        self.config['Window']['resizey'] = windowrect[3]        
    #-------------------------------------------------------------------------------------------
        
    def onCloseWindow(self, event = None):
        self.windowPosition()
        self.Destroy()
# - class BatchEditDialog(wx.Dialog):-----------------------------------------------------------
    
