#!/usr/bin/env python
#-*- coding: UTF-8 -*-

import os
import os.path
import codecs
import time
import datetime
import wx
import shutil
import logging
import logging.handlers
from . import mumadir
from   .mumamisc import *
from difflib import SequenceMatcher

try:
    from   mutagen.mp3 import MP3
    from   mutagen.oggvorbis import OggVorbis
    from   mutagen.easyid3 import EasyID3
    import mutagen.id3
except:
    pass

class PatchDialog(wx.Dialog):
    
    ID_PATCH = wx.NewId()

    def __init__(self, parent, writeToLogwin, makeDbBackup, database, patchlist):
        wx.Dialog.__init__(self, parent, -1, TITLE + ' - patch db with checked items',                           
                           style = wx.DEFAULT_DIALOG_STYLE)
        btnadj = int(parent.config['Misc']['norm'] == 'False')
        self.writeToLogwin = writeToLogwin
        self.makeDbBackup = makeDbBackup
        self.database = database
        self.patchlist = patchlist
        sizer = wx.BoxSizer(wx.VERTICAL)
        sb = wx.StaticBox(self, -1, ' List of patch candidates ')
        sbsz = wx.StaticBoxSizer(sb, wx.VERTICAL)
        
        self.lb = wx.CheckListBox(self, -1, choices = self.patchlist)
        sbsz.Add(self.lb, 1, wx.GROW | wx.ALL, GAP)
        #self.lb.SetSelection(0)

        self.butpatch = wx.Button(self, self.ID_PATCH, '&Patch')
        self.butcancel = wx.Button(self, wx.ID_CANCEL)
        self.Bind(wx.EVT_BUTTON, self.onButtonPatch, id = self.ID_PATCH)    
        butsz = wx.BoxSizer(wx.HORIZONTAL)
        if not btnadj: butsz.AddStretchSpacer()
        butsz.Add(self.butpatch, btnadj, wx.GROW | wx.RIGHT, GAP)
        butsz.Add(self.butcancel, btnadj, wx.GROW | wx.LEFT, GAP)

        sizer.Add(sbsz, 1, wx.GROW | LRT, BORDER)        
        sizer.Add(butsz, 0, wx.GROW | wx.ALL, BORDER)
        self.SetSizer(sizer)
        self.Fit()
        self.Centre()                
    #-------------------------------------------------------------------------------------------

    def onButtonPatch(self, event):
        self.makeDbBackup()
        for index in range(self.lb.GetCount()):
            if self.lb.IsChecked(index):
                patchstr = self.lb.GetString(index)
                ps99 = patchstr[0:6]
                psid = patchstr[23:29]
                self.database.patchTables(ps99, psid, delentry = True)
                self.writeToLogwin("Patch db - '%s' patched with '%s'" % (ps99, psid))    
        self.Destroy()    
# - class PatchDialog(wx.Dialog):----------------------------------------------------------------


class SyncDialog(wx.Dialog):
    
    ID_START = wx.NewId()
    ID_DEFAULT = wx.NewId()
    ID_BTN_PLACE = wx.NewId()
    ID_BTN_TITLE = wx.NewId()
    ID_BTN_BASE = wx.NewId()
    
    def __init__(self, parent, config, writeToLogwin, makeDbBackup, database):        
        """   """
        windowpos = (wx.Point(int(config['Window']['synposx']),
                              int(config['Window']['synposy'])))        
        wx.Dialog.__init__(self, parent, -1, TITLE + ' - analyze syncstate db and music volume',                           
                           pos = windowpos,
                           style = wx.DEFAULT_DIALOG_STYLE)
        self.Bind(wx.EVT_CLOSE, self.onButtonClose)
        self.parent = parent
        self.config = config
        self.writeToLogwin = writeToLogwin
        self.makeDbBackup = makeDbBackup
        self.basedir = config['Misc']['musicbase']
        self.tbasedir = config['Misc']['tmusicbase']
        btnadj = int(config['Misc']['norm'] == 'False')
        self.database = database
        self.background_color = wx.SystemSettings_GetColour(READONLY_COLOR)
        self.stopsync = True
        self.closerequest = False
        self.dotcount = 0
        self.startOL = True
        self.font = wx.Font(8, wx.DEFAULT, wx.NORMAL, wx.NORMAL)
        
        sizer = wx.BoxSizer(wx.VERTICAL)
        sb = wx.StaticBox(self, -1, ' Log messages ')
        row1 = wx.StaticBoxSizer(sb, wx.VERTICAL)
        self.tc = wx.TextCtrl(self, -1, style = wx.TE_RICH | wx.TE_MULTILINE | wx.TE_READONLY | wx.TE_DONTWRAP) #wx.TE_BESTWRAP) #wx.TE_DONTWRAP)      
        if LINUX and wx.VERSION_STRING < '3': self.tc.Enable(False)
        #background_color = wx.SystemSettings_GetColour(READONLY_COLOR)
        self.tc.SetBackgroundColour(self.background_color)
        row1.Add(self.tc, 2, wx.EXPAND | wx.ALL, GAP)
        sizer.Add(row1, 2, wx.GROW | LRT, BORDER)
        
        self.nb = wx.Notebook(self, -1)        
        sizer.Add(self.nb, 1, wx.GROW | LRT, BORDER)        
        self.initTab1()
        self.initTab2()
        self.initTab3()
        self.initTab4()        
        self.initTab5()
        self.initTab6()
        self.initTab7()
        self.initTab8()
        
        self.butstart = wx.Button(self, self.ID_START, '&Start sync')
        self.butdefault = wx.Button(self, self.ID_DEFAULT, '&Default range')
        self.butclose = wx.Button(self, wx.ID_CLOSE)
        self.Bind(wx.EVT_BUTTON, self.onButtonStart, id = self.ID_START)
        self.Bind(wx.EVT_BUTTON, self.onButtonDefault, id = self.ID_DEFAULT)
        self.Bind(wx.EVT_BUTTON, self.onButtonClose, id = wx.ID_CLOSE)     
        row3 = wx.BoxSizer(wx.HORIZONTAL)
        if not btnadj: row3.AddStretchSpacer()
        row3.Add(self.butstart, btnadj, wx.GROW | wx.RIGHT, GAP)
        row3.Add(self.butdefault, btnadj, wx.GROW | wx.LEFT | wx.RIGHT, GAP)
        row3.Add(self.butclose, btnadj, wx.GROW | wx.LEFT, GAP)
        sizer.Add(row3, 0, wx.GROW | wx.ALL, BORDER)
        self.SetSizer(sizer)
        self.Fit()
        self.nb.SetSelection(int(config['Misc']['tabselect']))
        
        logfilename = 'MusicMan.syn'
        logpath = os.path.join(HOMEDIR, logfilename)
        self.synlogger = logging.getLogger('SyncLog')
        self.synlogger.setLevel(logging.INFO)
        self.handler = logging.handlers.RotatingFileHandler(logpath, 'a', ACTION_LOG_SIZE, 4)
        formatter = logging.Formatter("%(asctime)s -- %(message)s", "%Y-%m-%d %H:%M:%S")
        self.handler.setFormatter(formatter)        
        self.synlogger.addHandler(self.handler)
        self.synlogger.info('%s started - %s' % (logpath, VERSION))        
    #-------------------------------------------------------------------------------------------
                                  
    def initTab1(self):
        tab1 = wx.Panel(self.nb, -1)
        tab1sz = wx.BoxSizer(wx.VERTICAL)
        gbsz = wx.GridBagSizer(BORDER, BORDER)
        
        st = wx.StaticText(tab1, -1, 'Music device:')
        gbsz.Add(st, wx.GBPosition(0, 0), wx.GBSpan(1, 1), LRT | wx.GROW, GAP)
        self.cbv = wx.CheckBox(tab1, -1, ' (if checked - compare music device / disk with titles in database)') 
        gbsz.Add(self.cbv, wx.GBPosition(0, 1), wx.GBSpan(1, 3), wx.RIGHT | wx.GROW, GAP)
                
        st = wx.StaticText(tab1, -1, 'Folder range:')
        gbsz.Add(st, wx.GBPosition(1, 0), wx.GBSpan(1, 1), LRT | wx.GROW, GAP)        
        self.scmd1 = wx.SpinCtrl(tab1, -1)
        self.scmd1.SetRange(1000, 8999)           
        self.scmd1.SetValue(int(self.config['Misc']['rangelow']))
        self.scmd1.SetToolTip(wx.ToolTip('>= 1000'))
        self.Bind(wx.EVT_SPINCTRL, self.onLow, self.scmd1)
        gbsz.Add(self.scmd1, wx.GBPosition(1, 1), wx.GBSpan(1, 1), wx.ALL, GAP)
        st = wx.StaticText(tab1, -1, ' to (including) ')
        gbsz.Add(st, wx.GBPosition(1, 2), wx.GBSpan(1, 1), LRT, GAP)
        self.scmd2 = wx.SpinCtrl(tab1, -1)
        self.scmd2.SetRange(1000, 8999)           
        self.scmd2.SetValue(int(self.config['Misc']['rangehigh']))
        self.scmd2.SetToolTip(wx.ToolTip('<= 8999'))
        self.Bind(wx.EVT_SPINCTRL, self.onHigh, self.scmd2)
        gbsz.Add(self.scmd2, wx.GBPosition(1, 3), wx.GBSpan(1, 1), LRT, GAP)

        tab1sz.Add(gbsz, 0, wx.GROW | LRT, BORDER)    
        tab1.SetSizer(tab1sz)
        self.nb.AddPage(tab1, 'Music device')
    #-------------------------------------------------------------------------------------------

    def initTab2(self):
        tab2 = wx.Panel(self.nb, -1)
        tab2sz = wx.BoxSizer(wx.VERTICAL)
        gbsz = wx.GridBagSizer(BORDER, BORDER)
        
        st = wx.StaticText(tab2, -1, '99xxxx test:')
        gbsz.Add(st, wx.GBPosition(0, 0), wx.GBSpan(1, 1), LRT | wx.GROW, GAP)        
        self.cb9 = wx.CheckBox(tab2, -1, ' (if checked - search for missing 99xxxx titles in database)')
        gbsz.Add(self.cb9, wx.GBPosition(0, 1), wx.GBSpan(1, 3), LRT | wx.GROW, GAP)
        self.Bind(wx.EVT_CHECKBOX, self.onCheck99xx, self.cb9)
        st = wx.StaticText(tab2, -1, 'Offer patch:')
        gbsz.Add(st, wx.GBPosition(1, 0), wx.GBSpan(1, 1), LRT | wx.GROW, GAP)        
        self.cbp = wx.CheckBox(tab2, -1, ' (if checked - update database to selected hits)')
        gbsz.Add(self.cbp, wx.GBPosition(1, 1), wx.GBSpan(1, 3), LRT | wx.GROW, GAP)         
        self.cbp.Disable()

        tab2sz.Add(gbsz, 0, wx.GROW | LRT, BORDER)
        tab2.SetSizer(tab2sz)
        self.nb.AddPage(tab2, '99xxxx Test')
    #-------------------------------------------------------------------------------------------


    def initTab3(self):
        tab3 = wx.Panel(self.nb, -1)
        tab3sz = wx.BoxSizer(wx.VERTICAL)
        gbsz = wx.GridBagSizer(BORDER, BORDER)

        st = wx.StaticText(tab3, -1, 'Duplicates:')
        gbsz.Add(st, wx.GBPosition(0, 0), wx.GBSpan(1, 1), LRT | wx.GROW, GAP)        
        self.cbd = wx.CheckBox(tab3, -1, ' (if checked - find title duplicates in database)') 
        gbsz.Add(self.cbd, wx.GBPosition(0, 1), wx.GBSpan(1, 3), LRT | wx.GROW, GAP)

        tab3sz.Add(gbsz, 0, wx.GROW | LRT, BORDER)
        tab3.SetSizer(tab3sz)
        self.nb.AddPage(tab3, 'Duplicates')
    #-------------------------------------------------------------------------------------------

    def initTab4(self):
        tab4 = wx.Panel(self.nb, -1)
        tab4sz = wx.BoxSizer(wx.VERTICAL)
        gbsz = wx.GridBagSizer(BORDER, BORDER)

        st = wx.StaticText(tab4, -1, 'SimiTitles:')
        gbsz.Add(st, wx.GBPosition(0, 0), wx.GBSpan(1, 1), LRT | wx.GROW, GAP)        
        self.cbs1 = wx.CheckBox(tab4, -1, ' (if checked - find similar titles in database)') 
        gbsz.Add(self.cbs1, wx.GBPosition(0, 1), wx.GBSpan(1, 3), wx.RIGHT | wx.GROW, 0)

        st = wx.StaticText(tab4, -1, 'Threshold:')
        gbsz.Add(st, wx.GBPosition(1, 0), wx.GBSpan(1, 1), LRT | wx.GROW, GAP)
        self.sctht = wx.SpinCtrl(tab4, -1)
        self.sctht.SetRange(50, 100)           
        self.sctht.SetValue(int(self.config['Misc']['thresht']))
        self.sctht.SetToolTip(wx.ToolTip('50 <= Threshold <= 100'))
        gbsz.Add(self.sctht, wx.GBPosition(1, 1), wx.GBSpan(1, 1), LRT, GAP)
        self.cbt = wx.CheckBox(tab4, -1, 'Include Interpreter: ', style = wx.ALIGN_RIGHT) 
        gbsz.Add(self.cbt, wx.GBPosition(1, 2), wx.GBSpan(1, 2), LRT | wx.GROW, GAP)

        st = wx.StaticText(tab4, -1, 'Folder range:')
        gbsz.Add(st, wx.GBPosition(2, 0), wx.GBSpan(1, 1), wx.ALL | wx.GROW, GAP)
        self.sctht1 = wx.SpinCtrl(tab4, -1)
        self.sctht1.SetRange(1000, 8999)           
        self.sctht1.SetValue(int(self.config['Misc']['rangelow']))
        self.sctht1.SetToolTip(wx.ToolTip('>= 1000'))
        self.Bind(wx.EVT_SPINCTRL, self.onLow, self.sctht1)
        gbsz.Add(self.sctht1, wx.GBPosition(2, 1), wx.GBSpan(1, 1), wx.ALL, GAP)
        st = wx.StaticText(tab4, -1, ' to (including) ')
        gbsz.Add(st, wx.GBPosition(2, 2), wx.GBSpan(1, 1), wx.ALL, GAP)
        self.sctht2 = wx.SpinCtrl(tab4, -1)
        self.sctht2.SetRange(1000, 8999)           
        self.sctht2.SetValue(int(self.config['Misc']['rangehigh']))
        self.sctht2.SetToolTip(wx.ToolTip('<= 8999'))
        self.Bind(wx.EVT_SPINCTRL, self.onHigh, self.sctht2)
        gbsz.Add(self.sctht2, wx.GBPosition(2, 3), wx.GBSpan(1, 1), wx.ALL, GAP)

        tab4sz.Add(gbsz, 0, wx.GROW | LRT, BORDER)
        tab4.SetSizer(tab4sz)
        self.nb.AddPage(tab4, 'SimTitles')
    #-------------------------------------------------------------------------------------------

    def initTab5(self):
        tab5 = wx.Panel(self.nb, -1)
        tab4sz = wx.BoxSizer(wx.VERTICAL)
        gbsz = wx.GridBagSizer(BORDER, BORDER)

        st = wx.StaticText(tab5, -1, 'SimInterp:')
        gbsz.Add(st, wx.GBPosition(0, 0), wx.GBSpan(1, 1), LRT | wx.GROW, GAP)        
        self.cbs2 = wx.CheckBox(tab5, -1, ' (if checked - find similar interpreters in database)') 
        gbsz.Add(self.cbs2, wx.GBPosition(0, 1), wx.GBSpan(1, 3), wx.RIGHT | wx.GROW, 0)

        st = wx.StaticText(tab5, -1, 'Threshold:')
        gbsz.Add(st, wx.GBPosition(1, 0), wx.GBSpan(1, 1), LRT | wx.GROW, GAP)
        self.scthi = wx.SpinCtrl(tab5, -1)
        self.scthi.SetRange(50, 100)           
        self.scthi.SetValue(int(self.config['Misc']['threshi']))
        self.scthi.SetToolTip(wx.ToolTip('50 <= Threshold <= 100'))
        gbsz.Add(self.scthi, wx.GBPosition(1, 1), wx.GBSpan(1, 1), LRT, GAP)

        st = wx.StaticText(tab5, -1, 'Start substring:')
        gbsz.Add(st, wx.GBPosition(2, 0), wx.GBSpan(1, 1), wx.ALL | wx.GROW, GAP)
        self.ctrlsseq = wx.TextCtrl(tab5, -1, '')
        gbsz.Add(self.ctrlsseq, wx.GBPosition(2, 1), wx.GBSpan(1, 3), wx.ALL | wx.GROW, GAP)
        st = wx.StaticText(tab5, -1, '(i.e. Beatles, not eatles)')
        gbsz.Add(st, wx.GBPosition(2, 4), wx.GBSpan(1, 1), wx.ALL, GAP)

        tab4sz.Add(gbsz, 0, wx.GROW | LRT, BORDER)
        tab5.SetSizer(tab4sz)
        self.nb.AddPage(tab5, 'SimInterp')
    #-------------------------------------------------------------------------------------------

    def initTab6(self):
        tab6 = wx.Panel(self.nb, -1)
        tab6sz = wx.BoxSizer(wx.VERTICAL)
        gbsz = wx.GridBagSizer(BORDER, BORDER)

        st = wx.StaticText(tab6, -1, 'Meta data:   ')
        gbsz.Add(st, wx.GBPosition(0, 0), wx.GBSpan(1, 1), LRT | wx.GROW, GAP)
        self.cbm = wx.CheckBox(tab6, -1, ' (if checked - insert database values into empty file meta tags)') 
        gbsz.Add(self.cbm, wx.GBPosition(0, 1), wx.GBSpan(1, 3), LRT | wx.GROW, GAP)
        self.Bind(wx.EVT_CHECKBOX, self.onCheckMeta, self.cbm)
        st = wx.StaticText(tab6, -1, 'Overwrite:   ')
        gbsz.Add(st, wx.GBPosition(1, 0), wx.GBSpan(1, 1), LRT | wx.GROW, GAP)
        self.cbo = wx.CheckBox(tab6, -1, ' (if checked - overwrite file meta tags with database values)') 
        gbsz.Add(self.cbo, wx.GBPosition(1, 1), wx.GBSpan(1, 3), LRT | wx.GROW, GAP)
        self.cbo.Disable()
        st = wx.StaticText(tab6, -1, 'Folder range:')
        gbsz.Add(st, wx.GBPosition(2, 0), wx.GBSpan(1, 1), wx.ALL | wx.GROW, GAP)
        self.scma1 = wx.SpinCtrl(tab6, -1)
        self.scma1.SetRange(1000, 8999)           
        self.scma1.SetValue(int(self.config['Misc']['rangelow']))
        self.scma1.SetToolTip(wx.ToolTip('>= 1000'))
        self.Bind(wx.EVT_SPINCTRL, self.onLow, self.scma1)
        gbsz.Add(self.scma1, wx.GBPosition(2, 1), wx.GBSpan(1, 1), wx.ALL, GAP)
        st = wx.StaticText(tab6, -1, ' to (including) ')
        gbsz.Add(st, wx.GBPosition(2, 2), wx.GBSpan(1, 1), wx.ALL, GAP)
        self.scma2 = wx.SpinCtrl(tab6, -1)
        self.scma2.SetRange(1000, 8999)           
        self.scma2.SetValue(int(self.config['Misc']['rangehigh']))
        self.scma2.SetToolTip(wx.ToolTip('<= 8999'))        
        self.Bind(wx.EVT_SPINCTRL, self.onHigh, self.scma2)
        gbsz.Add(self.scma2, wx.GBPosition(2, 3), wx.GBSpan(1, 1), wx.ALL, GAP)

        tab6sz.Add(gbsz, 0, wx.GROW | LRT, BORDER)
        tab6.SetSizer(tab6sz)
        self.nb.AddPage(tab6, 'Meta data')
    #-------------------------------------------------------------------------------------------
    
    def initTab7(self):
        tab7 = wx.Panel(self.nb, -1)
        tab7sz = wx.BoxSizer(wx.VERTICAL)
        gbsz = wx.GridBagSizer(BORDER, BORDER)
        
        st = wx.StaticText(tab7, -1, 'Extensions:')
        gbsz.Add(st, wx.GBPosition(0, 0), wx.GBSpan(1, 1), LRT | wx.GROW, GAP)
        self.cbe = wx.CheckBox(tab7, -1, ' (if checked - make all file extensions lower case)') 
        gbsz.Add(self.cbe, wx.GBPosition(0, 1), wx.GBSpan(1, 3), wx.RIGHT | wx.GROW, GAP)
        
        st = wx.StaticText(tab7, -1, 'Folder range:')
        gbsz.Add(st, wx.GBPosition(1, 0), wx.GBSpan(1, 1), LRT | wx.GROW, GAP)        
        self.scme1 = wx.SpinCtrl(tab7, -1)
        self.scme1.SetRange(1000, 8999)           
        self.scme1.SetValue(int(self.config['Misc']['rangelow']))
        self.scme1.SetToolTip(wx.ToolTip('>= 1000'))
        self.Bind(wx.EVT_SPINCTRL, self.onLow, self.scme1)
        gbsz.Add(self.scme1, wx.GBPosition(1, 1), wx.GBSpan(1, 1), wx.ALL, GAP)
        st = wx.StaticText(tab7, -1, ' to (including) ')
        gbsz.Add(st, wx.GBPosition(1, 2), wx.GBSpan(1, 1), LRT, GAP)
        self.scme2 = wx.SpinCtrl(tab7, -1)
        self.scme2.SetRange(1000, 8999)           
        self.scme2.SetValue(int(self.config['Misc']['rangehigh']))
        self.scme2.SetToolTip(wx.ToolTip('<= 8999'))
        self.Bind(wx.EVT_SPINCTRL, self.onHigh, self.scme2)
        gbsz.Add(self.scme2, wx.GBPosition(1, 3), wx.GBSpan(1, 1), LRT, GAP)

        tab7sz.Add(gbsz, 0, wx.GROW | LRT, BORDER)    
        tab7.SetSizer(tab7sz)
        self.nb.AddPage(tab7, 'Extensions')
    #-------------------------------------------------------------------------------------------

    def initTab8(self):
        tab8 = wx.Panel(self.nb, -1)
        tab8sz = wx.BoxSizer(wx.VERTICAL)
        gbsz = wx.GridBagSizer(BORDER, BORDER)
        
        st = wx.StaticText(tab8, -1, 'Pair devices:')
        gbsz.Add(st, wx.GBPosition(0, 0), wx.GBSpan(1, 1), LRT | wx.GROW, GAP)
        self.cbpd = wx.CheckBox(tab8, -1, ' (if checked - pair target device to master device)') 
        gbsz.Add(self.cbpd, wx.GBPosition(0, 1), wx.GBSpan(1, 3), wx.RIGHT | wx.GROW, GAP)
        
        st = wx.StaticText(tab8, -1, 'Target basedir:')
        gbsz.Add(st, wx.GBPosition(1, 0), wx.GBSpan(1, 1), LRT | wx.GROW, GAP)  
        self.ctrltmbd = wx.TextCtrl(tab8, -1, self.config['Misc']['tmusicbase'], style = wx.TE_READONLY)
        self.ctrltmbd.SetBackgroundColour(self.background_color)
        gbsz.Add(self.ctrltmbd, wx.GBPosition(1, 1), wx.GBSpan(1, 3), LRT | wx.GROW, GAP) 
        self.btnbase = wx.Button(tab8, self.ID_BTN_BASE, 'Browse')
        self.Bind(wx.EVT_BUTTON, self.onMusicBase, id = self.ID_BTN_BASE)
        gbsz.Add(self.btnbase, wx.GBPosition(1, 4), wx.GBSpan(1, 1), LRT | wx.GROW, GAP) 
        
        st = wx.StaticText(tab8, -1, 'Folder range:')
        gbsz.Add(st, wx.GBPosition(2, 0), wx.GBSpan(1, 1), LRT | wx.GROW, GAP)        
        self.scmp1 = wx.SpinCtrl(tab8, -1)
        self.scmp1.SetRange(1000, 8999)           
        self.scmp1.SetValue(int(self.config['Misc']['rangelow']))
        self.scmp1.SetToolTip(wx.ToolTip('>= 1000'))
        self.Bind(wx.EVT_SPINCTRL, self.onLow, self.scmp1)
        gbsz.Add(self.scmp1, wx.GBPosition(2, 1), wx.GBSpan(1, 1), wx.ALL, GAP)
        st = wx.StaticText(tab8, -1, ' to (including) ')
        gbsz.Add(st, wx.GBPosition(2, 2), wx.GBSpan(1, 1), LRT, GAP)
        self.scmp2 = wx.SpinCtrl(tab8, -1)
        self.scmp2.SetRange(1000, 8999)           
        self.scmp2.SetValue(int(self.config['Misc']['rangehigh']))
        self.scmp2.SetToolTip(wx.ToolTip('<= 8999'))
        self.Bind(wx.EVT_SPINCTRL, self.onHigh, self.scmp2)
        gbsz.Add(self.scmp2, wx.GBPosition(2, 3), wx.GBSpan(1, 1), LRT, GAP)

        tab8sz.Add(gbsz, 0, wx.GROW | LRT, BORDER)    
        tab8.SetSizer(tab8sz)
        self.nb.AddPage(tab8, 'Pair devices')
    #-------------------------------------------------------------------------------------------

    def getDuration(self, start = False):
        if start:
            self.starttime = datetime.datetime.now().replace(microsecond=0)
        else:
            return datetime.datetime.now().replace(microsecond=0) - self.starttime
    #-------------------------------------------------------------------------------------------
    
    def logText(self, text, color = None):
        if color == None: color = BLACK
        if color != BLACK:
            try:
                logtext = text.strip('\n')
                self.synlogger.info(logtext)
            except:
                pass
        start = self.tc.GetInsertionPoint()
        if not self.startOL: self.tc.WriteText('\n')
        self.tc.WriteText(text)
        stop = self.tc.GetLastPosition()
        self.tc.SetStyle(start, stop, wx.TextAttr(color))
        wx.SafeYield(self)
        self.tc.ShowPosition(stop)
        self.dotcount = 0
        self.startOL = True
    #-------------------------------------------------------------------------------------------

    def progressLog(self):
        self.dotcount += 1
        if self.dotcount % 32 == 0:
            if self.dotcount % 1600 == 0:
                self.tc.WriteText("\n. ")
                self.dotcount = 0
                self.startOL = True
            else:
                self.tc.WriteText(". ")
                self.startOL = False
            wx.SafeYield(self)
    #-------------------------------------------------------------------------------------------

    def onCheck99xx(self, event):
        self.cbp.Enable(self.cb9.GetValue())
    #-------------------------------------------------------------------------------------------

    def onCheckMeta(self, event):
        self.cbo.Enable(self.cbm.GetValue())
    #-------------------------------------------------------------------------------------------

    def onHigh(self, event = None):
        if event == None:
            value = 1000
        else:
            value = event.GetInt()
        self.scma1.SetRange(1000, value)
        self.scmd1.SetRange(1000, value)
        self.scme1.SetRange(1000, value)
        self.sctht1.SetRange(1000, value)
        self.scmp1.SetRange(1000, value)
        self.scma2.SetValue(value)
        self.scmd2.SetValue(value)
        self.scme2.SetValue(value)
        self.sctht2.SetValue(value)
        self.scmp2.SetValue(value)       
    #-------------------------------------------------------------------------------------------

    def onLow(self, event= None):
        if event == None:
            value = 8999
        else:
            value = event.GetInt()
        self.scma2.SetRange(value, 9900)
        self.scmd2.SetRange(value, 9900)
        self.scme2.SetRange(value, 9900)
        self.sctht2.SetRange(value, 9900)
        self.scmp2.SetRange(value, 9900)
        self.scma1.SetValue(value)
        self.scmd1.SetValue(value)
        self.scme1.SetValue(value)
        self.sctht1.SetValue(value)
        self.scmp1.SetValue(value) 
    #-------------------------------------------------------------------------------------------

    def printAnalyze(self, dirnum = 0):
        if dirnum:
            self.dirnumlist.append(dirnum)
        if len(self.dirnumlist) == 10 or (dirnum == 0 and len(self.dirnumlist) > 0):
            self.logText("\nanalyzing dir ")
            self.logText(', '.join(self.dirnumlist))
            self.dirnumlist = []          
    #-------------------------------------------------------------------------------------------

    def analyzeDir(self, dirnum):
        try:
            dirlist = os.listdir(os.path.join(self.basedir, dirnum[:2], dirnum))
        except:
            return True, [], [], []
            
        dirlist.sort()
        dirids = []
        for item in dirlist:
            head, tail = os.path.splitext(item)
            dirids.append(head)
        alldirids = dirids[:]
        unknownids = []
        doubles = []
        
        while self.recnum < self.matchcount:
            title_id = self.records[self.recnum][0]
            if title_id % 100 == 0:
                return False, dirids, unknownids, doubles
            titleidstr = str(title_id)
            if titleidstr in dirids:
                dirids.remove(titleidstr)
            else:
                unknownids.append(titleidstr)
            self.recnum += 1
            self.revcount += 1

        for dirid in dirids:
            if alldirids.count(dirid) > 1:
                dirids.remove(dirid)
                doubles.append(dirid)
                
        return False, dirids, unknownids, doubles     
    #-------------------------------------------------------------------------------------------

    def checkMusicDevice(self):        
        # test if harddisk is connected
        if not os.path.isdir(self.basedir):
            dlg = wx.MessageDialog(self, "music basedir\n'%s'\ncurrently not reacheable!" % (self.basedir), TITLE + ' - snchronize', wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()                                       
            return
        self.logText('starting at %s' % (time.strftime('%Y-%m-%d %H:%M:%S')), BLACK)
        self.getDuration(start = True) 
        #sql = 'SELECT Id_Title FROM muma_title WHERE Id_Title >= "250000" AND Id_Title < "250100"'
        sql = 'SELECT Id_Title FROM muma_title WHERE Id_Title >= "%i00" AND Id_Title <= "%i99"' % (self.scmd1.GetValue(), self.scmd2.GetValue())
        self.revcount = 0
        removelist = []
        self.records, self.matchcount = self.database.getMatches(sql)
        if self.matchcount:
            self.recnum = 0
            self.printAnalyze()
            while self.recnum < self.matchcount:
                if self.stopsync: return  
                title_id = self.records[self.recnum][0]
                self.recnum += 1
                if title_id % 100 == 0:     # this is a dir
                    dirnum = str(title_id)[0:4]
                    self.printAnalyze(dirnum)
                    direrror, dirids, unknownids, doubles = self.analyzeDir(dirnum)
                    if direrror:
                        self.printAnalyze()
                        self.logText("\nDir %s does not exist on disk" % (dirnum), LBLUE)
                        continue    
                    if len(dirids):
                        self.printAnalyze()
                        self.logText("\nTitles in dir %s on disk but not in db: %s" % (dirnum, ', '.join(dirids)), RED)
                        removelist += dirids                            
                    if len(unknownids):
                        self.printAnalyze()
                        self.logText("\nTitles in db but not in dir %s on disk: %s" % (dirnum, ', '.join(unknownids)), BLUE)
                    if len(doubles):
                        self.printAnalyze()
                        self.logText("\nTitles in dir %s on disk with different extensions: %s" % (dirnum, ', '.join(doubles)), MAGENTA)                        
            
            if self.stopsync: return        
            if len(removelist):
                dlg = wx.MessageDialog(self, "There are %s titles on device (disk) but not in db.\nDelete these title-files now?" % (len(removelist)), TITLE + ' - delete title-files', wx.YES | wx.NO | wx.NO_DEFAULT | wx.ICON_INFORMATION)                    
                if dlg.ShowModal() == wx.ID_YES:
                    errcount = 0
                    for titleid in removelist:
                        found, pathname = findTitleExt(self.config, titleid)
                        try:
                            os.remove(pathname)
                        except:
                            errcount += 1
                    if errcount == 0:
                        self.logText("\n>>> listed titles removed from device", RED)
                    else:
                        self.logText("\n>>> one or more listed titles COULD NOT be removed from device", RED)
        self.logText('\ntotal of %s title files reviewed\n' % (self.revcount), BLACK)
        self.logText('done at %s, execution time %s\n' % (time.strftime('%Y-%m-%d %H:%M:%S'), self.getDuration()), BLACK)
    #-------------------------------------------------------------------------------------------

    def ext2Lowercase(self):        
        # test if harddisk is connected
        if not os.path.isdir(self.basedir):
            dlg = wx.MessageDialog(self, "music basedir\n'%s'\ncurrently not reacheable!" % (self.basedir), TITLE + ' - snchronize', wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()                                       
            return
        self.logText('starting at %s\n' % (time.strftime('%Y-%m-%d %H:%M:%S')), BLACK)
        self.getDuration(start = True)  
        self.logText('changing file extensions to lowercase ...\n')

        wx.SafeYield(self)
        sql = 'SELECT Id_Title FROM muma_title WHERE Id_Title >= "%i01" AND Id_Title <= "%i99"' % (self.scme1.GetValue(), self.scme2.GetValue())
        records, matchcount = self.database.getMatches(sql)
        for record in records:
            if self.stopsync: return
            self.progressLog()
            id_title = str(record[0])
            if not id_title.endswith('00'):                
                found, pathname = findTitleExt(self.config, id_title)
                if found:
                    ext = pathname[-3:]
                    lowerext = ext.lower()                
                    if lowerext in ['mp3', 'ogg', 'wav', 'wma']:
                        newpathname = pathname[:-3] + lowerext
                        try:
                            os.rename(pathname, newpathname)
                        except:
                            pass

        self.logText('done at %s, execution time %s\n' % (time.strftime('%Y-%m-%d %H:%M:%S'), self.getDuration()), BLACK)
    #-------------------------------------------------------------------------------------------

    def check99xxxxTitles(self):
        self.logText('starting at %s\n' % (time.strftime('%Y-%m-%d %H:%M:%S')), BLACK)
        self.getDuration(start = True)  
        self.logText('searching for 99xxxx titles - this can take some time ...\n')
        wx.SafeYield(self)
        self.patchlist = []
        sql = 'SELECT Id_Title, Title, Interpreter FROM muma_title WHERE Id_Title >= "990000"'
        # sql = 'SELECT Id_Title, Title, Interpreter FROM muma_title WHERE Id_Title >= "990000" AND Id_Title < "990010"'
        records99, matchcount = self.database.getMatches(sql)
        if matchcount:
            for record in records99:
                if self.stopsync: return
                self.progressLog()
                query = 'SELECT Id_Title, Title, Interpreter FROM muma_title WHERE Id_Title < 990000 AND Title LIKE "%s" AND Interpreter LIKE "%s"' % (record[1], record[2])
                matches, matchcount = self.database.getMatches(query)
                if matchcount:
                    match = matches[0]
                    patchstr = "%s may be found as %s, %s, %s" % (record[0], match[0], match[1], match[2])
                    self.patchlist.append(patchstr)
                    self.logText("%s\n" % patchstr, GREEN)
        self.logText('99xxxx test done at %s, execution time %s\n' % (time.strftime('%Y-%m-%d %H:%M:%S'), self.getDuration()), BLACK)

        if self.cbp.IsChecked() and len(self.patchlist):
            patchdlg = PatchDialog(self, self.writeToLogwin, self.makeDbBackup, self.database, self.patchlist)
            patchdlg.ShowModal()
    #-------------------------------------------------------------------------------------------

    def findDuplicates(self):
        self.logText('starting at %s\n' % (time.strftime('%Y-%m-%d %H:%M:%S')), BLACK)
        self.getDuration(start = True) 
        self.logText('searching for duplicates - this can take a loooooong time ...\n')
        wx.SafeYield(self)   
        sql = 'SELECT Id_Title, Title, Interpreter FROM muma_title WHERE Id_Title < 990000'
        #sql = 'SELECT Id_Title, Title, Interpreter FROM muma_title WHERE Id_Title > 200000 AND Id_Title < 300000'
        records, matchcount = self.database.getMatches(sql)
        for record in records:
            if self.stopsync: return
            curid = record[0]
            if curid >= 600000 and curid < 700000 or \
               curid % 100 == 0 or record[1].startswith('---') \
                                or record[1].startswith('+++'): continue
            self.progressLog()
            query = 'SELECT Id_Title, Title, Interpreter FROM muma_title WHERE Id_Title > "%d" AND Id_Title < "990000" AND Title LIKE "%s" AND Interpreter LIKE "%s"' % (curid, record[1], record[2])
            duplicates, matchcount = self.database.getMatches(query)
            if matchcount:
                match = duplicates[0]
                if match[0] < 600000 or match[0] >= 700000:
                    self.logText("%s may be duplicated as %s, %s, %s\n" % (record[0], match[0], match[1], match[2]), MAGENTA)
        self.logText('duplicates test done at %s, execution time %s\n' % (time.strftime('%Y-%m-%d %H:%M:%S'), self.getDuration()), BLACK)
    #-------------------------------------------------------------------------------------------

    def findSimTitles(self):
        self.logText('starting at %s\n' % (time.strftime('%Y-%m-%d %H:%M:%S')), BLACK)
        self.getDuration(start = True)  
        self.logText('searching for title similarities - this very hard work can take a loooooong time ...\n')
        wx.SafeYield(self)
        threshold = self.sctht.GetValue() / 100.0
        if self.cbt.IsChecked():                           
            sql = 'SELECT Id_Title, Title, Interpreter FROM muma_title WHERE Id_Title >= "%i00" AND Id_Title <= "%i99"' % (self.scthi1.GetValue(), self.sctht2.GetValue())
            records, matchcount = self.database.getMatches(sql)
            for record in records:
                if self.stopsync: return
                curid = record[0]
                if curid >= 600000 and curid < 700000 or \
                   curid % 100 == 0 or record[1].startswith('---') \
                                    or record[1].startswith('+++'): continue
                self.progressLog()
                query = 'SELECT Id_Title, Title, Interpreter FROM muma_title WHERE Id_Title > "%d" AND Id_Title <= "%i99"' % (curid, self.sctht2.GetValue())
                results, matchcount = self.database.getMatches(query)
                for result in results:
                    if self.stopsync: return
                    simid = result[0]
                    if simid < 600000 or simid >= 700000:
                        if (SequenceMatcher(None, record[1], result[1]).quick_ratio() > threshold) and \
                           (SequenceMatcher(record[2], result[2]).quick_ratio() > threshold):
                            self.logText("%s, %s, %s ~~~ %s, %s, %s\n" % (record[0], record[1], record[2],result[0], result[1], result[2]), MAGENTA)
        else:
            sql = 'SELECT Id_Title, Title FROM muma_title WHERE Id_Title >= "%i00" AND Id_Title <= "%i99"' % (self.scthi1.GetValue(), self.sctht2.GetValue())
            records, matchcount = self.database.getMatches(sql)
            for record in records:
                if self.stopsync: return
                curid = record[0]
                if curid >= 600000 and curid < 700000 or \
                   curid % 100 == 0 or record[1].startswith('---') \
                                    or record[1].startswith('+++'): continue
                self.progressLog()
                query = 'SELECT Id_Title, Title FROM muma_title WHERE Id_Title > "%d" AND Id_Title <= "%i99"' % (curid, self.sctht2.GetValue())
                results, matchcount = self.database.getMatches(query)
                for result in results:
                    if self.stopsync: return
                    simid = result[0]
                    if simid < 600000 or simid >= 700000:
                        if SequenceMatcher(None, record[1], result[1]).quick_ratio() > threshold:
                            self.logText("%s, %s ~~~ %s, %s\n" % (record[0], record[1],result[0], result[1]), MAGENTA)   
        self.logText('title similarities test done at %s, execution time %s\n' % (time.strftime('%Y-%m-%d %H:%M:%S'), self.getDuration()), BLACK)
    #-------------------------------------------------------------------------------------------

    def findSimInterp(self):
        self.logText('starting at %s\n' % (time.strftime('%Y-%m-%d %H:%M:%S')), BLACK)
        self.getDuration(start = True)  
        self.logText('searching for interpreter similarities - this very hard work can take a loooooong time ...\n')
        wx.SafeYield(self)
        threshold = self.scthi.GetValue() / 100.0

        sql = 'SELECT DISTINCT Interpreter, Id_Title FROM muma_title WHERE Interpreter >= "%s" AND Id_Title >= "100000" AND Id_Title <= "899999" ORDER BY Interpreter' % (self.ctrlsseq.GetValue())
        records, matchcount = self.database.getMatches(sql)
        for index, record in enumerate(records):
            curid = record[1]
            if curid >= 600000 and curid < 700000 or curid % 100 == 0 \
                                                  or record[0] == '' \
                                                  or record[0].startswith('---') \
                                                  or record[0].startswith('+++'): continue
            self.progressLog()
            if self.stopsync: return

            curindex = index + 1
            while curindex < matchcount:
                currecord = records[curindex]
                if (currecord[1] < 600000 or currecord[1] >= 700000) and (record[0] != currecord[0]) \
                   and SequenceMatcher(None, record[0], currecord[0]).quick_ratio() > threshold:
                   self.logText("%s, %s ~~~ %s, %s\n" % (record[1], record[0], currecord[1], currecord[0]), MAGENTA)
                curindex += 1
        self.logText('interpreter similarities test done at %s, execution time %s\n' % (time.strftime('%Y-%m-%d %H:%M:%S'), self.getDuration()), BLACK)
    #-------------------------------------------------------------------------------------------

    def getTagInfo(self, titleid):
        self.ismp3 = False
        self.meta = None
        patch = False
        found, self.path = findTitleExt(self.config, titleid)
        if found:
            if self.path.endswith('mp3') or self.path.endswith('MP3'):
                try:
                    self.meta = MP3(self.path, ID3 = EasyID3)
                    self.meta.add_tags(ID3=EasyID3)
                except: # mutagen.id3.error:
                    pass
                self.ismp3 = True
            elif self.path.endswith('ogg') or self.path.endswith('OGG'):
                try:
                    self.meta = OggVorbis(self.path)
                except:
                    pass
            
            #try:                                   # (Python 2)
            #    if self.meta != None and self.cbo.IsChecked() or \
            #       not (self.meta.has_key('title') or self.meta.has_key('artist')):
            #        patch = True
            #except:
            #    pass
            try:
                if self.meta != None and self.cbo.IsChecked() or \
                   not ('title' in self.meta or 'artist' in self.meta):
                    patch = True
            except:
                pass
            
        return patch   
    #-------------------------------------------------------------------------------------------
        
    def copyMetaData(self):
        if self.config['Misc']['mutagenpresent'] == 'False':
            dlg = wx.MessageDialog(self, "Function disabled - module 'mutagen' not present", TITLE + ' - copy meta data', wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()
            return
        self.logText('starting at %s\n' % (time.strftime('%Y-%m-%d %H:%M:%S')), BLACK)
        self.getDuration(start = True) 
        self.logText('copying meta data - this can take a loooooong time ...\n')
        wx.SafeYield(self)
        patchcount = 0
        filecount = 0
        #sql = 'SELECT Id_Title, Title, Interpreter FROM muma_title WHERE Id_Title > 400507 AND Id_Title < 400510'
        sql = 'SELECT Id_Title, Title, Interpreter FROM muma_title WHERE Id_Title >= "%i01" AND Id_Title <= "%i99"' % (self.scma1.GetValue(), self.scma2.GetValue())
        records, matchcount = self.database.getMatches(sql)
        for record in records:
            if self.stopsync: return
            self.progressLog()
            id_title = str(record[0])
            if not id_title.endswith('00') and self.getTagInfo(id_title):
                filecount += 1
                #if self.meta.has_key('title') and self.meta['title'][0] == record[1] and \  # (Python 2)
                #   self.meta.has_key('artist') and self.meta['artist'][0] == record[2]:
                #    continue
                if 'title' in self.meta and self.meta['title'][0] == record[1] and \
                   'artist' in self.meta and self.meta['artist'][0] == record[2]:
                    continue                
                self.meta['title'] = [record[1]] 
                self.meta['artist'] = [record[2]]                
                try:
                    self.meta.save()
                    self.logText("patching meta tags of file '%s'\n" % (self.path), BLUE)
                    patchcount += 1
                except:
                    pass
        self.logText('%i files reviewed, %i files patched (meta data written)\n' % (filecount, patchcount), BLACK)         
        self.logText('copying meta data done at %s, execution time %s\n' % (time.strftime('%Y-%m-%d %H:%M:%S'), self.getDuration()), BLACK)
    #-------------------------------------------------------------------------------------------

    def onMusicBase(self, event):
        dlg = mumadir.DirDialog(self, TITLE + ' - select target music basepath', self.config, 
                                self.config['Misc']['tmusicbase'], mode = 'opend')
        if dlg.ShowModal() == wx.ID_OPEN:
            path = os.path.normpath(dlg.GetPath())
            self.config['Misc']['tmusicbase'] = path
            self.tbasedir = path
            self.ctrltmbd.SetValue(path)
        dlg.Destroy()
    #-------------------------------------------------------------------------------------------
    
    def pairDevices(self):
        # test if harddisk is connected
        if not os.path.isdir(self.basedir):
            dlg = wx.MessageDialog(self, "music basedir\n'%s'\ncurrently not reacheable!" % (self.basedir), TITLE + ' - snchronize', wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()                                       
            return
        if not os.path.isdir(self.tbasedir):
            dlg = wx.MessageDialog(self, "target basedir\n'%s'\ncurrently not reacheable!" % (self.basedir), TITLE + ' - snchronize', wx.OK | wx.ICON_ERROR)
            dlg.ShowModal()                                       
            return
               
        self.logText('starting at %s\n' % (time.strftime('%Y-%m-%d %H:%M:%S')), BLACK)
        self.logText('pairing target device with master device ...\n')
        self.getDuration(start = True)
        
        wx.SafeYield(self)
        errors = 0
        copied = 0
        basedirlen = len(self.basedir)
        sql = 'SELECT Id_Title FROM muma_title WHERE Id_Title >= "%i01" AND Id_Title <= "%i99"' % (self.scme1.GetValue(), self.scme2.GetValue())
        records, matchcount = self.database.getMatches(sql)
        for record in records:
            if self.stopsync: return
            self.progressLog()
            id_title = str(record[0])
            if not id_title.endswith('00'):                
                found, pathname = findTitleExt(self.config, id_title)                
                if found:
                    targetfile = os.path.join(self.tbasedir, pathname[basedirlen + 1:])
                    dirpart = targetfile[:-10]
                    try:
                        if not os.path.exists(dirpart):
                            os.makedirs(dirpart)
                        if not os.path.exists(targetfile) or \
                                       os.path.getsize(pathname) != os.path.getsize(targetfile):
                            shutil.copy2(pathname, targetfile)
                            copied += 1
                    except:
                        errors += 1

        self.logText('%d files checked, %d files copied, %d errors\n' % (matchcount, copied, errors), BLACK)
        self.logText('done at %s, execution time %s\n' % (time.strftime('%Y-%m-%d %H:%M:%S'), self.getDuration()), BLACK)
    #-------------------------------------------------------------------------------------------
  
    def onButtonDefault (self, event):
        self.onHigh()
        self.onLow()
    #-------------------------------------------------------------------------------------------
    
    def onButtonStart(self, event):
        self.config['Misc']['rangelow'] = str(self.scmd1.GetValue())
        self.config['Misc']['rangehigh'] = str(self.scmd2.GetValue())
        self.config['Misc']['thresht'] = str(self.sctht.GetValue())
        self.config['Misc']['threshi'] = str(self.scthi.GetValue())
        self.dirnumlist = []
        self.stopsync = not self.stopsync
        if self.stopsync:
            self.logText('\nsync aborted by user at %s\n\n' % (time.strftime('%Y-%m-%d %H:%M:%S')), BLACK) 
            try:
                self.synlogger.info('\n')
            except:
                pass
            self.butstart.SetLabel('&Start sync')
            wx.SafeYield(self)
            return

        self.butstart.SetLabel('&Stop sync')
        
        self.ischecked = False
        if self.cbv.IsChecked():
            self.checkMusicDevice()
            self.cbv.SetValue(False)
            self.ischecked = True
        if self.cb9.IsChecked():
            self.check99xxxxTitles()
            self.cb9.SetValue(False)
            self.ischecked = True
        if self.cbd.IsChecked():
            self.findDuplicates()
            self.cbd.SetValue(False)
            self.ischecked = True
        if self.cbs1.IsChecked():
            self.findSimTitles()
            self.cbs1.SetValue(False)
            self.ischecked = True
        if self.cbs2.IsChecked():
            self.findSimInterp()
            self.cbs2.SetValue(False)
            self.ischecked = True
        if self.cbm.IsChecked():
            self.copyMetaData()
            self.cbm.SetValue(False)
            self.ischecked = True
        if self.cbe.IsChecked():
            self.ext2Lowercase()
            self.cbe.SetValue(False)
            self.ischecked = True
        if self.cbpd.IsChecked():
            self.pairDevices()
            self.cbpd.SetValue(False)
            self.ischecked = True                
        if not self.ischecked:
            self.logText("no tab is checked - so there is nothing to do\n")
         
        try:
            self.synfile.close()
        except:
            pass
        self.butstart.SetLabel('&Start sync')
        self.stopsync = True
        if self.closerequest:
            self.windowPosition()
            self.Destroy()
    #-------------------------------------------------------------------------------------------

    def windowPosition(self):
        self.config['Misc']['tabselect'] = str(self.nb.GetSelection())
        windowrect = self.GetRect()
        self.config['Window']['synposx'] = windowrect[0]
        self.config['Window']['synposy'] = windowrect[1]
    #-------------------------------------------------------------------------------------------

    def onButtonClose(self, event):
        self.closerequest = True
        if self.stopsync:
            self.windowPosition()
            self.Destroy()
        else:
            self.stopsync = True
        self.synlogger.info('Sync shutdown\n')
        self.synlogger.removeHandler(self.handler)
# - class SyncDialog(wx.Dialog):----------------------------------------------------------------


